@@ -15,7 +15,8 @@ _job_quit_cb(void *data)
    ecore_main_loop_quit();
 }
 
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 {
    Ecore_Job *job1, *job2, *job3, *job_quit;
    char *str1 = "Job 1 started.";
@@ -24,8 +25,8 @@ int main(int argc, char **argv)
 
    if (!ecore_init())
      {
-	printf("ERROR: Cannot init Ecore!\n");
-	return -1;
+        printf("ERROR: Cannot init Ecore!\n");
+        return -1;
      }
 
    job1 = ecore_job_add(_job_print_cb, str1);
@@ -37,12 +38,13 @@ int main(int argc, char **argv)
 
    if (job2)
      {
-	char *str;
-	str = ecore_job_del(job2);
-	job2 = NULL;
-	printf("Deleted job 2. Its data was: \"%s\"\n", str);
+        char *str;
+        str = ecore_job_del(job2);
+        job2 = NULL;
+        printf("Deleted job 2. Its data was: \"%s\"\n", str);
      }
 
    ecore_main_loop_begin();
    ecore_shutdown();
 }
+
