@@ -80,6 +80,7 @@ static void _thread_callback(void        *data,
 static Eina_List *_thread_cb = NULL;
 static Ecore_Pipe *_thread_call = NULL;
 static Eina_Lock _thread_safety;
+static const int wakeup = 42;
 
 static int _thread_loop = 0;
 static Eina_Lock _thread_mutex;
@@ -171,12 +172,13 @@ ecore_init(void)
    _ecore_job_init();
    _ecore_time_init();
 
-   eina_lock_new(&_thread_safety);
    eina_lock_new(&_thread_mutex);
    eina_condition_new(&_thread_cond, &_thread_mutex);
    eina_lock_new(&_thread_feedback_mutex);
    eina_condition_new(&_thread_feedback_cond, &_thread_feedback_mutex);
-   _thread_call = ecore_pipe_add(_thread_callback, NULL);
+   _thread_call = _ecore_pipe_add(_thread_callback, NULL);
+   eina_lock_new(&_thread_safety);
+
    eina_lock_new(&_thread_id_lock);
 
    eina_lock_new(&_ecore_main_loop_lock);
@@ -220,13 +222,27 @@ shutdown_evil:
 EAPI int
 ecore_shutdown(void)
 {
+     Ecore_Pipe *p;
    /*
     * take a lock here because _ecore_event_shutdown() does callbacks
     */
      _ecore_lock();
+     if (_ecore_init_count <= 0)
+       {
+          ERR("Init count not greater than 0 in shutdown.");
+          _ecore_unlock();
+          return 0;
+       }
      if (--_ecore_init_count != 0)
        goto unlock;
    
+     if (_ecore_fps_debug) _ecore_fps_debug_shutdown();
+     _ecore_poller_shutdown();
+     _ecore_animator_shutdown();
+     _ecore_glib_shutdown();
+     _ecore_job_shutdown();
+     _ecore_thread_shutdown();
+
    /* this looks horrible - a hack for now, but something to note. as
     * we delete the _thread_call pipe a thread COULD be doing
     * ecore_pipe_write() or what not to it at the same time - we
@@ -236,12 +252,15 @@ ecore_shutdown(void)
     * ok - this causes other valgrind complaints regarding glib aquiring
     * locks internally. so fix bug a or bug b. let's leave the original
     * bug in then and leave this as a note for now
-     Ecore_Pipe *p;
+    */
+   /*
+    * It should be fine now as we do wait for thread to shutdown before
+    * we try to destroy the pipe.
+    */
      p = _thread_call;
      _thread_call = NULL;
-     ecore_pipe_wait(p, 1, 0.1);
-     ecore_pipe_del(p);
-    */
+     _ecore_pipe_wait(p, 1, 0.1);
+     _ecore_pipe_del(p);
      eina_lock_free(&_thread_safety);
      eina_condition_free(&_thread_cond);
      eina_lock_free(&_thread_mutex);
@@ -249,12 +268,7 @@ ecore_shutdown(void)
      eina_lock_free(&_thread_feedback_mutex);
      eina_lock_free(&_thread_id_lock);
 
-     if (_ecore_fps_debug) _ecore_fps_debug_shutdown();
-     _ecore_poller_shutdown();
-     _ecore_animator_shutdown();
-     _ecore_glib_shutdown();
-     _ecore_job_shutdown();
-     _ecore_thread_shutdown();
+
 #ifndef HAVE_EXOTIC
      _ecore_exe_shutdown();
 #endif
@@ -291,12 +305,92 @@ unlock:
      return _ecore_init_count;
 }
 
+struct _Ecore_Fork_Cb
+{
+   Ecore_Cb func;
+   void *data;
+   Eina_Bool delete_me : 1;
+};
+
+typedef struct _Ecore_Fork_Cb Ecore_Fork_Cb;
+
+static int fork_cbs_walking = 0;
+static Eina_List *fork_cbs = NULL;
+
+EAPI Eina_Bool
+ecore_fork_reset_callback_add(Ecore_Cb func, const void *data)
+{
+   Ecore_Fork_Cb *fcb;
+   
+   fcb = calloc(1, sizeof(Ecore_Fork_Cb));
+   if (!fcb) return EINA_FALSE;
+   fcb->func = func;
+   fcb->data = (void *)data;
+   fork_cbs = eina_list_append(fork_cbs, fcb);
+   return EINA_TRUE;
+}
+
+EAPI Eina_Bool
+ecore_fork_reset_callback_del(Ecore_Cb func, const void *data)
+{
+   Eina_List *l;
+   Ecore_Fork_Cb *fcb;
+
+   EINA_LIST_FOREACH(fork_cbs, l, fcb)
+     {
+        if ((fcb->func == func) && (fcb->data == data))
+          {
+             if (!fork_cbs_walking)
+               {
+                  fork_cbs = eina_list_remove_list(fork_cbs, l);
+                  free(fcb);
+               }
+             else
+               fcb->delete_me = EINA_TRUE;
+             return EINA_TRUE;
+          }
+     }
+   return EINA_FALSE;
+}
+
+EAPI void
+ecore_fork_reset(void)
+{
+   Eina_List *l, *ln;
+   Ecore_Fork_Cb *fcb;
+   
+   eina_lock_take(&_thread_safety);
+
+   ecore_pipe_del(_thread_call);
+   _thread_call = ecore_pipe_add(_thread_callback, NULL);
+   /* If there was something in the pipe, trigger a wakeup again */
+   if (_thread_cb) ecore_pipe_write(_thread_call, &wakeup, sizeof (int));
+
+   eina_lock_release(&_thread_safety);
+
+   // should this be done withing the eina lock stuff?
+   
+   fork_cbs_walking++;
+   EINA_LIST_FOREACH(fork_cbs, l, fcb)
+     {
+        fcb->func(fcb->data);
+     }
+   fork_cbs_walking--;
+   
+   EINA_LIST_FOREACH_SAFE(fork_cbs, l, ln, fcb)
+     {
+        if (fcb->delete_me)
+          {
+             fork_cbs = eina_list_remove_list(fork_cbs, l);
+             free(fcb);
+          }
+     }
+}
+
 /**
  * @}
  */
 
-static int wakeup = 42;
-
 EAPI void
 ecore_main_loop_thread_safe_call_async(Ecore_Cb callback,
                                        void    *data)
@@ -441,8 +535,8 @@ ecore_thread_main_loop_end(void)
 }
 
 EAPI void
-ecore_print_warning(const char *function,
-                    const char *sparam)
+ecore_print_warning(const char *function __UNUSED__,
+                    const char *sparam __UNUSED__)
 {
    WRN("***** Developer Warning ***** :\n"
        "\tThis program is calling:\n\n"
@@ -457,7 +551,7 @@ EAPI void
 _ecore_magic_fail(const void *d,
                   Ecore_Magic m,
                   Ecore_Magic req_m,
-                  const char *fname)
+                  const char *fname __UNUSED__)
 {
    ERR("\n"
        "*** ECORE ERROR: Ecore Magic Check Failed!!!\n"
@@ -699,10 +793,8 @@ _thread_safe_cleanup(void *data)
    eina_lock_free(&call->m);
 }
 
-static void
-_thread_callback(void        *data __UNUSED__,
-                 void        *buffer __UNUSED__,
-                 unsigned int nbyte __UNUSED__)
+void
+_ecore_main_call_flush(void)
 {
    Ecore_Safe_Call *call;
    Eina_List *callback;
@@ -752,3 +844,11 @@ _thread_callback(void        *data __UNUSED__,
      }
 }
 
+static void
+_thread_callback(void        *data __UNUSED__,
+                 void        *buffer __UNUSED__,
+                 unsigned int nbyte __UNUSED__)
+{
+   _ecore_main_call_flush();
+}
+
