@@ -21,13 +21,7 @@ static int app_argc = 0;
 static char **app_argv = NULL;
 
 /**
- * @addtogroup Ecore_Group Ecore - Main Loop and Job Functions.
- *
- * @{
- */
-
-/**
- * @addtogroup Ecore_Application_Group Ecore Application functions
+ * @addtogroup Ecore_Application_Group
  *
  * @{
  */
@@ -41,8 +35,11 @@ static char **app_argv = NULL;
  * for later use by ecore_app_restart() or ecore_app_args_get().
  */
 EAPI void
-ecore_app_args_set(int argc, const char **argv)
+ecore_app_args_set(int          argc,
+                   const char **argv)
 {
+   EINA_MAIN_LOOP_CHECK_RETURN;
+
    if ((argc < 1) ||
        (!argv)) return;
    app_argc = argc;
@@ -61,8 +58,11 @@ ecore_app_args_set(int argc, const char **argv)
  * same set by ecore_app_args_set().
  */
 EAPI void
-ecore_app_args_get(int *argc, char ***argv)
+ecore_app_args_get(int    *argc,
+                   char ***argv)
 {
+   EINA_MAIN_LOOP_CHECK_RETURN;
+
    if (argc) *argc = app_argc;
    if (argv) *argv = app_argv;
 }
@@ -78,6 +78,8 @@ ecore_app_args_get(int *argc, char ***argv)
 EAPI void
 ecore_app_restart(void)
 {
+   EINA_MAIN_LOOP_CHECK_RETURN;
+#ifdef HAVE_EXECVP
    char *args[4096];
    int i;
 
@@ -86,12 +88,9 @@ ecore_app_restart(void)
    for (i = 0; i < app_argc; i++) args[i] = app_argv[i];
    args[i] = NULL;
    execvp(app_argv[0], args);
+#endif
 }
 
 /**
  * @}
  */
-
-/**
- * @}
- */
