@@ -7,19 +7,23 @@
 #include "Ecore.h"
 #include "ecore_private.h"
 
-static Eina_Bool _ecore_job_event_handler(void *data, int type, void *ev);
-static void _ecore_job_event_free(void *data, void *ev);
+static Eina_Bool _ecore_job_event_handler(void *data,
+                                          int   type,
+                                          void *ev);
+static void _ecore_job_event_free(void *data,
+                                  void *ev);
 
 static int ecore_event_job_type = 0;
-static Ecore_Event_Handler* _ecore_job_handler = NULL;
+static Ecore_Event_Handler *_ecore_job_handler = NULL;
 
 struct _Ecore_Job
 {
    ECORE_MAGIC;
-   Ecore_Event  *event;
-   Ecore_Cb func;
-   void         *data;
+   Ecore_Event *event;
+   Ecore_Cb     func;
+   void        *data;
 };
+GENERIC_ALLOC_SIZE_DECLARE(Ecore_Job);
 
 void
 _ecore_job_init(void)
@@ -31,21 +35,12 @@ _ecore_job_init(void)
 void
 _ecore_job_shutdown(void)
 {
-   ecore_event_handler_del(_ecore_job_handler);
+   _ecore_event_handler_del(_ecore_job_handler);
    _ecore_job_handler = NULL;
 }
 
 /**
- * @addtogroup Ecore_Group Ecore - Main Loop and Job Functions.
- *
- * @{
- */
-
-/**
- * @addtogroup Ecore_Job_Group Ecore Job functions
- *
- * You can queue jobs that are to be done by the main loop when the current
- * event is dealt with.
+ * @addtogroup Ecore_Job_Group
  *
  * @{
  */
@@ -60,19 +55,21 @@ _ecore_job_shutdown(void)
  * @note    Once the job has been executed, the job handle is invalid.
  */
 EAPI Ecore_Job *
-ecore_job_add(Ecore_Cb func, const void *data)
+ecore_job_add(Ecore_Cb    func,
+              const void *data)
 {
    Ecore_Job *job;
-   
+
+   EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
    if (!func) return NULL;
 
-   job = calloc(1, sizeof(Ecore_Job));
+   job = ecore_job_calloc(1);
    if (!job) return NULL;
    ECORE_MAGIC_SET(job, ECORE_MAGIC_JOB);
    job->event = ecore_event_add(ecore_event_job_type, job, _ecore_job_event_free, NULL);
    if (!job->event)
      {
-        free(job);
+        ecore_job_mp_free(job);
         return NULL;
      }
    job->func = func;
@@ -89,7 +86,8 @@ EAPI void *
 ecore_job_del(Ecore_Job *job)
 {
    void *data;
-   
+
+   EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
    if (!ECORE_MAGIC_CHECK(job, ECORE_MAGIC_JOB))
      {
         ECORE_MAGIC_FAIL(job, ECORE_MAGIC_JOB,
@@ -102,26 +100,26 @@ ecore_job_del(Ecore_Job *job)
    return data;
 }
 
-/**
- * @}
- */
-
 /**
  * @}
  */
 
 static Eina_Bool
-_ecore_job_event_handler(void *data __UNUSED__, int type __UNUSED__, void *ev)
+_ecore_job_event_handler(void *data __UNUSED__,
+                         int   type __UNUSED__,
+                         void *ev)
 {
    Ecore_Job *job;
-   
+
    job = ev;
    job->func(job->data);
    return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_ecore_job_event_free(void *data __UNUSED__, void *ev)
+_ecore_job_event_free(void *data __UNUSED__,
+                      void *job)
 {
-   free(ev);
+   ecore_job_mp_free(job);
 }
+
