@@ -65,7 +65,8 @@
 
 # define HAVE_EPOLL   0
 # define EPOLLIN      1
-# define EPOLLOUT     2
+# define EPOLLPRI     2
+# define EPOLLOUT     4
 # define EPOLLERR     8
 
 #define EPOLL_CTL_ADD 1
@@ -163,6 +164,7 @@ struct _Ecore_Fd_Handler
    Eina_Bool              write_active : 1;
    Eina_Bool              error_active : 1;
    Eina_Bool              delete_me : 1;
+   Eina_Bool              file : 1;
 #if defined(USE_G_MAIN_LOOP)
    GPollFD                gfd;
 #endif
@@ -213,6 +215,7 @@ static int do_quit = 0;
 static Ecore_Fd_Handler *fd_handlers = NULL;
 static Ecore_Fd_Handler *fd_handler_current = NULL;
 static Eina_List *fd_handlers_with_prep = NULL;
+static Eina_List *file_fd_handlers = NULL;
 static Eina_List *fd_handlers_with_buffer = NULL;
 static Eina_List *fd_handlers_to_delete = NULL;
 
@@ -259,6 +262,26 @@ static gboolean _ecore_glib_idle_enterer_called;
 static gboolean ecore_fds_ready;
 #endif
 
+Eina_Bool
+_ecore_fd_close_on_exec(int fd)
+{
+#ifdef HAVE_EXECVP
+   int flags;
+
+   flags = fcntl(fd, F_GETFD);
+   if (flags == -1)
+     return EINA_FALSE;
+
+   flags |= FD_CLOEXEC;
+   if (fcntl(fd, F_SETFD, flags) == -1)
+     return EINA_FALSE;
+   return EINA_TRUE;
+#else
+   (void) fd;
+   return EINA_FALSE;
+#endif
+}
+
 static inline void
 _ecore_fd_valid(void)
 {
@@ -276,18 +299,18 @@ static inline void
 _ecore_try_add_to_call_list(Ecore_Fd_Handler *fdh)
 {
    /* check if this fdh is already in the list */
-    if (fdh->next_ready)
-      return;
-    if (fdh->read_active || fdh->write_active || fdh->error_active)
-      {
-         /*
-          * make sure next_ready is non-null by pointing to ourselves
-          * use that to indicate this fdh is in the ready list
-          * insert at the head of the list to avoid trouble
-          */
-           fdh->next_ready = fd_handlers_to_call ? fd_handlers_to_call : fdh;
-           fd_handlers_to_call = fdh;
-      }
+   if (fdh->next_ready)
+     return;
+   if (fdh->read_active || fdh->write_active || fdh->error_active)
+     {
+        /*
+         * make sure next_ready is non-null by pointing to ourselves
+         * use that to indicate this fdh is in the ready list
+         * insert at the head of the list to avoid trouble
+         */
+        fdh->next_ready = fd_handlers_to_call ? fd_handlers_to_call : fdh;
+        fd_handlers_to_call = fdh;
+     }
 }
 
 static inline int
@@ -296,7 +319,7 @@ _ecore_get_epoll_fd(void)
    if (epoll_pid && epoll_pid != getpid())
      {
         /* forked! */
-         _ecore_main_loop_shutdown();
+        _ecore_main_loop_shutdown();
      }
    if (epoll_pid == 0 && epoll_fd < 0)
      {
@@ -326,7 +349,7 @@ _ecore_poll_events_from_fdh(Ecore_Fd_Handler *fdh)
    int events = 0;
    if (fdh->flags & ECORE_FD_READ) events |= EPOLLIN;
    if (fdh->flags & ECORE_FD_WRITE) events |= EPOLLOUT;
-   if (fdh->flags & ECORE_FD_ERROR) events |= EPOLLERR;
+   if (fdh->flags & ECORE_FD_ERROR) events |= EPOLLERR | EPOLLPRI;
    return events;
 }
 
@@ -348,7 +371,7 @@ _ecore_main_fdh_poll_add(Ecore_Fd_Handler *fdh)
 {
    int r = 0;
 
-   if (HAVE_EPOLL && epoll_fd >= 0)
+   if ((!fdh->file) && HAVE_EPOLL && epoll_fd >= 0)
      {
         r = _ecore_epoll_add(_ecore_get_epoll_fd(), fdh->fd,
                              _ecore_poll_events_from_fdh(fdh), fdh);
@@ -369,7 +392,7 @@ _ecore_main_fdh_poll_add(Ecore_Fd_Handler *fdh)
 static inline void
 _ecore_main_fdh_poll_del(Ecore_Fd_Handler *fdh)
 {
-   if (HAVE_EPOLL && epoll_fd >= 0)
+   if ((!fdh->file) && HAVE_EPOLL && epoll_fd >= 0)
      {
         struct epoll_event ev;
         int efd = _ecore_get_epoll_fd();
@@ -408,7 +431,7 @@ static inline int
 _ecore_main_fdh_poll_modify(Ecore_Fd_Handler *fdh)
 {
    int r = 0;
-   if (HAVE_EPOLL && epoll_fd >= 0)
+   if ((!fdh->file) && HAVE_EPOLL && epoll_fd >= 0)
      {
         struct epoll_event ev;
         int efd = _ecore_get_epoll_fd();
@@ -743,6 +766,7 @@ _ecore_main_loop_init(void)
    if (epoll_fd < 0)
      WRN("Failed to create epoll fd!");
    epoll_pid = getpid();
+   _ecore_fd_close_on_exec(epoll_fd);
 
    /* add polls on all our file descriptors */
    Ecore_Fd_Handler *fdh;
@@ -778,6 +802,7 @@ _ecore_main_loop_init(void)
           WRN("failed to create timer fd!");
         else
           {
+             _ecore_fd_close_on_exec(timer_fd);
              ecore_timer_fd.fd = timer_fd;
              ecore_timer_fd.events = G_IO_IN;
              ecore_timer_fd.revents = 0;
@@ -860,6 +885,7 @@ ecore_main_loop_iterate(void)
    EINA_MAIN_LOOP_CHECK_RETURN;
 #ifndef USE_G_MAIN_LOOP
    _ecore_lock();
+   _ecore_time_loop_time = ecore_time_get();
    _ecore_main_loop_iterate_internal(1);
    _ecore_unlock();
 #else
@@ -890,6 +916,7 @@ ecore_main_loop_iterate_may_block(int may_block)
    EINA_MAIN_LOOP_CHECK_RETURN_VAL(0);
 #ifndef USE_G_MAIN_LOOP
    _ecore_lock();
+   _ecore_time_loop_time = ecore_time_get();
 in_main_loop++;
    _ecore_main_loop_iterate_internal(!may_block);
 in_main_loop--;
@@ -925,6 +952,7 @@ ecore_main_loop_begin(void)
 #ifndef USE_G_MAIN_LOOP
    _ecore_lock();
    in_main_loop++;
+   _ecore_time_loop_time = ecore_time_get();
    while (do_quit == 0) _ecore_main_loop_iterate_internal(0);
    do_quit = 0;
    in_main_loop--;
@@ -990,6 +1018,48 @@ ecore_main_loop_select_func_get(void)
    return main_loop_select;
 }
 
+Ecore_Fd_Handler *
+_ecore_main_fd_handler_add(int                    fd,
+                           Ecore_Fd_Handler_Flags flags,
+                           Ecore_Fd_Cb            func,
+                           const void            *data,
+                           Ecore_Fd_Cb            buf_func,
+                           const void            *buf_data)
+{
+   Ecore_Fd_Handler *fdh = NULL;
+
+   if ((fd < 0) || (flags == 0) || (!func)) return NULL;
+
+   fdh = ecore_fd_handler_calloc(1);
+   if (!fdh) return NULL;
+   ECORE_MAGIC_SET(fdh, ECORE_MAGIC_FD_HANDLER);
+   fdh->next_ready = NULL;
+   fdh->fd = fd;
+   fdh->flags = flags;
+   if (_ecore_main_fdh_poll_add(fdh) < 0)
+     {
+        int err = errno;
+        ERR("Failed to add poll on fd %d (errno = %d: %s)!", fd, err, strerror(err));
+        ecore_fd_handler_mp_free(fdh);
+        return NULL;
+     }
+   fdh->read_active = EINA_FALSE;
+   fdh->write_active = EINA_FALSE;
+   fdh->error_active = EINA_FALSE;
+   fdh->delete_me = EINA_FALSE;
+   fdh->func = func;
+   fdh->data = (void *)data;
+   fdh->buf_func = buf_func;
+   if (buf_func)
+     fd_handlers_with_buffer = eina_list_append(fd_handlers_with_buffer, fdh);
+   fdh->buf_data = (void *)buf_data;
+   fd_handlers = (Ecore_Fd_Handler *)
+     eina_inlist_append(EINA_INLIST_GET(fd_handlers),
+                        EINA_INLIST_GET(fdh));
+
+   return fdh;
+}
+
 EAPI Ecore_Fd_Handler *
 ecore_main_fd_handler_add(int                    fd,
                           Ecore_Fd_Handler_Flags flags,
@@ -999,6 +1069,22 @@ ecore_main_fd_handler_add(int                    fd,
                           const void            *buf_data)
 {
    Ecore_Fd_Handler *fdh = NULL;
+   EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
+   _ecore_lock();
+   fdh = _ecore_main_fd_handler_add(fd, flags, func, data, buf_func, buf_data);
+   _ecore_unlock();
+   return fdh;
+}
+
+EAPI Ecore_Fd_Handler *
+ecore_main_fd_handler_file_add(int                    fd,
+                               Ecore_Fd_Handler_Flags flags,
+                               Ecore_Fd_Cb            func,
+                               const void            *data,
+                               Ecore_Fd_Cb            buf_func,
+                               const void            *buf_data)
+{
+   Ecore_Fd_Handler *fdh = NULL;
 
    EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
    _ecore_lock();
@@ -1011,6 +1097,7 @@ ecore_main_fd_handler_add(int                    fd,
    fdh->next_ready = NULL;
    fdh->fd = fd;
    fdh->flags = flags;
+   fdh->file = EINA_TRUE;
    if (_ecore_main_fdh_poll_add(fdh) < 0)
      {
         int err = errno;
@@ -1032,6 +1119,7 @@ ecore_main_fd_handler_add(int                    fd,
    fd_handlers = (Ecore_Fd_Handler *)
      eina_inlist_append(EINA_INLIST_GET(fd_handlers),
                         EINA_INLIST_GET(fdh));
+   file_fd_handlers = eina_list_append(file_fd_handlers, fdh);
 unlock:
    _ecore_unlock();
 
@@ -1220,7 +1308,7 @@ _ecore_main_shutdown(void)
    if (in_main_loop)
      {
         ERR("\n"
-            "*** ECORE WARINING: Calling ecore_shutdown() while still in the main loop.\n"
+            "*** ECORE WARNING: Calling ecore_shutdown() while still in the main loop.\n"
             "***                 Program may crash or behave strangely now.");
         return;
      }
@@ -1240,6 +1328,8 @@ _ecore_main_shutdown(void)
      fd_handlers_with_prep = eina_list_free(fd_handlers_with_prep);
    if (fd_handlers_to_delete)
      fd_handlers_to_delete = eina_list_free(fd_handlers_to_delete);
+   if (file_fd_handlers)
+     file_fd_handlers = eina_list_free(file_fd_handlers);
 
    fd_handlers_to_call = NULL;
    fd_handlers_to_call_current = NULL;
@@ -1293,6 +1383,8 @@ _ecore_main_select(double timeout)
 {
    struct timeval tv, *t;
    fd_set rfds, wfds, exfds;
+   Ecore_Fd_Handler *fdh;
+   Eina_List *l;
    int max_fd;
    int ret;
 
@@ -1330,8 +1422,6 @@ _ecore_main_select(double timeout)
 
    if (!HAVE_EPOLL || epoll_fd < 0)
      {
-        Ecore_Fd_Handler *fdh;
-
         EINA_INLIST_FOREACH(fd_handlers, fdh)
           {
              if (!fdh->delete_me)
@@ -1360,7 +1450,26 @@ _ecore_main_select(double timeout)
          max_fd = _ecore_get_epoll_fd();
          FD_SET(max_fd, &rfds);
      }
-
+   EINA_LIST_FOREACH(file_fd_handlers, l, fdh)
+     if (!fdh->delete_me)
+       {
+          if (fdh->flags & ECORE_FD_READ)
+            {
+               FD_SET(fdh->fd, &rfds);
+               if (fdh->fd > max_fd) max_fd = fdh->fd;
+            }
+          if (fdh->flags & ECORE_FD_WRITE)
+            {
+               FD_SET(fdh->fd, &wfds);
+               if (fdh->fd > max_fd) max_fd = fdh->fd;
+            }
+          if (fdh->flags & ECORE_FD_ERROR)
+            {
+               FD_SET(fdh->fd, &exfds);
+               if (fdh->fd > max_fd) max_fd = fdh->fd;
+            }
+          if (fdh->fd > max_fd) max_fd = fdh->fd;
+       }
    if (_ecore_signal_count_get()) return -1;
 
    _ecore_unlock();
@@ -1382,8 +1491,6 @@ _ecore_main_select(double timeout)
           _ecore_main_fdh_epoll_mark_active();
         else
           {
-             Ecore_Fd_Handler *fdh;
-
              EINA_INLIST_FOREACH(fd_handlers, fdh)
                {
                   if (!fdh->delete_me)
@@ -1398,7 +1505,19 @@ _ecore_main_select(double timeout)
                     }
                }
           }
-
+        EINA_LIST_FOREACH(file_fd_handlers, l, fdh)
+          {
+             if (!fdh->delete_me)
+               {
+                  if (FD_ISSET(fdh->fd, &rfds))
+                    fdh->read_active = EINA_TRUE;
+                  if (FD_ISSET(fdh->fd, &wfds))
+                    fdh->write_active = EINA_TRUE;
+                  if (FD_ISSET(fdh->fd, &exfds))
+                    fdh->error_active = EINA_TRUE;
+                  _ecore_try_add_to_call_list(fdh);
+               }
+          }
         _ecore_main_fd_handlers_cleanup();
 #ifdef _WIN32
         _ecore_main_win32_handlers_cleanup();
@@ -1495,6 +1614,8 @@ _ecore_main_fd_handlers_cleanup(void)
           fd_handlers_with_prep = eina_list_remove(fd_handlers_with_prep, fdh);
         fd_handlers = (Ecore_Fd_Handler *)
           eina_inlist_remove(EINA_INLIST_GET(fd_handlers), EINA_INLIST_GET(fdh));
+        if (fdh->file)
+          file_fd_handlers = eina_list_remove(file_fd_handlers, fdh);
         ECORE_MAGIC_SET(fdh, ECORE_MAGIC_NONE);
         ecore_fd_handler_mp_free(fdh);
         fd_handlers_to_delete = eina_list_remove_list(fd_handlers_to_delete, l);
@@ -1852,12 +1973,21 @@ _ecore_main_win32_select(int             nfds __UNUSED__,
         long network_event;
 
         network_event = 0;
-        if (FD_ISSET(fdh->fd, readfds))
-          network_event |= FD_READ;
-        if (FD_ISSET(fdh->fd, writefds))
-          network_event |= FD_WRITE;
-        if (FD_ISSET(fdh->fd, exceptfds))
-          network_event |= FD_OOB;
+        if (readfds)
+          {
+             if (FD_ISSET(fdh->fd, readfds))
+               network_event |= FD_READ;
+          }
+        if (writefds)
+          {
+             if (FD_ISSET(fdh->fd, writefds))
+               network_event |= FD_WRITE;
+          }
+        if (exceptfds)
+          {
+             if (FD_ISSET(fdh->fd, exceptfds))
+               network_event |= FD_OOB;
+          }
 
         if (network_event)
           {
@@ -1897,9 +2027,12 @@ _ecore_main_win32_select(int             nfds __UNUSED__,
    result = MsgWaitForMultipleObjects(objects_nbr, (const HANDLE *)objects, EINA_FALSE,
                                       timeout, QS_ALLINPUT);
 
-   FD_ZERO(readfds);
-   FD_ZERO(writefds);
-   FD_ZERO(exceptfds);
+   if (readfds)
+     FD_ZERO(readfds);
+   if (writefds)
+     FD_ZERO(writefds);
+   if (exceptfds)
+     FD_ZERO(exceptfds);
 
    /* The result tells us the type of event we have. */
    if (result == WAIT_FAILED)
@@ -1907,7 +2040,7 @@ _ecore_main_win32_select(int             nfds __UNUSED__,
         char *m;
 
         m = evil_last_error_get();
-        ERR(" * %s\n", m);
+        ERR("%s", m);
         free(m);
         res = -1;
      }
@@ -1932,11 +2065,11 @@ _ecore_main_win32_select(int             nfds __UNUSED__,
 
         WSAEnumNetworkEvents(sockets[result], objects[result], &network_event);
 
-        if (network_event.lNetworkEvents & FD_READ)
+        if ((network_event.lNetworkEvents & FD_READ) && readfds)
           FD_SET(sockets[result], readfds);
-        if (network_event.lNetworkEvents & FD_WRITE)
+        if ((network_event.lNetworkEvents & FD_WRITE) && writefds)
           FD_SET(sockets[result], writefds);
-        if (network_event.lNetworkEvents & FD_OOB)
+        if ((network_event.lNetworkEvents & FD_OOB) && exceptfds)
           FD_SET(sockets[result], exceptfds);
 
         res = 1;
