@@ -7,6 +7,10 @@
 #include <errno.h>
 #include <math.h>
 
+#ifdef __SUNPRO_C
+# include <ieeefp.h>
+#endif
+
 #ifdef HAVE_ISFINITE
 # define ECORE_FINITE(t)  isfinite(t)
 #else
@@ -123,6 +127,277 @@ ecore_pipe_add(Ecore_Pipe_Cb handler,
                const void   *data)
 {
    Ecore_Pipe *p;
+
+   _ecore_lock();
+   p = _ecore_pipe_add(handler, data);
+   _ecore_unlock();
+
+   return p;
+}
+
+/**
+ * Free an Ecore_Pipe object created with ecore_pipe_add().
+ *
+ * @param p The Ecore_Pipe object to be freed.
+ * @return The pointer to the private data
+ */
+EAPI void *
+ecore_pipe_del(Ecore_Pipe *p)
+{
+   void *r;
+   EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
+   _ecore_lock();
+   r = _ecore_pipe_del(p);
+   _ecore_unlock();
+   return r;
+}
+
+/**
+ * Close the read end of an Ecore_Pipe object created with ecore_pipe_add().
+ *
+ * @param p The Ecore_Pipe object.
+ */
+EAPI void
+ecore_pipe_read_close(Ecore_Pipe *p)
+{
+   EINA_MAIN_LOOP_CHECK_RETURN;
+   _ecore_lock();
+   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
+     {
+        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_close");
+        goto out;
+     }
+   if (p->fd_handler)
+     {
+        _ecore_main_fd_handler_del(p->fd_handler);
+        p->fd_handler = NULL;
+     }
+   if (p->fd_read != PIPE_FD_INVALID)
+     {
+        pipe_close(p->fd_read);
+        p->fd_read = PIPE_FD_INVALID;
+     }
+out:
+   _ecore_unlock();
+}
+
+/**
+ * Stop monitoring if necessary the pipe for reading. See ecore_pipe_thaw()
+ * for monitoring it again.
+ *
+ * @param p The Ecore_Pipe object.
+ * @since 1.1
+ */
+EAPI void
+ecore_pipe_freeze(Ecore_Pipe *p)
+{
+   EINA_MAIN_LOOP_CHECK_RETURN;
+   _ecore_lock();
+   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
+     {
+        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_freeze");
+        goto out;
+     }
+   if (p->fd_handler)
+     {
+        _ecore_main_fd_handler_del(p->fd_handler);
+        p->fd_handler = NULL;
+     }
+out:
+   _ecore_unlock();
+}
+
+/**
+ * Start monitoring again the pipe for reading. See ecore_pipe_freeze() for
+ * stopping the monitoring activity. This will not work if
+ * ecore_pipe_read_close() was previously called on the same pipe.
+ *
+ * @param p The Ecore_Pipe object.
+ * @since 1.1
+ */
+EAPI void
+ecore_pipe_thaw(Ecore_Pipe *p)
+{
+   EINA_MAIN_LOOP_CHECK_RETURN;
+   _ecore_lock();
+   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
+     {
+        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_thaw");
+        goto out;
+     }
+   if (!p->fd_handler && p->fd_read != PIPE_FD_INVALID)
+     {
+        p->fd_handler = ecore_main_fd_handler_add(p->fd_read,
+                                                  ECORE_FD_READ,
+                                                  _ecore_pipe_read,
+                                                  p,
+                                                  NULL, NULL);
+     }
+out:
+   _ecore_unlock();
+}
+
+/**
+ * @brief Wait from another thread on the read side of a pipe.
+ *
+ * @param p The pipe to watch on.
+ * @param message_count The minimal number of message to wait before exiting.
+ * @param wait The amount of time in second to wait before exiting.
+ * @return the number of message catched during that wait call.
+ * @since 1.1
+ *
+ * Negative value for @p wait means infite wait.
+ */
+EAPI int
+ecore_pipe_wait(Ecore_Pipe *p,
+                int         message_count,
+                double      wait)
+{
+   int r;
+   _ecore_lock();
+   r = _ecore_pipe_wait(p, message_count, wait);
+   _ecore_unlock();
+   return r;
+}
+
+/**
+ * Close the write end of an Ecore_Pipe object created with ecore_pipe_add().
+ *
+ * @param p The Ecore_Pipe object.
+ */
+EAPI void
+ecore_pipe_write_close(Ecore_Pipe *p)
+{
+   _ecore_lock();
+   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
+     {
+        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_write_close");
+        goto out;
+     }
+   if (p->fd_write != PIPE_FD_INVALID)
+     {
+        pipe_close(p->fd_write);
+        p->fd_write = PIPE_FD_INVALID;
+     }
+out:
+   _ecore_unlock();
+}
+
+/**
+ * Write on the file descriptor the data passed as parameter.
+ *
+ * @param p      The Ecore_Pipe object.
+ * @param buffer The data to write into the pipe.
+ * @param nbytes The size of the @p buffer in bytes
+ * @return       @c EINA_TRUE on a successful write, @c EINA_FALSE on error.
+ */
+EAPI Eina_Bool
+ecore_pipe_write(Ecore_Pipe  *p,
+                 const void  *buffer,
+                 unsigned int nbytes)
+{
+   ssize_t ret;
+   size_t already_written = 0;
+   int retry = ECORE_PIPE_WRITE_RETRY;
+   Eina_Bool ok = EINA_FALSE;
+
+   _ecore_lock();
+   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
+     {
+        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_write");
+        goto out;
+     }
+
+   if (p->delete_me) goto out;
+
+   if (p->fd_write == PIPE_FD_INVALID) goto out;
+
+   /* First write the len into the pipe */
+   do
+     {
+        ret = pipe_write(p->fd_write, &nbytes, sizeof(nbytes));
+        if (ret == sizeof(nbytes))
+          {
+             retry = ECORE_PIPE_WRITE_RETRY;
+             break;
+          }
+        else if (ret > 0)
+          {
+             /* XXX What should we do here? */
+              ERR("The length of the data was not written complete"
+                  " to the pipe");
+              goto out;
+          }
+        else if (ret == PIPE_FD_ERROR && errno == EPIPE)
+          {
+             pipe_close(p->fd_write);
+             p->fd_write = PIPE_FD_INVALID;
+             goto out;
+          }
+        else if (ret == PIPE_FD_ERROR && errno == EINTR)
+          /* try it again */
+          ;
+        else
+          {
+             ERR("An unhandled error (ret: %zd errno: %d)"
+                 "occurred while writing to the pipe the length",
+                 ret, errno);
+          }
+     }
+   while (retry--);
+
+   if (retry != ECORE_PIPE_WRITE_RETRY) goto out;
+
+   /* and now pass the data to the pipe */
+   do
+     {
+        ret = pipe_write(p->fd_write,
+                         ((unsigned char *)buffer) + already_written,
+                         nbytes - already_written);
+
+        if (ret == (ssize_t)(nbytes - already_written))
+          {
+            ok = EINA_TRUE;
+            goto out;
+          }
+        else if (ret >= 0)
+          {
+             already_written -= ret;
+             continue;
+          }
+        else if (ret == PIPE_FD_ERROR && errno == EPIPE)
+          {
+             pipe_close(p->fd_write);
+             p->fd_write = PIPE_FD_INVALID;
+             goto out;
+          }
+        else if (ret == PIPE_FD_ERROR && errno == EINTR)
+          /* try it again */
+          ;
+        else
+          {
+             ERR("An unhandled error (ret: %zd errno: %d)"
+                 "occurred while writing to the pipe the length",
+                 ret, errno);
+          }
+     }
+   while (retry--);
+
+out:
+   _ecore_unlock();
+   return ok;
+}
+
+/**
+ * @}
+ */
+
+/* Private functions */
+Ecore_Pipe *
+_ecore_pipe_add(Ecore_Pipe_Cb handler,
+                const void   *data)
+{
+   Ecore_Pipe *p = NULL;
    int fds[2];
 
    EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
@@ -149,21 +424,15 @@ ecore_pipe_add(Ecore_Pipe_Cb handler,
                                              _ecore_pipe_read,
                                              p,
                                              NULL, NULL);
+
    return p;
 }
 
-/**
- * Free an Ecore_Pipe object created with ecore_pipe_add().
- *
- * @param p The Ecore_Pipe object to be freed.
- * @return The pointer to the private data
- */
-EAPI void *
-ecore_pipe_del(Ecore_Pipe *p)
+void *
+_ecore_pipe_del(Ecore_Pipe *p)
 {
-   void *data;
+   void *data = NULL;
 
-   EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
    if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
      {
         ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_del");
@@ -179,97 +448,10 @@ ecore_pipe_del(Ecore_Pipe *p)
    return data;
 }
 
-/**
- * Close the read end of an Ecore_Pipe object created with ecore_pipe_add().
- *
- * @param p The Ecore_Pipe object.
- */
-EAPI void
-ecore_pipe_read_close(Ecore_Pipe *p)
-{
-   EINA_MAIN_LOOP_CHECK_RETURN;
-   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
-     {
-        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_close");
-        return;
-     }
-   if (p->fd_handler)
-     {
-        _ecore_main_fd_handler_del(p->fd_handler);
-        p->fd_handler = NULL;
-     }
-   if (p->fd_read != PIPE_FD_INVALID)
-     {
-        pipe_close(p->fd_read);
-        p->fd_read = PIPE_FD_INVALID;
-     }
-}
-
-/**
- * Stop monitoring if necessary the pipe for reading. See ecore_pipe_thaw()
- * for monitoring it again.
- *
- * @param p The Ecore_Pipe object.
- * @since 1.1
- */
-EAPI void
-ecore_pipe_freeze(Ecore_Pipe *p)
-{
-   EINA_MAIN_LOOP_CHECK_RETURN;
-   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
-     {
-        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_freeze");
-        return;
-     }
-   if (p->fd_handler)
-     {
-        _ecore_main_fd_handler_del(p->fd_handler);
-        p->fd_handler = NULL;
-     }
-}
-
-/**
- * Start monitoring again the pipe for reading. See ecore_pipe_freeze() for
- * stopping the monitoring activity. This will not work if
- * ecore_pipe_read_close() was previously called on the same pipe.
- *
- * @param p The Ecore_Pipe object.
- * @since 1.1
- */
-EAPI void
-ecore_pipe_thaw(Ecore_Pipe *p)
-{
-   EINA_MAIN_LOOP_CHECK_RETURN;
-   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
-     {
-        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_read_thaw");
-        return;
-     }
-   if (!p->fd_handler && p->fd_read != PIPE_FD_INVALID)
-     {
-        p->fd_handler = ecore_main_fd_handler_add(p->fd_read,
-                                                  ECORE_FD_READ,
-                                                  _ecore_pipe_read,
-                                                  p,
-                                                  NULL, NULL);
-     }
-}
-
-/**
- * @brief Wait from another thread on the read side of a pipe.
- *
- * @param p The pipe to watch on.
- * @param message_count The minimal number of message to wait before exiting.
- * @param wait The amount of time in second to wait before exiting.
- * @return the number of message catched during that wait call.
- * @since 1.1
- *
- * Negative value for @p wait means infite wait.
- */
-EAPI int
-ecore_pipe_wait(Ecore_Pipe *p,
-                int         message_count,
-                double      wait)
+int
+_ecore_pipe_wait(Ecore_Pipe *p,
+                 int         message_count,
+                 double      wait)
 {
    struct timeval tv, *t;
    fd_set rset;
@@ -347,136 +529,27 @@ ecore_pipe_wait(Ecore_Pipe *p,
    return total;
 }
 
-/**
- * Close the write end of an Ecore_Pipe object created with ecore_pipe_add().
- *
- * @param p The Ecore_Pipe object.
- */
-EAPI void
-ecore_pipe_write_close(Ecore_Pipe *p)
-{
-   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
-     {
-        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_write_close");
-        return;
-     }
-   if (p->fd_write != PIPE_FD_INVALID)
-     {
-        pipe_close(p->fd_write);
-        p->fd_write = PIPE_FD_INVALID;
-     }
-}
-
-/**
- * Write on the file descriptor the data passed as parameter.
- *
- * @param p      The Ecore_Pipe object.
- * @param buffer The data to write into the pipe.
- * @param nbytes The size of the @p buffer in bytes
- * @return       @c EINA_TRUE on a successful write, @c EINA_FALSE on error.
- */
-EAPI Eina_Bool
-ecore_pipe_write(Ecore_Pipe  *p,
-                 const void  *buffer,
-                 unsigned int nbytes)
-{
-   ssize_t ret;
-   size_t already_written = 0;
-   int retry = ECORE_PIPE_WRITE_RETRY;
-
-   if (!ECORE_MAGIC_CHECK(p, ECORE_MAGIC_PIPE))
-     {
-        ECORE_MAGIC_FAIL(p, ECORE_MAGIC_PIPE, "ecore_pipe_write");
-        return EINA_FALSE;
-     }
-
-   if (p->delete_me) return EINA_FALSE;
-
-   if (p->fd_write == PIPE_FD_INVALID) return EINA_FALSE;
-
-   /* First write the len into the pipe */
-   do
-     {
-        ret = pipe_write(p->fd_write, &nbytes, sizeof(nbytes));
-        if (ret == sizeof(nbytes))
-          {
-             retry = ECORE_PIPE_WRITE_RETRY;
-             break;
-          }
-        else if (ret > 0)
-          {
-             /* XXX What should we do here? */
-              ERR("The length of the data was not written complete"
-                  " to the pipe");
-              return EINA_FALSE;
-          }
-        else if (ret == PIPE_FD_ERROR && errno == EPIPE)
-          {
-             pipe_close(p->fd_write);
-             p->fd_write = PIPE_FD_INVALID;
-             return EINA_FALSE;
-          }
-        else if (ret == PIPE_FD_ERROR && errno == EINTR)
-          /* try it again */
-          ;
-        else
-          {
-             ERR("An unhandled error (ret: %zd errno: %d)"
-                 "occurred while writing to the pipe the length",
-                 ret, errno);
-          }
-     }
-   while (retry--);
-
-   if (retry != ECORE_PIPE_WRITE_RETRY) return EINA_FALSE;
-
-   /* and now pass the data to the pipe */
-   do
-     {
-        ret = pipe_write(p->fd_write,
-                         ((unsigned char *)buffer) + already_written,
-                         nbytes - already_written);
-
-        if (ret == (ssize_t)(nbytes - already_written))
-          return EINA_TRUE;
-        else if (ret >= 0)
-          {
-             already_written -= ret;
-             continue;
-          }
-        else if (ret == PIPE_FD_ERROR && errno == EPIPE)
-          {
-             pipe_close(p->fd_write);
-             p->fd_write = PIPE_FD_INVALID;
-             return EINA_FALSE;
-          }
-        else if (ret == PIPE_FD_ERROR && errno == EINTR)
-          /* try it again */
-          ;
-        else
-          {
-             ERR("An unhandled error (ret: %zd errno: %d)"
-                 "occurred while writing to the pipe the length",
-                 ret, errno);
-          }
-     }
-   while (retry--);
-
-   return EINA_FALSE;
-}
-
-/**
- * @}
- */
-
-/* Private function */
 static void
 _ecore_pipe_unhandle(Ecore_Pipe *p)
 {
    p->handling--;
    if (p->delete_me)
      {
-        ecore_pipe_del(p);
+        _ecore_pipe_del(p);
+     }
+}
+
+static void
+_ecore_pipe_handler_call(Ecore_Pipe *p,
+                         unsigned char *buf,
+                         size_t len)
+{
+   void *data = (void*) p->data;
+   if (!p->delete_me)
+     {
+        _ecore_unlock();
+        p->handler(data, buf, len);
+        _ecore_lock();
      }
 }
 
@@ -519,8 +592,7 @@ _ecore_pipe_read(void             *data,
                     if (i == 0)
                       {
      /* no data on first try through means an error */
-                          if (!p->delete_me)
-                            p->handler((void *)p->data, NULL, 0);
+                          _ecore_pipe_handler_call(p, NULL, 0);
                           if (p->passed_data) free(p->passed_data);
                           p->passed_data = NULL;
                           p->already_read = 0;
@@ -557,8 +629,7 @@ _ecore_pipe_read(void             *data,
                 {
                    if (WSAGetLastError() != WSAEWOULDBLOCK)
                      {
-                        if (!p->delete_me)
-                          p->handler((void *)p->data, NULL, 0);
+                        _ecore_pipe_handler_call(p, NULL, 0);
                         if (p->passed_data) free(p->passed_data);
                         p->passed_data = NULL;
                         p->already_read = 0;
@@ -579,8 +650,7 @@ _ecore_pipe_read(void             *data,
          * never happen */
         if (p->len == 0)
           {
-             if (!p->delete_me)
-               p->handler((void *)p->data, NULL, 0);
+             _ecore_pipe_handler_call(p, NULL, 0);
              /* reset all values to 0 */
              if (p->passed_data) free(p->passed_data);
              p->passed_data = NULL;
@@ -598,8 +668,7 @@ _ecore_pipe_read(void             *data,
              /* alloc failed - error case */
              if (!p->passed_data)
                {
-                  if (!p->delete_me)
-                    p->handler((void *)p->data, NULL, 0);
+                  _ecore_pipe_handler_call(p, NULL, 0);
      /* close the pipe */
                   p->already_read = 0;
                   p->len = 0;
@@ -621,8 +690,7 @@ _ecore_pipe_read(void             *data,
         /* if we read enough data to finish the message/buffer */
         if (ret == (ssize_t)(p->len - p->already_read))
           {
-             if (!p->delete_me)
-               p->handler((void *)p->data, p->passed_data, p->len);
+             _ecore_pipe_handler_call(p, p->passed_data, p->len);
              free(p->passed_data);
              /* reset all values to 0 */
              p->passed_data = NULL;
@@ -663,8 +731,7 @@ _ecore_pipe_read(void             *data,
           {
              if (WSAGetLastError() != WSAEWOULDBLOCK)
                {
-                  if (!p->delete_me)
-                    p->handler((void *)p->data, NULL, 0);
+                  _ecore_pipe_handler_call(p, NULL, 0);
                   if (p->passed_data) free(p->passed_data);
                   p->passed_data = NULL;
                   p->already_read = 0;
