@@ -1,3 +1,4 @@
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -65,35 +66,62 @@ typedef struct
    void  *val;
 } win32_thread;
 
+static Eina_List    *_ecore_thread_win32_threads = NULL;
+static Eina_Lock     _ecore_thread_win32_lock;
+
 #  define PH(x)     win32_thread * x
 #  define PHE(x, y) ((x) == (y))
-#  define PHS()     (HANDLE)GetCurrentThreadId()
 
-int
+static win32_thread *
+_ecore_thread_win32_self()
+{
+   win32_thread *t;
+   Eina_List *l;
+
+   LKL(_ecore_thread_win32_lock);
+   EINA_LIST_FOREACH(_ecore_thread_win32_threads, l, t)
+     if (t->thread == GetCurrentThread())
+       {
+          LKU(_ecore_thread_win32_lock);
+          return t;
+       }
+
+   LKU(_ecore_thread_win32_lock);
+   return NULL;
+}
+
+#  define PHS()     _ecore_thread_win32_self()
+
+static int
 _ecore_thread_win32_create(win32_thread         **x,
                            LPTHREAD_START_ROUTINE f,
                            void                  *d)
 {
    win32_thread *t;
+
    t = (win32_thread *)calloc(1, sizeof(win32_thread));
    if (!t)
      return -1;
 
+   LKL(_ecore_thread_win32_lock);
    (t)->thread = CreateThread(NULL, 0, f, d, 0, NULL);
    if (!t->thread)
      {
         free(t);
+        LKU(_ecore_thread_win32_lock);
         return -1;
      }
    t->val = d;
    *x = t;
+   _ecore_thread_win32_threads = eina_list_append(_ecore_thread_win32_threads, t);
+   LKU(_ecore_thread_win32_lock);
 
    return 0;
 }
 
 #  define PHC(x, f, d) _ecore_thread_win32_create(&(x), (LPTHREAD_START_ROUTINE)f, d)
 
-int
+static int
 _ecore_thread_win32_join(win32_thread *x,
                          void        **res)
 {
@@ -103,6 +131,7 @@ _ecore_thread_win32_join(win32_thread *x,
         CloseHandle(x->thread);
      }
    if (res) *res = x->val;
+   _ecore_thread_win32_threads = eina_list_remove(_ecore_thread_win32_threads, x);
    free(x);
 
    return 0;
@@ -167,7 +196,7 @@ struct _Ecore_Pthread_Worker
 
    const void     *data;
 
-   volatile int cancel;
+   int cancel;
 
 #ifdef EFL_HAVE_THREADS
    LK(cancel_mutex);
@@ -181,14 +210,6 @@ struct _Ecore_Pthread_Worker
 };
 
 #ifdef EFL_HAVE_THREADS
-typedef struct _Ecore_Pthread_Data Ecore_Pthread_Data;
-struct _Ecore_Pthread_Data
-{
-   Ecore_Pthread_Worker *death_job;
-   void                 *data;
-                         PH(thread);
-};
-
 typedef struct _Ecore_Pthread_Notify Ecore_Pthread_Notify;
 struct _Ecore_Pthread_Notify
 {
@@ -224,10 +245,11 @@ static void _ecore_thread_handler(void *data);
 
 static int _ecore_thread_count = 0;
 
-static Eina_List *_ecore_active_job_threads = NULL;
+static Eina_List *_ecore_running_job = NULL;
 static Eina_List *_ecore_pending_job_threads = NULL;
 static Eina_List *_ecore_pending_job_threads_feedback = NULL;
 static LK(_ecore_pending_job_threads_mutex);
+static LK(_ecore_running_job_mutex);
 
 static Eina_Hash *_ecore_thread_global_hash = NULL;
 static LRWK(_ecore_thread_global_hash_lock);
@@ -239,7 +261,7 @@ static Eina_Bool have_main_loop_thread = 0;
 static Eina_Trash *_ecore_thread_worker_trash = NULL;
 static int _ecore_thread_worker_count = 0;
 
-static void                 *_ecore_thread_worker(Ecore_Pthread_Data *pth);
+static void                 *_ecore_thread_worker(void *);
 static Ecore_Pthread_Worker *_ecore_thread_worker_new(void);
 
 static PH(get_main_loop_thread) (void)
@@ -267,6 +289,7 @@ _ecore_thread_worker_free(Ecore_Pthread_Worker *worker)
 
    if (_ecore_thread_worker_count > ((_ecore_thread_count_max + 1) * 16))
      {
+        _ecore_thread_worker_count--;
         free(worker);
         return;
      }
@@ -284,47 +307,9 @@ _ecore_thread_data_free(void *data)
 }
 
 static void
-_ecore_thread_end(Ecore_Pthread_Data *pth,
-                  Ecore_Thread       *work)
+_ecore_thread_join(PH(thread))
 {
-   Ecore_Pthread_Worker *worker = (Ecore_Pthread_Worker *)work;
-
-   if (((!worker->message_run) || 
-        (!worker->feedback_run) ||
-        ((worker->feedback_run) && (!worker->no_queue))) &&
-       (!worker->no_queue))
-     _ecore_thread_count--;
-
-   if (PHJ(pth->thread) != 0)
-     return;
-
-   if (eina_list_count(_ecore_pending_job_threads) > 0
-       && (unsigned int)_ecore_thread_count < eina_list_count(_ecore_pending_job_threads)
-       && _ecore_thread_count < _ecore_thread_count_max)
-     {
-        /* One more thread should be created. */
-         INF("spawning threads because of still pending jobs.");
-
-         pth->death_job = _ecore_thread_worker_new();
-         if (!pth->death_job) goto end;
-
-         eina_threads_init();
-
-         if (PHC(pth->thread, _ecore_thread_worker, pth) == 0)
-           {
-              _ecore_thread_count++;
-              return;
-           }
-
-         eina_threads_shutdown();
-
-end:
-         if (pth->death_job) _ecore_thread_worker_free(pth->death_job);
-     }
-
-   _ecore_active_job_threads = eina_list_remove(_ecore_active_job_threads, pth);
-
-   free(pth);
+   PHJ(thread);
 }
 
 static void
@@ -445,44 +430,47 @@ static void
 _ecore_short_job(PH(thread))
 {
    Ecore_Pthread_Worker *work;
+   int cancel;
 
-   while (_ecore_pending_job_threads)
+   LKL(_ecore_pending_job_threads_mutex);
+   
+   if (!_ecore_pending_job_threads)
      {
-        int cancel;
-
-        LKL(_ecore_pending_job_threads_mutex);
-
-        if (!_ecore_pending_job_threads)
-          {
-             LKU(_ecore_pending_job_threads_mutex);
-             break;
-          }
-
-        work = eina_list_data_get(_ecore_pending_job_threads);
-        _ecore_pending_job_threads = eina_list_remove_list(_ecore_pending_job_threads,
-                                                           _ecore_pending_job_threads);
-
         LKU(_ecore_pending_job_threads_mutex);
+        return;
+     }
+   
+   work = eina_list_data_get(_ecore_pending_job_threads);
+   _ecore_pending_job_threads = eina_list_remove_list(_ecore_pending_job_threads,
+                                                      _ecore_pending_job_threads);
+   LKU(_ecore_pending_job_threads_mutex);
 
-        LKL(work->cancel_mutex);
-        cancel = work->cancel;
-        LKU(work->cancel_mutex);
-        work->self = thread;
-        if (!cancel)
-          work->u.short_run.func_blocking((void *) work->data, (Ecore_Thread*) work);
+   LKL(_ecore_running_job_mutex);
+   _ecore_running_job = eina_list_append(_ecore_running_job, work);
+   LKU(_ecore_running_job_mutex);
+   
+   LKL(work->cancel_mutex);
+   cancel = work->cancel;
+   LKU(work->cancel_mutex);
+   work->self = thread;
+   if (!cancel)
+     work->u.short_run.func_blocking((void *) work->data, (Ecore_Thread*) work);
 
-        if (work->reschedule)
-          {
-             work->reschedule = EINA_FALSE;
-
-             LKL(_ecore_pending_job_threads_mutex);
-             _ecore_pending_job_threads = eina_list_append(_ecore_pending_job_threads, work);
-             LKU(_ecore_pending_job_threads_mutex);
-          }
-        else
-          {
-             ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
-          }
+   LKL(_ecore_running_job_mutex);
+   _ecore_running_job = eina_list_remove(_ecore_running_job, work);
+   LKU(_ecore_running_job_mutex);
+   
+   if (work->reschedule)
+     {
+        work->reschedule = EINA_FALSE;
+        
+        LKL(_ecore_pending_job_threads_mutex);
+        _ecore_pending_job_threads = eina_list_append(_ecore_pending_job_threads, work);
+        LKU(_ecore_pending_job_threads_mutex);
+     }
+   else
+     {
+        ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
      }
 }
 
@@ -490,53 +478,52 @@ static void
 _ecore_feedback_job(PH(thread))
 {
    Ecore_Pthread_Worker *work;
-
-   while (_ecore_pending_job_threads_feedback)
+   int cancel;
+   
+   LKL(_ecore_pending_job_threads_mutex);
+   
+   if (!_ecore_pending_job_threads_feedback)
      {
-        int cancel;
-
-        LKL(_ecore_pending_job_threads_mutex);
-
-        if (!_ecore_pending_job_threads_feedback)
-          {
-             LKU(_ecore_pending_job_threads_mutex);
-             break;
-          }
-
-        work = eina_list_data_get(_ecore_pending_job_threads_feedback);
-        _ecore_pending_job_threads_feedback = eina_list_remove_list(_ecore_pending_job_threads_feedback,
-                                                                    _ecore_pending_job_threads_feedback);
-
         LKU(_ecore_pending_job_threads_mutex);
+        return;
+     }
+   
+   work = eina_list_data_get(_ecore_pending_job_threads_feedback);
+   _ecore_pending_job_threads_feedback = eina_list_remove_list(_ecore_pending_job_threads_feedback,
+                                                               _ecore_pending_job_threads_feedback);
+   LKU(_ecore_pending_job_threads_mutex);
+   LKL(_ecore_running_job_mutex);
+   _ecore_running_job = eina_list_append(_ecore_running_job, work);
+   LKU(_ecore_running_job_mutex);
+   
+   LKL(work->cancel_mutex);
+   cancel = work->cancel;
+   LKU(work->cancel_mutex);
+   work->self = thread;
+   if (!cancel)
+     work->u.feedback_run.func_heavy((void *) work->data, (Ecore_Thread *) work);
 
-        LKL(work->cancel_mutex);
-        cancel = work->cancel;
-        LKU(work->cancel_mutex);
-        work->self = thread;
-        if (!cancel)
-          work->u.feedback_run.func_heavy((void *) work->data, (Ecore_Thread *) work);
+   LKL(_ecore_running_job_mutex);
+   _ecore_running_job = eina_list_remove(_ecore_running_job, work);
+   LKU(_ecore_running_job_mutex);
 
-        if (work->reschedule)
-          {
-             work->reschedule = EINA_FALSE;
-
-             LKL(_ecore_pending_job_threads_mutex);
-             _ecore_pending_job_threads_feedback = eina_list_append(_ecore_pending_job_threads_feedback, work);
-             LKU(_ecore_pending_job_threads_mutex);
-          }
-        else
-          {
-             ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
-          }
+   if (work->reschedule)
+     {
+        work->reschedule = EINA_FALSE;
+        
+        LKL(_ecore_pending_job_threads_mutex);
+        _ecore_pending_job_threads_feedback = eina_list_append(_ecore_pending_job_threads_feedback, work);
+        LKU(_ecore_pending_job_threads_mutex);
+     }
+   else
+     {
+        ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
      }
 }
 
 static void *
 _ecore_direct_worker(Ecore_Pthread_Worker *work)
 {
-   Ecore_Pthread_Worker *end;
-   Ecore_Pthread_Data *pth;
-
 #ifdef EFL_POSIX_THREADS
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
@@ -544,59 +531,23 @@ _ecore_direct_worker(Ecore_Pthread_Worker *work)
 
    eina_sched_prio_drop();
 
-   pth = malloc(sizeof (Ecore_Pthread_Data));
-   if (!pth) return NULL;
-
-   pth->thread = PHS();
-
-   work->self = pth->thread;
+   work->self = PHS();
    if (work->message_run)
      work->u.message_run.func_main((void *) work->data, (Ecore_Thread *) work);
    else
      work->u.feedback_run.func_heavy((void *) work->data, (Ecore_Thread *) work);
 
-   if (work->message_run)
-     {
-        end = work->u.message_run.direct_worker;
-        work->u.message_run.direct_worker = NULL;
-     }
-   else
-     {
-        end = work->u.feedback_run.direct_worker;
-        work->u.feedback_run.direct_worker = NULL;
-     }
-
    ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
 
-   if (!end)
-     {
-        free(pth);
-        return NULL;
-     }
-
-   end->data = pth;
-   end->u.short_run.func_blocking = NULL;
-   end->func_end = (void *)_ecore_thread_end;
-   end->func_cancel = NULL;
-   end->cancel = EINA_FALSE;
-   end->feedback_run = EINA_FALSE;
-   end->message_run = EINA_FALSE;
-//   end->no_queue = EINA_FALSE;
-   end->kill = EINA_FALSE;
-   end->hash = NULL;
-   LKI(end->mutex);
-   CDI(end->cond, end->mutex);
-
-   ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, end);
+   ecore_main_loop_thread_safe_call_async((Ecore_Cb) _ecore_thread_join, 
+					  (void*) PHS());
 
    return NULL;
 }
 
 static void *
-_ecore_thread_worker(Ecore_Pthread_Data *pth)
+_ecore_thread_worker(void *data __UNUSED__)
 {
-   Ecore_Pthread_Worker *work;
-
 #ifdef EFL_POSIX_THREADS
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
@@ -605,8 +556,8 @@ _ecore_thread_worker(Ecore_Pthread_Data *pth)
    eina_sched_prio_drop();
 
 restart:
-   if (_ecore_pending_job_threads) _ecore_short_job(pth->thread);
-   if (_ecore_pending_job_threads_feedback) _ecore_feedback_job(pth->thread);
+   _ecore_short_job(PHS());
+   _ecore_feedback_job(PHS());
 
    /* FIXME: Check if there is feedback running task todo, and switch to feedback run handler. */
 
@@ -622,7 +573,7 @@ restart:
 #ifdef _WIN32
    Sleep(1); /* around 50ms */
 #else
-   usleep(200);
+   usleep(50);
 #endif
 
    LKL(_ecore_pending_job_threads_mutex);
@@ -631,24 +582,12 @@ restart:
         LKU(_ecore_pending_job_threads_mutex);
         goto restart;
      }
+   _ecore_thread_count--;
+
+   ecore_main_loop_thread_safe_call_async((Ecore_Cb) _ecore_thread_join,
+					  (void*) PHS());
    LKU(_ecore_pending_job_threads_mutex);
 
-   work = pth->death_job;
-   if (!work) return NULL;
-
-   work->data = pth;
-   work->u.short_run.func_blocking = NULL;
-   work->func_end = (void *)_ecore_thread_end;
-   work->func_cancel = NULL;
-   work->cancel = EINA_FALSE;
-   work->feedback_run = EINA_FALSE;
-   work->message_run = EINA_FALSE;
-   work->kill = EINA_FALSE;
-//   work->no_queue = EINA_FALSE;
-   work->hash = NULL;
-
-   ecore_main_loop_thread_safe_call_async(_ecore_thread_handler, work);
-
    return NULL;
 }
 
@@ -662,8 +601,11 @@ _ecore_thread_worker_new(void)
 
    result = eina_trash_pop(&_ecore_thread_worker_trash);
 
-   if (!result) result = calloc(1, sizeof(Ecore_Pthread_Worker));
-   else _ecore_thread_worker_count--;
+   if (!result) 
+     {
+       result = calloc(1, sizeof(Ecore_Pthread_Worker));
+       _ecore_thread_worker_count++;
+     }
 
    LKI(result->cancel_mutex);
    LKI(result->mutex);
@@ -683,9 +625,13 @@ _ecore_thread_init(void)
      _ecore_thread_count_max = 1;
 
 #ifdef EFL_HAVE_THREADS
+# ifdef EFL_HAVE_WIN32_THREADS
+   LKI(_ecore_thread_win32_lock);
+# endif
    LKI(_ecore_pending_job_threads_mutex);
    LRWKI(_ecore_thread_global_hash_lock);
    LKI(_ecore_thread_global_hash_mutex);
+   LKI(_ecore_running_job_mutex);
    CDI(_ecore_thread_global_hash_cond, _ecore_thread_global_hash_mutex);
 #endif
 }
@@ -696,7 +642,9 @@ _ecore_thread_shutdown(void)
    /* FIXME: If function are still running in the background, should we kill them ? */
 #ifdef EFL_HAVE_THREADS
     Ecore_Pthread_Worker *work;
-    Ecore_Pthread_Data *pth;
+    Eina_List *l;
+    Eina_Bool test;
+    int iteration = 0;
 
     LKL(_ecore_pending_job_threads_mutex);
 
@@ -715,13 +663,35 @@ _ecore_thread_shutdown(void)
       }
 
     LKU(_ecore_pending_job_threads_mutex);
+    LKL(_ecore_running_job_mutex);
 
-    /* FIXME: Improve emergency shutdown, now that we use async call, we can do something */
-    EINA_LIST_FREE(_ecore_active_job_threads, pth)
+    EINA_LIST_FOREACH(_ecore_running_job, l, work)
+      ecore_thread_cancel((Ecore_Thread*) work);
+
+    LKU(_ecore_running_job_mutex);
+
+    do
       {
-         PHA(pth->thread);
-         PHJ(pth->thread);
+	 LKL(_ecore_pending_job_threads_mutex);
+	 if (_ecore_thread_count > 0)
+	   {
+	      test = EINA_TRUE;
+	   }
+	 else
+	   {
+	      test = EINA_FALSE;
+	   }
+	 LKU(_ecore_pending_job_threads_mutex);
+	 iteration++;
+	 if (test) usleep(50000);
       }
+    while (test == EINA_TRUE && iteration < 20);
+
+    if (iteration == 20 && _ecore_thread_count > 0)
+      {
+	 ERR("%i of the child thread are still running after 1s. This can lead to a segv. Sorry.", _ecore_thread_count);
+      }
+
     if (_ecore_thread_global_hash)
       eina_hash_free(_ecore_thread_global_hash);
     have_main_loop_thread = 0;
@@ -734,7 +704,11 @@ _ecore_thread_shutdown(void)
     LKD(_ecore_pending_job_threads_mutex);
     LRWKD(_ecore_thread_global_hash_lock);
     LKD(_ecore_thread_global_hash_mutex);
+    LKD(_ecore_running_job_mutex);
     CDD(_ecore_thread_global_hash_cond);
+# ifdef EFL_HAVE_WIN32_THREADS
+   LKU(_ecore_thread_win32_lock);
+# endif
 #endif
 }
 
@@ -745,8 +719,9 @@ ecore_thread_run(Ecore_Thread_Cb func_blocking,
                  const void     *data)
 {
    Ecore_Pthread_Worker *work;
+   Eina_Bool tried = EINA_FALSE;
 #ifdef EFL_HAVE_THREADS
-   Ecore_Pthread_Data *pth = NULL;
+   PH(thread);
 #endif
 
    EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
@@ -788,34 +763,27 @@ ecore_thread_run(Ecore_Thread_Cb func_blocking,
    LKU(_ecore_pending_job_threads_mutex);
 
    /* One more thread could be created. */
-   pth = malloc(sizeof (Ecore_Pthread_Data));
-   if (!pth) goto on_error;
-
-   pth->death_job = _ecore_thread_worker_new();
-   if (!pth->death_job) goto on_error;
-
    eina_threads_init();
 
-   if (PHC(pth->thread, _ecore_thread_worker, pth) == 0)
+   LKL(_ecore_pending_job_threads_mutex);
+
+ retry:
+   if (PHC(thread, _ecore_thread_worker, NULL) == 0)
      {
         _ecore_thread_count++;
+	LKU(_ecore_pending_job_threads_mutex);
         return (Ecore_Thread *)work;
      }
-
-   eina_threads_shutdown();
-
-on_error:
-   if (pth)
+   if (!tried)
      {
-        if (pth->death_job) _ecore_thread_worker_free(pth->death_job);
-        free(pth);
+       _ecore_main_call_flush();
+       tried = EINA_TRUE;
+       goto retry;
      }
 
    if (_ecore_thread_count == 0)
      {
-        LKL(_ecore_pending_job_threads_mutex);
         _ecore_pending_job_threads = eina_list_remove(_ecore_pending_job_threads, work);
-        LKU(_ecore_pending_job_threads_mutex);
 
         if (work->func_cancel)
           work->func_cancel((void *) work->data, (Ecore_Thread *) work);
@@ -826,6 +794,10 @@ on_error:
         free(work);
         work = NULL;
      }
+   LKU(_ecore_pending_job_threads_mutex);
+
+   eina_threads_shutdown();
+
    return (Ecore_Thread *)work;
 #else
    /*
@@ -962,7 +934,8 @@ ecore_thread_feedback_run(Ecore_Thread_Cb        func_heavy,
 {
 #ifdef EFL_HAVE_THREADS
    Ecore_Pthread_Worker *worker;
-   Ecore_Pthread_Data *pth = NULL;
+   Eina_Bool tried = EINA_FALSE;
+   PH(thread);
 
    EINA_MAIN_LOOP_CHECK_RETURN_VAL(NULL);
    
@@ -998,8 +971,15 @@ ecore_thread_feedback_run(Ecore_Thread_Cb        func_heavy,
 
         eina_threads_init();
 
+     retry_direct:
         if (PHC(t, _ecore_direct_worker, worker) == 0)
           return (Ecore_Thread *)worker;
+	if (!tried)
+	  {
+	     _ecore_main_call_flush();
+	     tried = EINA_TRUE;
+	     goto retry_direct;
+	  }
 
         if (worker->u.feedback_run.direct_worker)
           {
@@ -1024,35 +1004,32 @@ ecore_thread_feedback_run(Ecore_Thread_Cb        func_heavy,
    LKU(_ecore_pending_job_threads_mutex);
 
    /* One more thread could be created. */
-   pth = malloc(sizeof (Ecore_Pthread_Data));
-   if (!pth) goto on_error;
-
-   pth->death_job = _ecore_thread_worker_new();
-   if (!pth->death_job) goto on_error;
-
    eina_threads_init();
 
-   if (PHC(pth->thread, _ecore_thread_worker, pth) == 0)
+   LKL(_ecore_pending_job_threads_mutex);
+ retry:
+   if (PHC(thread, _ecore_thread_worker, NULL) == 0)
      {
         _ecore_thread_count++;
+	LKU(_ecore_pending_job_threads_mutex);
         return (Ecore_Thread *)worker;
      }
+   if (!tried)
+     {
+        _ecore_main_call_flush();
+	tried = EINA_TRUE;
+	goto retry;
+     }
+   LKU(_ecore_pending_job_threads_mutex);
 
    eina_threads_shutdown();
 
 on_error:
-   if (pth)
-     {
-        if (pth->death_job) _ecore_thread_worker_free(pth->death_job);
-        free(pth);
-     }
-
+   LKL(_ecore_pending_job_threads_mutex);
    if (_ecore_thread_count == 0)
      {
-        LKL(_ecore_pending_job_threads_mutex);
         _ecore_pending_job_threads_feedback = eina_list_remove(_ecore_pending_job_threads_feedback,
                                                                worker);
-        LKU(_ecore_pending_job_threads_mutex);
 
         if (func_cancel) func_cancel((void *)data, NULL);
 
@@ -1064,6 +1041,7 @@ on_error:
              worker = NULL;
           }
      }
+   LKU(_ecore_pending_job_threads_mutex);
 
    return (Ecore_Thread *)worker;
 #else
@@ -1129,7 +1107,7 @@ ecore_thread_feedback(Ecore_Thread *thread,
         Ecore_Pthread_Message *msg;
         Ecore_Pthread_Notify *notify;
 
-        msg = malloc(sizeof (Ecore_Pthread_Message*));
+        msg = malloc(sizeof (Ecore_Pthread_Message));
         if (!msg) return EINA_FALSE;
         msg->data = data;
         msg->callback = EINA_FALSE;
@@ -1310,7 +1288,7 @@ ecore_thread_max_set(int num)
    EINA_MAIN_LOOP_CHECK_RETURN;
    if (num < 1) return;
    /* avoid doing something hilarious by blocking dumb users */
-   if (num >= (2 * eina_cpu_count())) return;
+   if (num > (16 * eina_cpu_count())) num = 16 * eina_cpu_count();
 
    _ecore_thread_count_max = num;
 }
@@ -1318,7 +1296,7 @@ ecore_thread_max_set(int num)
 EAPI void
 ecore_thread_max_reset(void)
 {
-   EINA_MAIN_LOOP_CHECK_RETURN_VAL(0);
+   EINA_MAIN_LOOP_CHECK_RETURN;
    _ecore_thread_count_max = eina_cpu_count();
 }
 
