@@ -7,8 +7,13 @@
 #include <unistd.h>
 #include <time.h>
 #include <fcntl.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_ERRNO_H
+# include <errno.h>
+#endif
+
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
@@ -21,10 +26,6 @@
 # include <ws2tcpip.h>
 #endif
 
-#ifdef HAVE_EVIL
-# include <Evil.h>
-#endif
-
 #include <Ecore.h>
 #include <ecore_private.h>
 
@@ -59,13 +60,13 @@ ecore_con_local_shutdown(void)
 }
 
 int
-ecore_con_local_connect(Ecore_Con_Server                                                   *svr,
-                        Eina_Bool                                                           (*cb_done)(void *data,
-                                                                          Ecore_Fd_Handler *fd_handler),
-                        void *data                                                          __UNUSED__,
-                        void                                                                (*cb_free)(void *data,
-                                                                          void             *ev))
+ecore_con_local_connect(Ecore_Con_Server *svr,
+                        Eina_Bool (*cb_done)(void *data, Ecore_Fd_Handler *fd_handler),
+                        void *data __UNUSED__)
 {
+#ifndef HAVE_LOCAL_SOCKETS
+   return 0;
+#else
    char buf[4096];
    struct sockaddr_un socket_unix;
    int curstate = 0;
@@ -147,7 +148,10 @@ ecore_con_local_connect(Ecore_Con_Server
 
    if (connect(svr->fd, (struct sockaddr *)&socket_unix,
                socket_unix_len) < 0)
-     return 0;
+     {
+        ERR("local connection failed: %s", strerror(errno));
+        return 0;
+     }
 
    svr->path = strdup(buf);
    if (!svr->path)
@@ -162,22 +166,10 @@ ecore_con_local_connect(Ecore_Con_Server
    if (!svr->fd_handler)
      return 0;
 
-   if (!svr->delete_me)
-     {
-        /* we got our server! */
-         Ecore_Con_Event_Server_Add *e;
-
-         e = calloc(1, sizeof(Ecore_Con_Event_Server_Add));
-         if (e)
-           {
-              svr->event_count++;
-              e->server = svr;
-              ecore_event_add(ECORE_CON_EVENT_SERVER_ADD, e,
-                              cb_free, NULL);
-           }
-     }
+   if (!svr->delete_me) ecore_con_event_server_add(svr);
 
    return 1;
+#endif
 }
 
 int
@@ -190,6 +182,7 @@ ecore_con_local_listen(
   void *data
   __UNUSED__)
 {
+#ifdef HAVE_LOCAL_SOCKETS
    char buf[4096];
    struct sockaddr_un socket_unix;
    struct linger lin;
@@ -326,6 +319,7 @@ start:
 error_umask:
    umask(pmode);
 error:
+#endif /* HAVE_LOCAL_SOCKETS */
    return 0;
 }
 
