@@ -2,6 +2,32 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -557,7 +583,7 @@ ecore_con_socks_init(void)
    unsigned char addr6[sizeof(struct in6_addr)];
 #endif
 
-   /* ECORE_CON_SOCKS_V4=[user@]host:port-[1|0] */
+   /* ECORE_CON_SOCKS_V4=[user@]host-port:[1|0] */
    socks = getenv("ECORE_CON_SOCKS_V4");
    if (!socks)
      {
