@@ -489,6 +489,7 @@ static Ecore_Evas_Engine_Func _ecore_directfb_engine_func =
      NULL,                              /* ignore events */
      NULL,                              /* alpha */
      NULL, //transparent
+     NULL, // profiles_set
 
      NULL,
      NULL,
@@ -498,7 +499,8 @@ static Ecore_Evas_Engine_Func _ecore_directfb_engine_func =
      NULL,
 
      NULL, // render
-     NULL  // screen_geometry_get
+     NULL, // screen_geometry_get
+     NULL  // screen_dpi_get
 };
 #endif
 
@@ -592,6 +594,7 @@ ecore_evas_directfb_new(const char *disp_name __UNUSED__, int windowed __UNUSED_
 EAPI Ecore_DirectFB_Window *
 ecore_evas_directfb_window_get(const Ecore_Evas *ee)
 {
+   if (!(!strcmp(ee->driver, "directfb"))) return 0;
    return (Ecore_DirectFB_Window *) _ecore_evas_directfb_window_get(ee);
 }
 #else
