@@ -269,7 +269,7 @@ _ecore_evas_ews_move_resize(Ecore_Evas *ee, int x, int y, int w, int h)
 }
 
 static void
-_ecore_evas_ews_rotation_set(Ecore_Evas *ee, int rot, int resize)
+_ecore_evas_ews_rotation_set(Ecore_Evas *ee, int rot, int resize __UNUSED__)
 {
    if (ee->rotation == rot) return;
    ee->rotation = rot;
@@ -683,6 +683,7 @@ static const Ecore_Evas_Engine_Func _ecore_ews_engine_func =
      _ecore_evas_ews_ignore_events_set,
      _ecore_evas_ews_alpha_set,
      _ecore_evas_ews_transparent_set,
+     NULL, // profiles_set
    
      NULL,
      NULL,
@@ -692,7 +693,8 @@ static const Ecore_Evas_Engine_Func _ecore_ews_engine_func =
      NULL,
 
      _ecore_evas_ews_render,
-     _ecore_evas_ews_screen_geometry_get
+     _ecore_evas_ews_screen_geometry_get,
+     NULL  // screen_dpi_get
 };
 
 void
