@@ -2,6 +2,32 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -340,6 +366,7 @@ struct _Extn
         Shmfile    *shmfile;
         Eina_List  *updates;
         Eina_Bool   have_lock : 1;
+        Eina_Bool   have_real_lock : 1;
    } file;
 };
 
@@ -380,6 +407,30 @@ _ecore_evas_extn_event(Ecore_Evas *ee, int event)
                    _ecore_evas_extn_event_free, ee);
 }
 
+static Eina_Bool
+_ecore_evas_lock_other_have(Ecore_Evas *ee)
+{
+   Eina_List *l;
+   Ecore_Evas *ee2;
+   Extn *extn, *extn2;
+   
+   extn = ee->engine.buffer.data;
+   if (!extn) return EINA_FALSE;
+   // brute force - i know. i expect extn_ee_list to be fairly short. could
+   // be improved with a hash of lockfiles
+   EINA_LIST_FOREACH(extn_ee_list, l, ee2)
+     {
+        if (ee == ee2) continue;
+        extn2 = ee2->engine.buffer.data;
+        if (!extn2) continue;
+        if ((extn->file.lock) && (extn2->file.lock) &&
+            (!strcmp(extn->file.lock, extn2->file.lock)) &&
+            (extn2->file.have_real_lock))
+          return EINA_TRUE;
+     }
+   return EINA_FALSE;
+}
+
 static void
 _ecore_evas_socket_lock(Ecore_Evas *ee)
 {
@@ -389,8 +440,10 @@ _ecore_evas_socket_lock(Ecore_Evas *ee)
    if (!extn) return;
    if (extn->file.lockfd < 0) return;
    if (extn->file.have_lock) return;
-   flock(extn->file.lockfd, LOCK_EX);
    extn->file.have_lock = EINA_TRUE;
+   if (_ecore_evas_lock_other_have(ee)) return;
+   lockf(extn->file.lockfd, F_LOCK, 0);
+   extn->file.have_real_lock = EINA_TRUE;
 }
 
 static void
@@ -402,8 +455,9 @@ _ecore_evas_socket_unlock(Ecore_Evas *ee)
    if (!extn) return;
    if (extn->file.lockfd < 0) return;
    if (!extn->file.have_lock) return;
-   flock(extn->file.lockfd, LOCK_UN);
    extn->file.have_lock = EINA_FALSE;
+   if (!extn->file.have_real_lock) return;
+   lockf(extn->file.lockfd, F_ULOCK, 0);
 }
 
 static void
@@ -424,7 +478,7 @@ static void
 _ecore_evas_extn_plug_image_obj_del(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Ecore_Evas *ee = data;
-   if (ee) ecore_evas_free(ee);
+   ecore_evas_free(ee);
 }
 
 static void
@@ -475,6 +529,7 @@ _ecore_evas_extn_free(Ecore_Evas *ee)
    if (extn)
      {
         Ecore_Event_Handler *hdl;
+        Ipc_Data_Update *ipc;
 
         if (extn->file.have_lock)
           _ecore_evas_socket_unlock(ee);
@@ -503,6 +558,9 @@ _ecore_evas_extn_free(Ecore_Evas *ee)
                shmfile_close(extn->file.shmfile);
           }
 
+        EINA_LIST_FREE(extn->file.updates, ipc)
+          free(ipc);
+
         EINA_LIST_FREE(extn->ipc.handlers, hdl)
           ecore_event_handler_del(hdl);
         free(extn);
@@ -633,6 +691,7 @@ _ecore_evas_extn_cb_mouse_in(void *data, Evas *e __UNUSED__, Evas_Object *obj __
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Mouse_In ipc;
+        memset(&ipc, 0, sizeof(ipc));
 
         ipc.timestamp = ev->timestamp;
         ipc.mask = _ecore_evas_modifiers_locks_mask_get(ee->evas);
@@ -653,6 +712,7 @@ _ecore_evas_extn_cb_mouse_out(void *data, Evas *e __UNUSED__, Evas_Object *obj _
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Mouse_Out ipc;
+        memset(&ipc, 0, sizeof(ipc));
 
         ipc.timestamp = ev->timestamp;
         ipc.mask = _ecore_evas_modifiers_locks_mask_get(ee->evas);
@@ -672,14 +732,32 @@ _ecore_evas_extn_cb_mouse_down(void *data, Evas *e __UNUSED__, Evas_Object *obj
    if (!extn) return;
    if (extn->ipc.server)
      {
-        Ipc_Data_Ev_Mouse_Down ipc;
+       /* We have send mouse move event before mouse down event */
+       {
+          Ipc_Data_Ev_Mouse_Move ipc_move;
+          memset(&ipc_move, 0, sizeof(ipc_move));
+          Evas_Coord x, y;
 
-        ipc.b = ev->button;
-        ipc.flags = ev->flags;
-        ipc.timestamp = ev->timestamp;
-        ipc.mask = _ecore_evas_modifiers_locks_mask_get(ee->evas);
-        ipc.event_flags = ev->event_flags;
-        ecore_ipc_server_send(extn->ipc.server, MAJOR, OP_EV_MOUSE_DOWN, 0, 0, 0, &ipc, sizeof(ipc));
+          x = ev->canvas.x;
+          y = ev->canvas.y;
+          _ecore_evas_extn_coord_translate(ee, &x, &y);
+          ipc_move.x = x;
+          ipc_move.y = y;
+          ipc_move.timestamp = ev->timestamp;
+          ipc_move.mask = _ecore_evas_modifiers_locks_mask_get(ee->evas);
+          ipc_move.event_flags = ev->event_flags;
+          ecore_ipc_server_send(extn->ipc.server, MAJOR, OP_EV_MOUSE_MOVE, 0, 0, 0, &ipc_move, sizeof(ipc_move));
+       }
+       {
+          Ipc_Data_Ev_Mouse_Down ipc;
+          memset(&ipc, 0, sizeof(ipc));
+          ipc.b = ev->button;
+          ipc.flags = ev->flags;
+          ipc.timestamp = ev->timestamp;
+          ipc.mask = _ecore_evas_modifiers_locks_mask_get(ee->evas);
+          ipc.event_flags = ev->event_flags;
+          ecore_ipc_server_send(extn->ipc.server, MAJOR, OP_EV_MOUSE_DOWN, 0, 0, 0, &ipc, sizeof(ipc));
+       }
      }
 }
 
@@ -695,6 +773,7 @@ _ecore_evas_extn_cb_mouse_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Mouse_Up ipc;
+        memset(&ipc, 0, sizeof(ipc));
 
         ipc.b = ev->button;
         ipc.flags = ev->flags;
@@ -717,6 +796,7 @@ _ecore_evas_extn_cb_mouse_move(void *data, Evas *e __UNUSED__, Evas_Object *obj
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Mouse_Move ipc;
+        memset(&ipc, 0, sizeof(ipc));
         Evas_Coord x, y;
 
         x = ev->cur.canvas.x;
@@ -743,6 +823,7 @@ _ecore_evas_extn_cb_mouse_wheel(void *data, Evas *e __UNUSED__, Evas_Object *obj
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Mouse_Wheel ipc;
+        memset(&ipc, 0, sizeof(ipc));
 
         ipc.direction = ev->direction;
         ipc.z = ev->z;
@@ -765,6 +846,7 @@ _ecore_evas_extn_cb_multi_down(void *data, Evas *e __UNUSED__, Evas_Object *obj
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Multi_Down ipc;
+        memset(&ipc, 0, sizeof(ipc));
         Evas_Coord x, y;
 
         ipc.d = ev->device;
@@ -801,6 +883,7 @@ _ecore_evas_extn_cb_multi_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Multi_Up ipc;
+        memset(&ipc, 0, sizeof(ipc));
         Evas_Coord x, y;
 
         ipc.d = ev->device;
@@ -836,6 +919,7 @@ _ecore_evas_extn_cb_multi_move(void *data, Evas *e __UNUSED__, Evas_Object *obj
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Multi_Move ipc;
+        memset(&ipc, 0, sizeof(ipc));
         Evas_Coord x, y;
 
         ipc.d = ev->device;
@@ -858,15 +942,6 @@ _ecore_evas_extn_cb_multi_move(void *data, Evas *e __UNUSED__, Evas_Object *obj
      }
 }
 
-static void
-_ecore_evas_extn_cb_free(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
-{
-   Ecore_Evas *ee;
-
-   ee = data;
-   if (ee->driver) _ecore_evas_free(ee);
-}
-
 static void
 _ecore_evas_extn_cb_key_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
@@ -991,6 +1066,7 @@ _ecore_evas_extn_cb_hold(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUS
    if (extn->ipc.server)
      {
         Ipc_Data_Ev_Hold ipc;
+        memset(&ipc, 0, sizeof(ipc));
 
         ipc.hold = ev->hold;
         ipc.timestamp = ev->timestamp;
@@ -1103,7 +1179,7 @@ static const Ecore_Evas_Engine_Func _ecore_extn_plug_engine_func =
    NULL,
    NULL,
    NULL, //transparent
-
+   NULL, // profiles_set
    NULL,
    NULL,
    NULL,
@@ -1112,7 +1188,8 @@ static const Ecore_Evas_Engine_Func _ecore_extn_plug_engine_func =
    NULL,
 
    NULL, // render
-   NULL  // screen_geometry_get
+   NULL, // screen_geometry_get
+   NULL  // screen_dpi_get
 };
 
 static Eina_Bool
@@ -1199,6 +1276,7 @@ _ipc_server_data(void *data, int type __UNUSED__, void *event)
                      evas_object_image_data_update_add(ee->engine.buffer.image,
                                                        ipc->x, ipc->y,
                                                        ipc->w, ipc->h);
+                   free(ipc);
                 }
            }
          break;
@@ -1206,6 +1284,7 @@ _ipc_server_data(void *data, int type __UNUSED__, void *event)
          if ((e->data) && (e->size > 0) &&
              (((unsigned char *)e->data)[e->size - 1] == 0))
            {
+              if (extn->file.have_lock) _ecore_evas_socket_unlock(ee);
               if (extn->file.lockfd) close(extn->file.lockfd);
               if (extn->file.lock) eina_stringshare_del(extn->file.lock);
               extn->file.lock = eina_stringshare_add(e->data);
@@ -1312,7 +1391,8 @@ ecore_evas_extn_plug_new(Ecore_Evas *ee_target)
    if (!ee) return NULL;
 
    o = evas_object_image_filled_add(ee_target->evas);
-   evas_object_image_content_hint_set(o, EVAS_IMAGE_CONTENT_HINT_DYNAMIC);
+   /* this make problem in gl engine, so I'll block this until solve problem 
+   evas_object_image_content_hint_set(o, EVAS_IMAGE_CONTENT_HINT_DYNAMIC);*/
    evas_object_image_colorspace_set(o, EVAS_COLORSPACE_ARGB8888);
    evas_object_image_alpha_set(o, 1);
    evas_object_image_size_set(o, 1, 1);
@@ -1372,9 +1452,6 @@ ecore_evas_extn_plug_new(Ecore_Evas *ee_target)
    evas_object_event_callback_add(ee->engine.buffer.image,
                                   EVAS_CALLBACK_MULTI_MOVE,
                                   _ecore_evas_extn_cb_multi_move, ee);
-   evas_object_event_callback_add(ee->engine.buffer.image,
-                                  EVAS_CALLBACK_FREE,
-                                  _ecore_evas_extn_cb_free, ee);
    evas_object_event_callback_add(ee->engine.buffer.image,
                                   EVAS_CALLBACK_KEY_DOWN,
                                   _ecore_evas_extn_cb_key_down, ee);
@@ -1584,6 +1661,8 @@ _ecore_evas_extn_socket_render(Ecore_Evas *ee)
           rend |= ee2->engine.func->fn_render(ee2);
         if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
+   if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
+
    if (ee->engine.buffer.pixels)
      {
         _ecore_evas_socket_lock(ee);
@@ -1610,6 +1689,7 @@ _ecore_evas_extn_socket_render(Ecore_Evas *ee)
            ecore_ipc_client_send(client, MAJOR, OP_UPDATE_DONE, 0, 0, 0, NULL, 0);
      }
 
+   if (ee->func.fn_post_render) ee->func.fn_post_render(ee);
    return updates ? 1 : rend;
 }
 
@@ -1987,6 +2067,7 @@ static const Ecore_Evas_Engine_Func _ecore_extn_socket_engine_func =
    NULL,
    _ecore_evas_extn_socket_alpha_set,
    NULL, //transparent
+   NULL, // profiles_set
 
    NULL,
    NULL,
@@ -1996,7 +2077,8 @@ static const Ecore_Evas_Engine_Func _ecore_extn_socket_engine_func =
    NULL,
    
    _ecore_evas_extn_socket_render, // render
-   NULL  // screen_geometry_get
+   NULL,  // screen_geometry_get
+   NULL,  // screen_dpi_get
 };
 
 #endif
