@@ -551,6 +551,7 @@ static Ecore_Evas_Engine_Func _ecore_fb_engine_func =
      NULL,
      NULL,
      NULL, //transparent
+     NULL, // profiles_set
 
      NULL,
      NULL,
@@ -560,7 +561,8 @@ static Ecore_Evas_Engine_Func _ecore_fb_engine_func =
      NULL,
 
      NULL, // render
-     NULL  // screen_geometry_get
+     NULL, // screen_geometry_get
+     NULL  // screen_dpi_get
 };
 #endif
 
@@ -639,7 +641,7 @@ ecore_evas_fb_new(const char *disp_name, int rotation, int w, int h)
      }
 
    einfo = (Evas_Engine_Info_FB *)evas_engine_info_get(ee->evas);
-   if (einfo)
+   if (einfo && disp_name)
      {
         einfo->info.virtual_terminal = 0;
         einfo->info.device_number = strtol(disp_name, NULL, 10);
@@ -663,6 +665,11 @@ ecore_evas_fb_new(const char *disp_name, int rotation, int w, int h)
 
    ee->engine.func->fn_render = _ecore_evas_fb_render;
    _ecore_evas_register(ee);
+   ecore_event_window_register(1, ee, ee->evas,
+                               (Ecore_Event_Mouse_Move_Cb)_ecore_evas_mouse_move_process,
+                               (Ecore_Event_Multi_Move_Cb)_ecore_evas_mouse_multi_move_process,
+                               (Ecore_Event_Multi_Down_Cb)_ecore_evas_mouse_multi_down_process,
+                               (Ecore_Event_Multi_Up_Cb)_ecore_evas_mouse_multi_up_process);
    fb_ee = ee;
    evas_event_feed_mouse_in(ee->evas, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
    return ee;
