@@ -177,6 +177,7 @@ struct _Ecore_Evas_Engine_Func
    void (*fn_ignore_events_set) (Ecore_Evas *ee, int ignore);
    void (*fn_alpha_set) (Ecore_Evas *ee, int alpha);
    void (*fn_transparent_set) (Ecore_Evas *ee, int transparent);
+   void (*fn_profiles_set) (Ecore_Evas *ee, const char **profiles, int num_profiles);
 
    void (*fn_window_group_set) (Ecore_Evas *ee, const Ecore_Evas *ee_group);
    void (*fn_aspect_set) (Ecore_Evas *ee, double aspect);
@@ -187,6 +188,7 @@ struct _Ecore_Evas_Engine_Func
 
    int (*fn_render) (Ecore_Evas *ee);
    void (*fn_screen_geometry_get) (const Ecore_Evas *ee, int *x, int *y, int *w, int *h);
+   void (*fn_screen_dpi_get) (const Ecore_Evas *ee, int *xdpi, int *ydpi);
 };
 
 struct _Ecore_Evas_Engine
@@ -287,6 +289,7 @@ struct _Ecore_Evas_Engine
         size_t pool_size;
         void *pool_data;
         struct wl_buffer *buffer;
+        Eina_Bool buffer_valid;
 # endif
 
      } wl;
@@ -330,6 +333,7 @@ struct _Ecore_Evas
       char           *title;
       char           *name;
       char           *clas;
+      char           *profile;
       struct {
 	 int          w, h;
       } min,
@@ -396,6 +400,7 @@ struct _Ecore_Evas
    unsigned char no_comp_sync  : 1;
    unsigned char semi_sync  : 1;
    unsigned char deleted : 1;
+   int           gl_sync_draw_done; // added by gl77.lee
 };
 
 void _ecore_evas_ref(Ecore_Evas *ee);
