@@ -2,6 +2,32 @@
 # include "config.h"
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 //#define LOGFNS 1
 
 #ifdef LOGFNS
@@ -72,7 +98,7 @@ static void _ecore_evas_wl_alpha_set(Ecore_Evas *ee, int alpha);
 static void _ecore_evas_wl_transparent_set(Ecore_Evas *ee, int transparent);
 static int _ecore_evas_wl_render(Ecore_Evas *ee);
 static void _ecore_evas_wl_screen_geometry_get(const Ecore_Evas *ee __UNUSED__, int *x, int *y, int *w, int *h);
-
+static void _ecore_evas_wl_screen_dpi_get(const Ecore_Evas *ee __UNUSED__, int *xdpi, int *ydpi);
 static Eina_Bool _ecore_evas_wl_cb_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
 static Eina_Bool _ecore_evas_wl_cb_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
 static Eina_Bool _ecore_evas_wl_cb_focus_in(void *data __UNUSED__, int type __UNUSED__, void *event);
@@ -143,6 +169,7 @@ static Ecore_Evas_Engine_Func _ecore_wl_engine_func =
    _ecore_evas_wl_ignore_events_set,
    _ecore_evas_wl_alpha_set,
    _ecore_evas_wl_transparent_set,
+   NULL, // func profiles set
    NULL,
    NULL,
    NULL,
@@ -150,7 +177,8 @@ static Ecore_Evas_Engine_Func _ecore_wl_engine_func =
    NULL,
    NULL,
    _ecore_evas_wl_render, 
-   _ecore_evas_wl_screen_geometry_get
+   _ecore_evas_wl_screen_geometry_get,
+   _ecore_evas_wl_screen_dpi_get
 };
 
 /* external variables */
@@ -240,8 +268,9 @@ ecore_evas_wayland_egl_new(const char *disp_name, unsigned int parent, int x, in
    if ((einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas)))
      {
         einfo->info.display = ecore_wl_display_get();
-        einfo->info.destination_alpha = EINA_FALSE;
+        einfo->info.destination_alpha = ee->alpha;
         einfo->info.rotation = ee->rotation;
+        einfo->info.depth = 32;
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("Failed to set Evas Engine Info for '%s'", ee->driver);
@@ -269,8 +298,8 @@ ecore_evas_wayland_egl_new(const char *disp_name, unsigned int parent, int x, in
         evas_object_move(ee->engine.wl.frame, 0, 0);
      }
 
-   ecore_evas_input_event_register(ee);
    _ecore_evas_register(ee);
+   ecore_evas_input_event_register(ee);
 
    ecore_event_window_register(ee->prop.window, ee, ee->evas, 
                                (Ecore_Event_Mouse_Move_Cb)_ecore_evas_mouse_move_process, 
@@ -278,7 +307,7 @@ ecore_evas_wayland_egl_new(const char *disp_name, unsigned int parent, int x, in
                                (Ecore_Event_Multi_Down_Cb)_ecore_evas_mouse_multi_down_process, 
                                (Ecore_Event_Multi_Up_Cb)_ecore_evas_mouse_multi_up_process);
 
-   evas_event_feed_mouse_in(ee->evas, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
+   /* evas_event_feed_mouse_in(ee->evas, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL); */
 
    return ee;
 }
@@ -429,51 +458,96 @@ _ecore_evas_wl_move(Ecore_Evas *ee, int x, int y)
    if (!ee) return;
    ee->req.x = x;
    ee->req.y = y;
-   ee->x = x;
-   ee->y = y;
-   if (ee->engine.wl.win) ecore_wl_window_move(ee->engine.wl.win, x, y);
-   if (ee->func.fn_move) ee->func.fn_move(ee);
+   if ((ee->x != x) || (ee->y != y))
+     {
+        ee->x = x;
+        ee->y = y;
+        if (ee->engine.wl.win) 
+          ecore_wl_window_update_location(ee->engine.wl.win, x, y);
+        if (ee->func.fn_move) ee->func.fn_move(ee);
+     }
 }
 
 static void 
 _ecore_evas_wl_resize(Ecore_Evas *ee, int w, int h)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   int fw = 0, fh = 0;
 
    if (!ee) return;
    if (w < 1) w = 1;
    if (h < 1) h = 1;
-//   if ((ee->w == w) && (ee->h == h)) return;
-
-   if (ee->prop.min.w > w) w = ee->prop.min.w;
-   else if (w > ee->prop.max.w) w = ee->prop.max.w;
-   if (ee->prop.min.h > h) h = ee->prop.min.h;
-   else if (h > ee->prop.max.h) h = ee->prop.max.h;
 
    ee->req.w = w;
    ee->req.h = h;
 
+   if (!ee->prop.fullscreen)
+     {
+        if (ee->prop.min.w > w) w = ee->prop.min.w;
+        else if (w > ee->prop.max.w) w = ee->prop.max.w;
+        if (ee->prop.min.h > h) h = ee->prop.min.h;
+        else if (h > ee->prop.max.h) h = ee->prop.max.h;
+
+        evas_output_framespace_get(ee->evas, NULL, NULL, &fw, &fh);
+        w += fw;
+        h += fh;
+     }
+
 //   ecore_wl_window_damage(ee->engine.wl.win, 0, 0, ee->w, ee->h);
 
-   ee->w = w;
-   ee->h = h;
+   if ((ee->w != w) || (ee->h != h))
+     {
+        ee->w = w;
+        ee->h = h;
 
-   /* change evas output & viewport sizes */
-   evas_output_size_set(ee->evas, ee->w, ee->h);
-   evas_output_viewport_set(ee->evas, 0, 0, ee->w, ee->h);
-   evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
-   if (ee->engine.wl.frame)
-     evas_object_resize(ee->engine.wl.frame, ee->w, ee->h);
+        if ((ee->rotation == 90) || (ee->rotation == 270))
+          {
+             evas_output_size_set(ee->evas, h, w);
+             evas_output_viewport_set(ee->evas, 0, 0, h, w);
+          }
+        else 
+          {
+             evas_output_size_set(ee->evas, w, h);
+             evas_output_viewport_set(ee->evas, 0, 0, w, h);
+          }
 
-   /* set new engine destination */
-   /* evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo); */
+        if (ee->prop.avoid_damage)
+          {
+             int pdam = 0;
 
-   /* ecore_wl_window_damage(ee->engine.wl.win, 0, 0, ee->w, ee->h); */
-   ecore_wl_flush();
+             pdam = ecore_evas_avoid_damage_get(ee);
+             ecore_evas_avoid_damage_set(ee, 0);
+             ecore_evas_avoid_damage_set(ee, pdam);
+          }
 
-   ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+        if (ee->engine.wl.frame)
+          evas_object_resize(ee->engine.wl.frame, w, h);
 
-   if (ee->func.fn_resize) ee->func.fn_resize(ee);
+        /* set new engine destination */
+        /* evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo); */
+
+        /* ecore_wl_window_damage(ee->engine.wl.win, 0, 0, ee->w, ee->h); */
+
+        // WAS ACTIVE
+        /* ecore_wl_flush(); */
+
+        if (ee->engine.wl.win)
+          {
+             Evas_Engine_Info_Wayland_Egl *einfo;
+
+             if ((einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas)))
+               {
+                  einfo->info.edges = ee->engine.wl.win->edges;
+                  if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
+                    ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+               }
+
+             ecore_wl_window_update_size(ee->engine.wl.win, w, h);
+             ecore_wl_window_buffer_attach(ee->engine.wl.win, NULL, 0, 0);
+          }
+
+        if (ee->func.fn_resize) ee->func.fn_resize(ee);
+     }
 }
 
 static void 
@@ -488,26 +562,38 @@ _ecore_evas_wl_show(Ecore_Evas *ee)
    if (ee->engine.wl.win)
      {
         ecore_wl_window_show(ee->engine.wl.win);
-        ecore_wl_flush();
-     }
+        ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+        ecore_wl_window_buffer_attach(ee->engine.wl.win, NULL, 0, 0);
 
-   einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas);
-   if (!einfo)
-     {
-        ERR("Failed to get Evas Engine Info for '%s'", ee->driver);
-        return;
+        if ((ee->prop.clas) && (ee->engine.wl.win->shell_surface))
+          wl_shell_surface_set_class(ee->engine.wl.win->shell_surface, 
+                                     ee->prop.clas);
+        if ((ee->prop.title) && (ee->engine.wl.win->shell_surface))
+          wl_shell_surface_set_title(ee->engine.wl.win->shell_surface, 
+                                     ee->prop.title);
      }
 
-   einfo->info.surface = ecore_wl_window_surface_get(ee->engine.wl.win);
-   evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
-
    if (ee->engine.wl.frame)
      {
         evas_object_show(ee->engine.wl.frame);
         evas_object_resize(ee->engine.wl.frame, ee->w, ee->h);
      }
 
-   /* ecore_wl_window_buffer_attach(ee->engine.wl.win, ee->engine.wl.buffer, 0, 0); */
+   if (ee->engine.wl.win)
+     {
+        einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas);
+        if (!einfo)
+          {
+             ERR("Failed to get Evas Engine Info for '%s'", ee->driver);
+             return;
+          }
+
+        einfo->info.surface = ecore_wl_window_surface_get(ee->engine.wl.win);
+        /* if (einfo->info.surface) */
+        evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
+        /* else */
+        /*   printf("Failed to get a Surface from Ecore_Wl\n"); */
+     }
 
    ee->visible = 1;
    if (ee->func.fn_show) ee->func.fn_show(ee);
@@ -522,12 +608,6 @@ _ecore_evas_wl_hide(Ecore_Evas *ee)
 
    if ((!ee) || (!ee->visible)) return;
 
-   if (ee->engine.wl.win) 
-     {
-        ecore_wl_window_hide(ee->engine.wl.win);
-        ecore_wl_flush();
-     }
-
    einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas);
    if (einfo)
      {
@@ -535,6 +615,9 @@ _ecore_evas_wl_hide(Ecore_Evas *ee)
         evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
      }
 
+   if (ee->engine.wl.win) 
+     ecore_wl_window_hide(ee->engine.wl.win);
+
    ee->visible = 0;
    ee->should_be_visible = 0;
 
@@ -563,9 +646,14 @@ _ecore_evas_wl_title_set(Ecore_Evas *ee, const char *title)
      {
         EE_Wl_Smart_Data *sd;
 
-        if (!(sd = evas_object_smart_data_get(ee->engine.wl.frame))) return;
-        evas_object_text_text_set(sd->text, ee->prop.title);
+        if ((sd = evas_object_smart_data_get(ee->engine.wl.frame)))
+          evas_object_text_text_set(sd->text, ee->prop.title);
+
      }
+
+   if ((ee->prop.title) && (ee->engine.wl.win->shell_surface))
+     wl_shell_surface_set_title(ee->engine.wl.win->shell_surface, 
+                                ee->prop.title);
 }
 
 static void 
@@ -580,7 +668,10 @@ _ecore_evas_wl_name_class_set(Ecore_Evas *ee, const char *n, const char *c)
    ee->prop.clas = NULL;
    if (n) ee->prop.name = strdup(n);
    if (c) ee->prop.clas = strdup(c);
-   /* FIXME: Forward these changes to Wayland somehow */
+
+   if ((ee->prop.clas) && (ee->engine.wl.win->shell_surface))
+     wl_shell_surface_set_class(ee->engine.wl.win->shell_surface, 
+                                ee->prop.clas);
 }
 
 static void 
@@ -667,6 +758,8 @@ _ecore_evas_wl_maximized_set(Ecore_Evas *ee, int max)
    if (ee->prop.maximized == max) return;
    ee->prop.maximized = max;
    ecore_wl_window_maximized_set(ee->engine.wl.win, max);
+   if (ee->func.fn_state_change)
+     ee->func.fn_state_change(ee);
 }
 
 static void 
@@ -674,10 +767,12 @@ _ecore_evas_wl_fullscreen_set(Ecore_Evas *ee, int full)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
-   if ((!ee) || (!ee->visible)) return;
+   if (!ee) return;
    if (ee->prop.fullscreen == full) return;
    ee->prop.fullscreen = full;
    ecore_wl_window_fullscreen_set(ee->engine.wl.win, full);
+   if (ee->func.fn_state_change)
+     ee->func.fn_state_change(ee);
 }
 
 static void 
@@ -785,6 +880,21 @@ _ecore_evas_wl_screen_geometry_get(const Ecore_Evas *ee __UNUSED__, int *x, int
    ecore_wl_screen_size_get(w, h);
 }
 
+static void 
+_ecore_evas_wl_screen_dpi_get(const Ecore_Evas *ee __UNUSED__, int *xdpi, int *ydpi)
+{
+   int dpi = 0;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (xdpi) *xdpi = 0;
+   if (ydpi) *ydpi = 0;
+   /* FIXME: Ideally this needs to get the DPI from a specific screen */
+   dpi = ecore_wl_dpi_get();
+   if (xdpi) *xdpi = dpi;
+   if (ydpi) *ydpi = dpi;
+}
+
 void 
 _ecore_evas_wayland_egl_resize(Ecore_Evas *ee, int location)
 {
@@ -793,6 +903,15 @@ _ecore_evas_wayland_egl_resize(Ecore_Evas *ee, int location)
    if (!ee) return;
    if (ee->engine.wl.win) 
      {
+        Evas_Engine_Info_Wayland_Egl *einfo;
+
+        if ((einfo = (Evas_Engine_Info_Wayland_Egl *)evas_engine_info_get(ee->evas)))
+          {
+             einfo->info.edges = location;
+             if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
+               ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+          }
+
         ee->engine.wl.win->resizing = EINA_TRUE;
         ecore_wl_window_resize(ee->engine.wl.win, ee->w, ee->h, location);
      }
@@ -823,10 +942,13 @@ _ecore_evas_wl_cb_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *eve
    ee = ecore_event_window_match(ev->window);
    if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
    if (ev->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
    if (ee->func.fn_mouse_in) ee->func.fn_mouse_in(ee);
    ecore_event_evas_modifier_lock_update(ee->evas, ev->modifiers);
    evas_event_feed_mouse_in(ee->evas, ev->timestamp, NULL);
    _ecore_evas_mouse_move_process(ee, ev->x, ev->y, ev->timestamp);
+   ee->in = EINA_TRUE;
+
    return ECORE_CALLBACK_PASS_ON;
 }
 
@@ -891,6 +1013,7 @@ _ecore_evas_wl_cb_window_configure(void *data __UNUSED__, int type __UNUSED__, v
 {
    Ecore_Evas *ee;
    Ecore_Wl_Event_Window_Configure *ev;
+   int nw = 0, nh = 0;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
@@ -898,18 +1021,43 @@ _ecore_evas_wl_cb_window_configure(void *data __UNUSED__, int type __UNUSED__, v
    ee = ecore_event_window_match(ev->win);
    if (!ee) return ECORE_CALLBACK_PASS_ON;
    if (ev->win != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
+   if (ee->prop.fullscreen)
+     {
+        _ecore_evas_wl_move(ee, ev->x, ev->y);
+        _ecore_evas_wl_resize(ee, ev->w, ev->h);
+
+        return ECORE_CALLBACK_PASS_ON;
+     }
+
    if ((ee->x != ev->x) || (ee->y != ev->y))
      {
-        ee->x = ev->x;
-        ee->y = ev->y;
         ee->req.x = ee->x;
         ee->req.y = ee->y;
         if (ee->func.fn_move) ee->func.fn_move(ee);
      }
-   if ((ee->req.w != ev->w) || (ee->req.h != ev->h))
+
+   nw = ev->w;
+   nh = ev->h;
+
+   if ((ee->prop.maximized) || (!ee->prop.fullscreen))
      {
-        ee->req.w = ev->w;
-        ee->req.h = ev->h;
+        int fw = 0, fh = 0;
+
+        evas_output_framespace_get(ee->evas, NULL, NULL, &fw, &fh);
+        nw = ev->w - fw;
+        nh = ev->h - fh;
+     }
+
+   if (ee->prop.min.w > nw) nw = ee->prop.min.w;
+   else if (nw > ee->prop.max.w) nw = ee->prop.max.w;
+   if (ee->prop.min.h > nh) nh = ee->prop.min.h;
+   else if (nh > ee->prop.max.h) nh = ee->prop.max.h;
+
+   if ((ee->w != nw) || (ee->h != nh))
+     {
+        ee->req.w = nw;
+        ee->req.h = nh;
         if (ee->func.fn_resize) ee->func.fn_resize(ee);
      }
 
@@ -954,7 +1102,6 @@ _ecore_evas_wl_smart_add(Evas_Object *obj)
    sd->h = 1;
 
    sd->frame = evas_object_rectangle_add(evas);
-   evas_object_is_frame_object_set(sd->frame, EINA_TRUE);
    evas_object_color_set(sd->frame, 249, 249, 249, 255);
    evas_object_smart_member_add(sd->frame, obj);
 
@@ -963,6 +1110,7 @@ _ecore_evas_wl_smart_add(Evas_Object *obj)
    evas_object_text_style_set(sd->text, EVAS_TEXT_STYLE_PLAIN);
    evas_object_text_font_set(sd->text, "Sans", 10);
    evas_object_text_text_set(sd->text, "Smart Test");
+   evas_object_smart_member_add(sd->text, obj);
 
    evas_object_smart_data_set(obj, sd);
 }
