@@ -2,6 +2,32 @@
 # include "config.h"
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 //#define LOGFNS 1
 
 #ifdef LOGFNS
@@ -40,6 +66,12 @@ struct _EE_Wl_Smart_Data
    Evas_Coord x, y, w, h;
 };
 
+struct _Ecore_Evas_Engine_Wl_Data
+{
+   Ecore_Wl_Window *win;
+   Evas_Object *frame;
+};
+
 /* local function prototypes */
 static int _ecore_evas_wl_init(void);
 static int _ecore_evas_wl_shutdown(void);
@@ -54,6 +86,7 @@ static void _ecore_evas_wl_callback_mouse_in_set(Ecore_Evas *ee, void (*func)(Ec
 static void _ecore_evas_wl_callback_mouse_out_set(Ecore_Evas *ee, void (*func)(Ecore_Evas *ee));
 static void _ecore_evas_wl_move(Ecore_Evas *ee, int x, int y);
 static void _ecore_evas_wl_resize(Ecore_Evas *ee, int w, int h);
+static void _ecore_evas_wl_move_resize(Ecore_Evas *ee, int x, int y, int w, int h);
 static void _ecore_evas_wl_show(Ecore_Evas *ee);
 static void _ecore_evas_wl_hide(Ecore_Evas *ee);
 static void _ecore_evas_wl_raise(Ecore_Evas *ee);
@@ -72,8 +105,12 @@ static void _ecore_evas_wl_alpha_set(Ecore_Evas *ee, int alpha);
 static void _ecore_evas_wl_transparent_set(Ecore_Evas *ee, int transparent);
 static int _ecore_evas_wl_render(Ecore_Evas *ee);
 static void _ecore_evas_wl_screen_geometry_get(const Ecore_Evas *ee __UNUSED__, int *x, int *y, int *w, int *h);
+static void _ecore_evas_wl_screen_dpi_get(const Ecore_Evas *ee __UNUSED__, int *xdpi, int *ydpi);
+static void _ecore_evas_wl_ensure_pool_size(Ecore_Evas *ee, int w, int h);
 static struct wl_shm_pool *_ecore_evas_wl_shm_pool_create(int size, void **data);
 
+static void _ecore_evas_wl_frame_complete(void *data, struct wl_callback *callback, uint32_t tm EINA_UNUSED);
+
 static void _ecore_evas_wl_buffer_new(Ecore_Evas *ee, struct wl_shm_pool *pool);
 
 static Eina_Bool _ecore_evas_wl_cb_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
@@ -98,6 +135,11 @@ static Evas_Object *_ecore_evas_wl_frame_add(Evas *evas);
 static int _ecore_evas_wl_init_count = 0;
 static Ecore_Event_Handler *_ecore_evas_wl_event_hdls[5];
 
+static const struct wl_callback_listener frame_listener =
+{
+   _ecore_evas_wl_frame_complete,
+};
+
 static Ecore_Evas_Engine_Func _ecore_wl_engine_func = 
 {
    _ecore_evas_wl_free,
@@ -118,7 +160,7 @@ static Ecore_Evas_Engine_Func _ecore_wl_engine_func =
    _ecore_evas_wl_move,
    NULL, // managed_move
    _ecore_evas_wl_resize,
-   NULL, // move_resize
+   _ecore_evas_wl_move_resize,
    NULL, // rotation_set
    NULL, // shaped_set
    _ecore_evas_wl_show,
@@ -146,14 +188,16 @@ static Ecore_Evas_Engine_Func _ecore_wl_engine_func =
    _ecore_evas_wl_ignore_events_set,
    _ecore_evas_wl_alpha_set,
    _ecore_evas_wl_transparent_set,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
+   NULL, // func profiles set
+   NULL, // window group set
+   NULL, // aspect set
+   NULL, // urgent set
+   NULL, // modal set
+   NULL, // demand attention set
+   NULL, // focus skip set
    _ecore_evas_wl_render, 
-   _ecore_evas_wl_screen_geometry_get
+   _ecore_evas_wl_screen_geometry_get,
+   _ecore_evas_wl_screen_dpi_get
 };
 
 /* external variables */
@@ -218,6 +262,7 @@ ecore_evas_wayland_shm_new(const char *disp_name, unsigned int parent, int x, in
    ee->prop.request_pos = 0;
    ee->prop.sticky = 0;
    ee->prop.draw_frame = frame;
+   ee->alpha = EINA_FALSE;
 
    ee->evas = evas_new();
    evas_data_attach_set(ee->evas, ee);
@@ -229,8 +274,7 @@ ecore_evas_wayland_shm_new(const char *disp_name, unsigned int parent, int x, in
    if (ee->prop.draw_frame)
      evas_output_framespace_set(ee->evas, 4, 18, 8, 22);
 
-   if (parent)
-     p = ecore_wl_window_find(parent);
+   if (parent) p = ecore_wl_window_find(parent);
 
    /* FIXME: Get if parent is alpha, and set */
 
@@ -241,6 +285,8 @@ ecore_evas_wayland_shm_new(const char *disp_name, unsigned int parent, int x, in
 
    if ((einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas)))
      {
+        einfo->info.rotation = ee->rotation;
+        einfo->info.destination_alpha = ee->alpha;
         einfo->info.rotation = ee->rotation;
         einfo->info.debug = EINA_FALSE;
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
@@ -338,21 +384,10 @@ _ecore_evas_wl_shutdown(void)
 static void 
 _ecore_evas_wl_pre_free(Ecore_Evas *ee)
 {
-   /* Evas_Engine_Info_Wayland_Shm *einfo; */
-
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
+   if (!ee) return;
    if (ee->engine.wl.frame) evas_object_del(ee->engine.wl.frame);
-
-   /* FIXME: remove the shm pool */
-   /* einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas); */
-   /* if ((einfo) && (einfo->info.dest)) */
-   /*   { */
-   /*      int ret = 0; */
-
-   /*      ret = munmap(einfo->info.dest, ((ee->w * sizeof(int)) * ee->h)); */
-   /*      if (!ret) ERR("Failed to unmap engine destination: %m"); */
-   /*   } */
 }
 
 static void 
@@ -445,6 +480,7 @@ _ecore_evas_wl_move(Ecore_Evas *ee, int x, int y)
 
    ee->req.x = x;
    ee->req.y = y;
+
    if ((ee->x != x) || (ee->y != y))
      {
         ee->x = x;
@@ -458,20 +494,31 @@ _ecore_evas_wl_move(Ecore_Evas *ee, int x, int y)
 static void 
 _ecore_evas_wl_resize(Ecore_Evas *ee, int w, int h)
 {
+   Evas_Engine_Info_Wayland_Shm *einfo;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
    if (!ee) return;
    if (w < 1) w = 1;
    if (h < 1) h = 1;
 
-   LOGFN(__FILE__, __LINE__, __FUNCTION__);
-
-   if (ee->prop.min.w > w) w = ee->prop.min.w;
-   else if (w > ee->prop.max.w) w = ee->prop.max.w;
-   if (ee->prop.min.h > h) h = ee->prop.min.h;
-   else if (h > ee->prop.max.h) h = ee->prop.max.h;
-
    ee->req.w = w;
    ee->req.h = h;
 
+   if (!ee->prop.fullscreen)
+     {
+        int fw = 0, fh = 0;
+
+        if (ee->prop.min.w > w) w = ee->prop.min.w;
+        else if (w > ee->prop.max.w) w = ee->prop.max.w;
+        if (ee->prop.min.h > h) h = ee->prop.min.h;
+        else if (h > ee->prop.max.h) h = ee->prop.max.h;
+
+        evas_output_framespace_get(ee->evas, NULL, NULL, &fw, &fh);
+        w += fw;
+        h += fh;
+     }
+
    if ((ee->w != w) || (ee->h != h))
      {
         ee->w = w;
@@ -500,65 +547,38 @@ _ecore_evas_wl_resize(Ecore_Evas *ee, int w, int h)
         if (ee->engine.wl.frame)
           evas_object_resize(ee->engine.wl.frame, w, h);
 
-        if (ee->engine.wl.buffer) wl_buffer_destroy(ee->engine.wl.buffer);
-        ee->engine.wl.buffer = NULL;
-
-   Evas_Engine_Info_Wayland_Shm *einfo;
-   struct wl_shm_pool *pool = NULL;
-   int stride = 0;
-   size_t len = 0;
-   void *data;
-
-   stride = ee->w * sizeof(int);
-   len = stride * ee->h;
-
-   if ((ee->engine.wl.pool) && (len < ee->engine.wl.pool_size))
-     {
-        pool = ee->engine.wl.pool;
-        data = ee->engine.wl.pool_data;
-     }
-   else
-     {
-        int w = 0, size = 0;
-
-        ecore_wl_screen_size_get(&w, NULL);
-
-        if (w == 0) w = 1024;
-
-        size = (6 * w * w);
-        pool = 
-          _ecore_evas_wl_shm_pool_create(size, &data);
-        ee->engine.wl.pool_size = size;
-     }
-
-   if (data != ee->engine.wl.pool_data)
-     {
-        if (ee->engine.wl.pool)
-          wl_shm_pool_destroy(ee->engine.wl.pool);
-
-        ee->engine.wl.pool = pool;
-        ee->engine.wl.pool_data = data;
-     }
+        if (ee->engine.wl.buffer)
+          {
+             if(!ee->engine.wl.buffer_valid)
+               {
+                  wl_buffer_destroy(ee->engine.wl.buffer);
+               }
+             else
+               {
+                  ee->engine.wl.buffer_valid = EINA_FALSE;
+               }
+             ee->engine.wl.buffer = NULL;
+          }
 
+        _ecore_evas_wl_ensure_pool_size(ee, w, h);
 
         if (ee->engine.wl.pool)
           _ecore_evas_wl_buffer_new(ee, ee->engine.wl.pool);
 
-   einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas);
-   if (!einfo)
-     {
-        ERR("Failed to get Evas Engine Info for '%s'", ee->driver);
-        return;
-     }
+        einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas);
+        if (!einfo)
+          {
+            ERR("Failed to get Evas Engine Info for '%s'", ee->driver);
+            return;
+          }
 
-   einfo->info.dest = data;
-   evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
+        einfo->info.dest = ee->engine.wl.pool_data;
+        evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
 
         if (ee->engine.wl.win)
           {
-             ecore_wl_window_buffer_attach(ee->engine.wl.win, 
-                                           ee->engine.wl.buffer, 0, 0);
-             ecore_wl_window_update_size(ee->engine.wl.win, w, h);
+//             if (!ee->prop.fullscreen)
+               ecore_wl_window_update_size(ee->engine.wl.win, w, h);
           }
 
         if (ee->func.fn_resize) ee->func.fn_resize(ee);
@@ -566,51 +586,63 @@ _ecore_evas_wl_resize(Ecore_Evas *ee, int w, int h)
 }
 
 static void 
+_ecore_evas_wl_move_resize(Ecore_Evas *ee, int x, int y, int w, int h)
+{
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!ee) return;
+   if ((ee->x != x) || (ee->y != y))
+     _ecore_evas_wl_move(ee, x, y);
+   if ((ee->w != w) || (ee->h != h))
+     _ecore_evas_wl_resize(ee, w, h);
+}
+
+static void
+_ecore_evas_wl_ensure_pool_size(Ecore_Evas *ee, int w, int h)
+{
+   int stride = 0;
+   size_t len = 0;
+
+   stride = w * sizeof(int);
+   len = stride * h;
+
+   if ((ee->engine.wl.pool) && (len < ee->engine.wl.pool_size))
+     return;
+   else
+     {
+        struct wl_shm_pool *pool = NULL;
+        void *data;
+        int size;
+
+        if (ee->engine.wl.pool)
+          wl_shm_pool_destroy(ee->engine.wl.pool);
+
+        /*
+         * Make the pool 1.5 times the current requirement to allow growth
+         * without requiring a new pool allocation
+         */
+        size = 1.5 * len;
+        pool = _ecore_evas_wl_shm_pool_create(size, &data);
+
+        ee->engine.wl.pool = pool;
+        ee->engine.wl.pool_size = size;
+        ee->engine.wl.pool_data = data;
+     }
+}
+
+static void
 _ecore_evas_wl_show(Ecore_Evas *ee)
 {
    Evas_Engine_Info_Wayland_Shm *einfo;
-   struct wl_shm_pool *pool = NULL;
-   int stride = 0;
-   size_t len = 0;
-   void *data;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
    if ((!ee) || (ee->visible)) return;
 
-   stride = ee->w * sizeof(int);
-   len = stride * ee->h;
+   _ecore_evas_wl_ensure_pool_size(ee, ee->w, ee->h);
 
-   if ((ee->engine.wl.pool) && (len < ee->engine.wl.pool_size))
-     {
-        pool = ee->engine.wl.pool;
-        data = ee->engine.wl.pool_data;
-     }
-   else
-     {
-        int w = 0, size = 0;
-
-        ecore_wl_screen_size_get(&w, NULL);
-
-        /* set a default width */
-        if (w == 0) w = 1024;
-
-        size = (6 * w * w);
-        pool = 
-          _ecore_evas_wl_shm_pool_create(size, &data);
-        ee->engine.wl.pool_size = size;
-     }
-
-   if (data != ee->engine.wl.pool_data)
-     {
-        if (ee->engine.wl.pool)
-          wl_shm_pool_destroy(ee->engine.wl.pool);
-
-        ee->engine.wl.pool = pool;
-        ee->engine.wl.pool_data = data;
-     }
-
-   _ecore_evas_wl_buffer_new(ee, pool);
+   if (ee->engine.wl.pool)
+     _ecore_evas_wl_buffer_new(ee, ee->engine.wl.pool);
 
    einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas);
    if (!einfo)
@@ -619,7 +651,7 @@ _ecore_evas_wl_show(Ecore_Evas *ee)
         return;
      }
 
-   einfo->info.dest = data;
+   einfo->info.dest = ee->engine.wl.pool_data;
    evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo);
 
    /* ecore_wl_flush(); */
@@ -627,9 +659,17 @@ _ecore_evas_wl_show(Ecore_Evas *ee)
    if (ee->engine.wl.win)
      {
         ecore_wl_window_show(ee->engine.wl.win);
+        ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+        ee->engine.wl.buffer_valid = EINA_TRUE;
         ecore_wl_window_buffer_attach(ee->engine.wl.win, 
                                       ee->engine.wl.buffer, 0, 0);
-        ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+
+        if ((ee->prop.clas) && (ee->engine.wl.win->shell_surface))
+          wl_shell_surface_set_class(ee->engine.wl.win->shell_surface, 
+                                     ee->prop.clas);
+        if ((ee->prop.title) && (ee->engine.wl.win->shell_surface))
+          wl_shell_surface_set_title(ee->engine.wl.win->shell_surface, 
+                                     ee->prop.title);
      }
 
    if (ee->engine.wl.frame)
@@ -693,9 +733,13 @@ _ecore_evas_wl_title_set(Ecore_Evas *ee, const char *title)
      {
         EE_Wl_Smart_Data *sd;
 
-        if (!(sd = evas_object_smart_data_get(ee->engine.wl.frame))) return;
-        evas_object_text_text_set(sd->text, ee->prop.title);
+        if ((sd = evas_object_smart_data_get(ee->engine.wl.frame)))
+          evas_object_text_text_set(sd->text, ee->prop.title);
      }
+
+   if ((ee->prop.title) && (ee->engine.wl.win->shell_surface))
+     wl_shell_surface_set_title(ee->engine.wl.win->shell_surface, 
+                                ee->prop.title);
 }
 
 static void 
@@ -710,7 +754,10 @@ _ecore_evas_wl_name_class_set(Ecore_Evas *ee, const char *n, const char *c)
    ee->prop.clas = NULL;
    if (n) ee->prop.name = strdup(n);
    if (c) ee->prop.clas = strdup(c);
-   /* FIXME: Forward these changes to Wayland somehow */
+
+   if ((ee->prop.clas) && (ee->engine.wl.win->shell_surface))
+     wl_shell_surface_set_class(ee->engine.wl.win->shell_surface, 
+                                ee->prop.clas);
 }
 
 static void 
@@ -797,6 +844,8 @@ _ecore_evas_wl_maximized_set(Ecore_Evas *ee, int max)
    if (ee->prop.maximized == max) return;
    ee->prop.maximized = max;
    ecore_wl_window_maximized_set(ee->engine.wl.win, max);
+   if (ee->func.fn_state_change)
+     ee->func.fn_state_change(ee);
 }
 
 static void 
@@ -804,10 +853,12 @@ _ecore_evas_wl_fullscreen_set(Ecore_Evas *ee, int full)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
-   if ((!ee) || (!ee->visible)) return;
+   if (!ee) return;
    if (ee->prop.fullscreen == full) return;
    ee->prop.fullscreen = full;
    ecore_wl_window_fullscreen_set(ee->engine.wl.win, full);
+   if (ee->func.fn_state_change)
+     ee->func.fn_state_change(ee);
 }
 
 static void 
@@ -824,21 +875,47 @@ static void
 _ecore_evas_wl_alpha_set(Ecore_Evas *ee, int alpha)
 {
    Evas_Engine_Info_Wayland_Shm *einfo;
+   Ecore_Wl_Window *win;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
    if (!ee) return;
    if ((ee->alpha == alpha)) return;
    ee->alpha = alpha;
-   if (ee->engine.wl.win)
-     ecore_wl_window_transparent_set(ee->engine.wl.win, alpha);
+
+   /* FIXME: NB: We should really add a ecore_wl_window_alpha_set function
+    * but we are in API freeze, so just hack it in for now and fix when 
+    * freeze is over */
+   if ((win = ee->engine.wl.win))
+     win->alpha = alpha;
+
+   /* if (ee->engine.wl.win) */
+   /*   ecore_wl_window_transparent_set(ee->engine.wl.win, alpha); */
+
+   if (ee->engine.wl.buffer) wl_buffer_destroy(ee->engine.wl.buffer);
+   ee->engine.wl.buffer = NULL;
+
+   _ecore_evas_wl_ensure_pool_size(ee, ee->w, ee->h);
+
+   if (ee->engine.wl.pool)
+     _ecore_evas_wl_buffer_new(ee, ee->engine.wl.pool);
+
    if ((einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas)))
      {
         einfo->info.destination_alpha = alpha;
+        einfo->info.dest = ee->engine.wl.pool_data;
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
         evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
      }
+
+   if (ee->engine.wl.win)
+     {
+        ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+        ee->engine.wl.buffer_valid = EINA_TRUE;
+        ecore_wl_window_buffer_attach(ee->engine.wl.win, 
+                                      ee->engine.wl.buffer, 0, 0);
+     }
 }
 
 static void 
@@ -851,33 +928,75 @@ _ecore_evas_wl_transparent_set(Ecore_Evas *ee, int transparent)
    if (!ee) return;
    if ((ee->transparent == transparent)) return;
    ee->transparent = transparent;
+
    if (ee->engine.wl.win)
      ecore_wl_window_transparent_set(ee->engine.wl.win, transparent);
+
+   if (ee->engine.wl.buffer) wl_buffer_destroy(ee->engine.wl.buffer);
+   ee->engine.wl.buffer = NULL;
+
+   _ecore_evas_wl_ensure_pool_size(ee, ee->w, ee->h);
+
+   if (ee->engine.wl.pool)
+     _ecore_evas_wl_buffer_new(ee, ee->engine.wl.pool);
+
    if ((einfo = (Evas_Engine_Info_Wayland_Shm *)evas_engine_info_get(ee->evas)))
      {
         einfo->info.destination_alpha = transparent;
+        einfo->info.dest = ee->engine.wl.pool_data;
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
         evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
      }
+
+   if (ee->engine.wl.win)
+     {
+        ecore_wl_window_update_size(ee->engine.wl.win, ee->w, ee->h);
+        ee->engine.wl.buffer_valid = EINA_TRUE;
+        ecore_wl_window_buffer_attach(ee->engine.wl.win, 
+                                      ee->engine.wl.buffer, 0, 0);
+     }
+}
+
+static void
+_ecore_evas_wl_frame_complete(void *data, struct wl_callback *callback, uint32_t tm EINA_UNUSED)
+{
+   Ecore_Evas *ee = data;
+   Ecore_Wl_Window *win = NULL;
+
+   if (!ee) return;
+   if (!(win = ee->engine.wl.win)) return;
+
+   win->frame_callback = NULL;
+   win->frame_pending = EINA_FALSE;
+   wl_callback_destroy(callback);
+
+   if (win->surface)
+     {
+        win->frame_callback = wl_surface_frame(win->surface);
+        wl_callback_add_listener(win->frame_callback, &frame_listener, ee);
+     }
 }
 
 static int 
 _ecore_evas_wl_render(Ecore_Evas *ee)
 {
    int rend = 0;
+   Ecore_Wl_Window *win = NULL;
+
+   if (!(win = ee->engine.wl.win)) return 0;
 
    if (!ee) return 0;
    if (!ee->visible)
      evas_norender(ee->evas);
    else
      {
-        Eina_List *ll = NULL, *updates = NULL;
+        Eina_List *ll = NULL;
         Ecore_Evas *ee2 = NULL;
 
         if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
 
-        EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2) 
+        EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
           {
              if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
              if (ee2->engine.func->fn_render)
@@ -885,25 +1004,43 @@ _ecore_evas_wl_render(Ecore_Evas *ee)
              if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
           }
 
-        if ((updates = evas_render_updates(ee->evas))) 
+        if (!win->frame_pending)
           {
-             Eina_List *l = NULL;
-             Eina_Rectangle *r;
+             Eina_List *updates;
 
-             LOGFN(__FILE__, __LINE__, __FUNCTION__);
+             if (!win->frame_callback)
+               {
+                  win->frame_callback = wl_surface_frame(win->surface);
+                  wl_callback_add_listener(win->frame_callback,
+                                           &frame_listener, ee);
+               }
 
-             EINA_LIST_FOREACH(updates, l, r) 
-               ecore_wl_window_damage(ee->engine.wl.win, 
-                                      r->x, r->y, r->w, r->h);
+             if ((updates = evas_render_updates(ee->evas)))
+               {
+                  Eina_List *l = NULL;
+                  Eina_Rectangle *r;
 
-             ecore_wl_flush();
+                  LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
-             evas_render_updates_free(updates);
-             _ecore_evas_idle_timeout_update(ee);
-             rend = 1;
+                  ee->engine.wl.buffer_valid = EINA_TRUE;
+                  ecore_wl_window_buffer_attach(ee->engine.wl.win,
+                                                ee->engine.wl.buffer, 0, 0);
+                  EINA_LIST_FOREACH(updates, l, r)
+                     ecore_wl_window_damage(ee->engine.wl.win,
+                                            r->x, r->y, r->w, r->h);
+                  ecore_wl_window_commit(ee->engine.wl.win);
+                  ecore_wl_flush();
+
+                  evas_render_updates_free(updates);
+                  _ecore_evas_idle_timeout_update(ee);
+                  rend = 1;
+
+                  if (ee->func.fn_post_render) ee->func.fn_post_render(ee);
+
+                  win->frame_pending = EINA_TRUE;
+
+               }
           }
-
-        if (ee->func.fn_post_render) ee->func.fn_post_render(ee);
      }
    return rend;
 }
@@ -918,6 +1055,21 @@ _ecore_evas_wl_screen_geometry_get(const Ecore_Evas *ee __UNUSED__, int *x, int
    ecore_wl_screen_size_get(w, h);
 }
 
+static void 
+_ecore_evas_wl_screen_dpi_get(const Ecore_Evas *ee __UNUSED__, int *xdpi, int *ydpi)
+{
+   int dpi = 0;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (xdpi) *xdpi = 0;
+   if (ydpi) *ydpi = 0;
+   /* FIXME: Ideally this needs to get the DPI from a specific screen */
+   dpi = ecore_wl_dpi_get();
+   if (xdpi) *xdpi = dpi;
+   if (ydpi) *ydpi = dpi;
+}
+
 static struct wl_shm_pool *
 _ecore_evas_wl_shm_pool_create(int size, void **data)
 {
@@ -961,6 +1113,22 @@ _ecore_evas_wl_shm_pool_create(int size, void **data)
    return pool;
 }
 
+static void
+_ecore_evas_wl_buffer_release(void *data, struct wl_buffer *buffer)
+{
+   Ecore_Evas *ee = data;
+
+   if (ee->engine.wl.buffer == buffer)
+     ee->engine.wl.buffer_valid = EINA_FALSE;
+   else
+     wl_buffer_destroy(buffer);
+}
+
+static const struct wl_buffer_listener _buffer_listener_release =
+{
+   _ecore_evas_wl_buffer_release
+};
+
 static void 
 _ecore_evas_wl_buffer_new(Ecore_Evas *ee, struct wl_shm_pool *pool)
 {
@@ -974,8 +1142,11 @@ _ecore_evas_wl_buffer_new(Ecore_Evas *ee, struct wl_shm_pool *pool)
 
    stride = (ee->w * sizeof(int));
 
-   ee->engine.wl.buffer = 
+   ee->engine.wl.buffer =
      wl_shm_pool_create_buffer(pool, 0, ee->w, ee->h, stride, format);
+
+   wl_buffer_add_listener(ee->engine.wl.buffer,
+                          &_buffer_listener_release, ee);
 }
 
 void 
@@ -1016,14 +1187,13 @@ _ecore_evas_wl_cb_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *eve
    ee = ecore_event_window_match(ev->window);
    if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
    if (ev->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
-   if (!ee->in)
-     {
-        if (ee->func.fn_mouse_in) ee->func.fn_mouse_in(ee);
-        ecore_event_evas_modifier_lock_update(ee->evas, ev->modifiers);
-        evas_event_feed_mouse_in(ee->evas, ev->timestamp, NULL);
-        _ecore_evas_mouse_move_process(ee, ev->x, ev->y, ev->timestamp);
-        ee->in = EINA_TRUE;
-     }
+
+   if (ee->func.fn_mouse_in) ee->func.fn_mouse_in(ee);
+   ecore_event_evas_modifier_lock_update(ee->evas, ev->modifiers);
+   evas_event_feed_mouse_in(ee->evas, ev->timestamp, NULL);
+   _ecore_evas_mouse_move_process(ee, ev->x, ev->y, ev->timestamp);
+   ee->in = EINA_TRUE;
+
    return ECORE_CALLBACK_PASS_ON;
 }
 
@@ -1042,7 +1212,7 @@ _ecore_evas_wl_cb_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *ev
    if (ee->in)
      {
         ecore_event_evas_modifier_lock_update(ee->evas, ev->modifiers);
-        _ecore_evas_mouse_move_process(ee, ev->x, ev->y, ev->timestamp);
+//        _ecore_evas_mouse_move_process(ee, ev->x, ev->y, ev->timestamp);
         evas_event_feed_mouse_out(ee->evas, ev->timestamp, NULL);
         if (ee->func.fn_mouse_out) ee->func.fn_mouse_out(ee);
         if (ee->prop.cursor.object) evas_object_hide(ee->prop.cursor.object);
@@ -1092,6 +1262,7 @@ _ecore_evas_wl_cb_window_configure(void *data __UNUSED__, int type __UNUSED__, v
 {
    Ecore_Evas *ee;
    Ecore_Wl_Event_Window_Configure *ev;
+   int nw = 0, nh = 0;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
@@ -1100,20 +1271,42 @@ _ecore_evas_wl_cb_window_configure(void *data __UNUSED__, int type __UNUSED__, v
    if (!ee) return ECORE_CALLBACK_PASS_ON;
    if (ev->win != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
 
+   if (ee->prop.fullscreen)
+     {
+        _ecore_evas_wl_move(ee, ev->x, ev->y);
+        _ecore_evas_wl_resize(ee, ev->w, ev->h);
+
+        return ECORE_CALLBACK_PASS_ON;
+     }
+
    if ((ee->x != ev->x) || (ee->y != ev->y))
      {
-        /* ee->x = ev->x; */
-        /* ee->y = ev->y; */
         ee->req.x = ee->x;
         ee->req.y = ee->y;
         if (ee->func.fn_move) ee->func.fn_move(ee);
      }
-   if ((ee->req.w != ev->w) || (ee->req.h != ev->h))
+
+   nw = ev->w;
+   nh = ev->h;
+
+   if ((ee->prop.maximized) || (!ee->prop.fullscreen))
      {
-        /* ee->w = ev->w; */
-        /* ee->h = ev->h; */
-        ee->req.w = ev->w;
-        ee->req.h = ev->h;
+        int fw = 0, fh = 0;
+
+        evas_output_framespace_get(ee->evas, NULL, NULL, &fw, &fh);
+        nw = ev->w - fw;
+        nh = ev->h - fh;
+     }
+
+   if (ee->prop.min.w > nw) nw = ee->prop.min.w;
+   else if (nw > ee->prop.max.w) nw = ee->prop.max.w;
+   if (ee->prop.min.h > nh) nh = ee->prop.min.h;
+   else if (nh > ee->prop.max.h) nh = ee->prop.max.h;
+
+   if ((ee->w != nw) || (ee->h != nh))
+     {
+        ee->req.w = nw;
+        ee->req.h = nh;
         if (ee->func.fn_resize) ee->func.fn_resize(ee);
      }
 
@@ -1158,7 +1351,6 @@ _ecore_evas_wl_smart_add(Evas_Object *obj)
    sd->h = 1;
 
    sd->frame = evas_object_rectangle_add(evas);
-   evas_object_is_frame_object_set(sd->frame, EINA_TRUE);
    evas_object_color_set(sd->frame, 249, 249, 249, 255);
    evas_object_smart_member_add(sd->frame, obj);
 
@@ -1167,6 +1359,7 @@ _ecore_evas_wl_smart_add(Evas_Object *obj)
    evas_object_text_style_set(sd->text, EVAS_TEXT_STYLE_PLAIN);
    evas_object_text_font_set(sd->text, "Sans", 10);
    evas_object_text_text_set(sd->text, "Smart Test");
+   evas_object_smart_member_add(sd->text, obj);
 
    evas_object_smart_data_set(obj, sd);
 }
