@@ -3,8 +3,10 @@
 
 #include "Ecore.h"
 #include "ecore_private.h"
+#include "Ecore_Input.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <termios.h>
@@ -29,6 +31,8 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include <Ecore_Fb.h>
+
 /* ecore_fb_li.c */
 struct _Ecore_Fb_Input_Device
 {
@@ -49,10 +53,14 @@ struct _Ecore_Fb_Input_Device
                 double last;
                 double prev;
                 double threshold;
+                Eina_Bool did_double;
+                Eina_Bool did_triple;
                 /* absolute axis */
                 int min_w, min_h;
                 double rel_w, rel_h;
                 int event;
+                int prev_button;
+                int last_button;
         } mouse;
         struct
         {
@@ -61,11 +69,15 @@ struct _Ecore_Fb_Input_Device
                 int alt;
                 int lock;
         } keyboard;
+        void *window;
 };
 
 /* ecore_fb_ts.c */
 EAPI int    ecore_fb_ts_init(void);
 EAPI void   ecore_fb_ts_shutdown(void);
+EAPI void   ecore_fb_ts_events_window_set(void *window);
+EAPI void  *ecore_fb_ts_events_window_get(void);
+EAPI void   ecore_fb_ts_event_window_set(void *window);
 
 /* ecore_fb_vt.c */
 int  ecore_fb_vt_init(void);
