@@ -22,7 +22,7 @@ struct _Ecore_Fb_Ts_Event
    unsigned short pressure;
    unsigned short x;
    unsigned short y;
-   unsigned short _unused; 
+   unsigned short _unused;
 };
 
 struct _Ecore_Fb_Ts_Calibrate
@@ -74,6 +74,7 @@ struct ts_sample _ecore_fb_tslib_event;
 #endif
 
 static double _ecore_fb_double_click_time = 0.25;
+static void *_ecore_fb_ts_event_window = NULL;
 
 EAPI int
 ecore_fb_ts_init(void)
@@ -84,13 +85,13 @@ ecore_fb_ts_init(void)
      {
         printf( "ECORE_FB: TSLIB_TSDEVICE = '%s'\n", tslib_tsdevice );
         _ecore_fb_tslib_tsdev = ts_open( tslib_tsdevice, 1 ); /* 1 = nonblocking, 0 = blocking */
-        
+
         if ( !_ecore_fb_tslib_tsdev )
           {
              printf( "ECORE_FB: Can't ts_open (%s)\n", strerror( errno ) );
              return 0;
           }
-        
+
         if ( ts_config( _ecore_fb_tslib_tsdev ) )
           {
              printf( "ECORE_FB: Can't ts_config (%s)\n", strerror( errno ) );
@@ -108,7 +109,7 @@ ecore_fb_ts_init(void)
 #endif
    if (_ecore_fb_ts_fd >= 0)
      {
-        _ecore_fb_ts_fd_handler_handle = ecore_main_fd_handler_add(_ecore_fb_ts_fd, 
+        _ecore_fb_ts_fd_handler_handle = ecore_main_fd_handler_add(_ecore_fb_ts_fd,
                                                                    ECORE_FD_READ,
                                                                    _ecore_fb_ts_fd_handler, NULL,
                                                                    NULL, NULL);
@@ -131,6 +132,19 @@ ecore_fb_ts_shutdown(void)
    if (_ecore_fb_ts_fd >= 0) close(_ecore_fb_ts_fd);
    _ecore_fb_ts_fd = -1;
    _ecore_fb_ts_fd_handler_handle = NULL;
+   _ecore_fb_ts_event_window = NULL;
+}
+
+EAPI void
+ecore_fb_ts_event_window_set(void *window)
+{
+   _ecore_fb_ts_event_window = window;
+}
+
+EAPI void *
+ecore_fb_ts_event_window_get(void)
+{
+    return _ecore_fb_ts_event_window;
 }
 
 /**
@@ -153,7 +167,7 @@ EAPI void
 ecore_fb_touch_screen_calibrate_set(int xscale, int xtrans, int yscale, int ytrans, int xyswap)
 {
    Ecore_Fb_Ts_Calibrate cal;
-   
+
    if (_ecore_fb_ts_fd < 0) return;
    cal.xscale = xscale;
    cal.xtrans = xtrans;
@@ -181,7 +195,7 @@ EAPI void
 ecore_fb_touch_screen_calibrate_get(int *xscale, int *xtrans, int *yscale, int *ytrans, int *xyswap)
 {
    Ecore_Fb_Ts_Calibrate cal;
-   
+
    if (_ecore_fb_ts_fd < 0) return;
    if (!_ecore_fb_ts_apply_cal)
      {
@@ -204,7 +218,7 @@ _ecore_fb_ts_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UN
    static double last_time = 0;
    static double last_last_time = 0;
    int v = 0;
-   
+
    do
      {
         int x, y, pressure;
@@ -224,7 +238,6 @@ _ecore_fb_ts_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UN
         y = _ecore_fb_tslib_event.y;
         pressure = _ecore_fb_tslib_event.pressure;
         v = 1; /* loop, there might be more samples */
-        t = ecore_time_get();
 #else
         ptr = (char *)&(_ecore_fb_ts_event);
         ptr += _ecore_fb_ts_event_byte_count;
@@ -245,30 +258,40 @@ _ecore_fb_ts_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UN
              y = _ecore_fb_ts_event.y;
           }
         pressure = _ecore_fb_ts_event.pressure;
-#endif   
+#endif
+        t = ecore_loop_time_get();
         /* add event to queue */
         /* always add a move event */
         if ((pressure) || (prev_pressure))
           {
              /* MOVE: mouse is down and was */
-             Ecore_Fb_Event_Mouse_Move *e;
-             
-             e = calloc(1, sizeof(Ecore_Fb_Event_Mouse_Move));
+             Ecore_Event_Mouse_Move *e;
+
+             e = calloc(1, sizeof(Ecore_Event_Mouse_Move));
              if (!e) goto retry;
              e->x = x;
              e->y = y;
-             ecore_event_add(ECORE_FB_EVENT_MOUSE_MOVE, e, NULL, NULL);
+             e->root.x = e->x;
+             e->root.y = e->y;
+             e->window = 1;
+             e->event_window = e->window;
+             e->root_window = e->window;
+             e->same_screen = 1;
+             e->timestamp = ecore_loop_time_get() * 1000.0;
+             ecore_event_add(ECORE_EVENT_MOUSE_MOVE, e, NULL, NULL);
           }
         if ((pressure) && (!prev_pressure))
           {
              /* DOWN: mouse is down, but was not now */
-             Ecore_Fb_Event_Mouse_Button_Down *e;
-             
-             e = calloc(1, sizeof(Ecore_Fb_Event_Mouse_Button_Down));
+             Ecore_Event_Mouse_Button *e;
+
+             e = calloc(1, sizeof(Ecore_Event_Mouse_Button));
              if (!e) goto retry;
              e->x = x;
              e->y = y;
-             e->button = 1;
+             e->root.x = e->x;
+             e->root.y = e->y;
+             e->buttons = 1;
              if ((t - last_time) <= _ecore_fb_double_click_time)
                {
                   e->double_click = 1;
@@ -288,23 +311,35 @@ _ecore_fb_ts_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UN
                {
                   did_triple = 0;
                }
-             ecore_event_add(ECORE_FB_EVENT_MOUSE_BUTTON_DOWN, e, NULL, NULL);
+             e->window = 1;
+             e->event_window = e->window;
+             e->root_window = e->window;
+             e->same_screen = 1;
+             e->timestamp = ecore_loop_time_get() * 1000.0;
+             ecore_event_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, e, NULL, NULL);
           }
         else if ((!pressure) && (prev_pressure))
           {
              /* UP: mouse was down, but is not now */
-             Ecore_Fb_Event_Mouse_Button_Up *e;
-             
-             e = calloc(1, sizeof(Ecore_Fb_Event_Mouse_Button_Up));
+             Ecore_Event_Mouse_Button *e;
+
+             e = calloc(1, sizeof(Ecore_Event_Mouse_Button));
              if (!e) goto retry;
              e->x = prev_x;
              e->y = prev_y;
-             e->button = 1;
+             e->root.x = e->x;
+             e->root.y = e->y;
+             e->buttons = 1;
              if (did_double)
                 e->double_click = 1;
              if (did_triple)
                 e->triple_click = 1;
-             ecore_event_add(ECORE_FB_EVENT_MOUSE_BUTTON_UP, e, NULL, NULL);
+             e->window = 1;
+             e->event_window = e->window;
+             e->root_window = e->window;
+             e->same_screen = 1;
+             e->timestamp = ecore_loop_time_get() * 1000.0;
+             ecore_event_add(ECORE_EVENT_MOUSE_BUTTON_UP, e, NULL, NULL);
           }
         if (did_triple)
           {
@@ -316,7 +351,7 @@ _ecore_fb_ts_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UN
              last_last_time = last_time;
              last_time = t;
           }
-retry:             
+retry:
         prev_x = x;
         prev_y = y;
         prev_pressure = pressure;
