@@ -2,6 +2,32 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -404,7 +430,7 @@ ecore_file_recursive_rm(const char *dir)
    struct stat st;
    int ret;
 
-   if (readlink(dir, buf, sizeof(buf)) > 0)
+   if (readlink(dir, buf, sizeof(buf) - 1) > 0)
      return ecore_file_unlink(dir);
 
    ret = stat(dir, &st);
@@ -1017,19 +1043,41 @@ ecore_file_escape_name(const char *filename)
      {
         if ((q - buf) > (PATH_MAX - 6)) return NULL;
         if (
-            (*p == ' ') || (*p == '\t') || (*p == '\n') ||
-            (*p == '\\') || (*p == '\'') || (*p == '\"') ||
-            (*p == ';') || (*p == '!') || (*p == '#') ||
-            (*p == '$') || (*p == '%') || (*p == '&') ||
-            (*p == '*') || (*p == '(') || (*p == ')') ||
-            (*p == '[') || (*p == ']') || (*p == '{') ||
-            (*p == '}') || (*p == '|') || (*p == '<') ||
-            (*p == '>') || (*p == '?')
+            (*p == ' ') || (*p == '\\') || (*p == '\'') ||
+            (*p == '\"') || (*p == ';') || (*p == '!') ||
+            (*p == '#') || (*p == '$') || (*p == '%') ||
+            (*p == '&') || (*p == '*') || (*p == '(') ||
+            (*p == ')') || (*p == '[') || (*p == ']') ||
+            (*p == '{') || (*p == '}') || (*p == '|') ||
+            (*p == '<') || (*p == '>') || (*p == '?')
             )
           {
              *q = '\\';
              q++;
           }
+        else if (*p == '\t')
+          {
+             *q = '\\';
+             q++;
+             *q = '\\';
+             q++;
+             *q = 't';
+             q++;
+             p++;
+             continue;
+          }
+        else if (*p == '\n')
+          {
+            *q = '\\';
+            q++;
+            *q = '\\';
+            q++;
+            *q = 'n';
+            q++;
+            p++;
+	    continue;
+          }
+
         *q = *p;
         q++;
         p++;
