@@ -1,10 +1,6 @@
 #ifndef ECORE_FILE_PRIVATE_H_
 #define ECORE_FILE_PRIVATE_H_
 
-#ifndef _FILE_OFFSET_BITS
-# define _FILE_OFFSET_BITS  64
-#endif
-
 #ifdef __linux__
 # include <features.h>
 #endif
@@ -13,9 +9,12 @@
 # include <Evil.h>
 #endif
 
+#ifdef HAVE_ESCAPE
+# include <Escape.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dirent.h>
 
 #include "Ecore.h"
 #include "ecore_private.h"
