@@ -8,6 +8,10 @@
  * @defgroup Ecore_IMF_Evas_Group Ecore Input Method Context Evas Helper Functions
  *
  * Helper functions to make it easy to use Evas with Ecore_IMF.
+ * Converts each event from Evas to the corresponding event of Ecore_IMF.
+ *
+ * An example of usage of these functions can be found at:
+ * @li @ref ecore_imf_example_c
  */
 
 static const char *_ecore_imf_evas_event_empty = "";
@@ -29,6 +33,8 @@ _ecore_imf_evas_event_modifiers_wrap(Evas_Modifier *evas_modifiers,
      *imf_keyboard_modifiers |= ECORE_IMF_KEYBOARD_MODIFIER_SHIFT;
    if (evas_key_modifier_is_set(evas_modifiers, "Super") || evas_key_modifier_is_set(evas_modifiers, "Hyper"))
      *imf_keyboard_modifiers |= ECORE_IMF_KEYBOARD_MODIFIER_WIN;
+   if (evas_key_modifier_is_set(evas_modifiers, "AltGr"))
+     *imf_keyboard_modifiers |= ECORE_IMF_KEYBOARD_MODIFIER_ALTGR;
 }
 
 /* Converts the Evas locks to Ecore_IMF keyboard locks */
