@@ -2,6 +2,32 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #include <string.h>
 
 #ifdef HAVE_NETINET_IN_H
@@ -412,12 +438,16 @@ ecore_ipc_server_connect(Ecore_Ipc_Type compl_type, char *name, int port, const
    Ecore_Ipc_Server *svr;
    Ecore_Ipc_Type type;
    Ecore_Con_Type extra = 0;
+   int features;
 
    svr = calloc(1, sizeof(Ecore_Ipc_Server));
    if (!svr) return NULL;
-   type = compl_type;
-   type &= ~ECORE_IPC_USE_SSL;
-   if (compl_type & ECORE_IPC_USE_SSL) extra = ECORE_CON_USE_SSL;
+   type = compl_type & ECORE_IPC_TYPE;
+   features = compl_type & ECORE_IPC_SSL;
+   if ((features & ECORE_IPC_USE_SSL) == ECORE_IPC_USE_SSL)
+     extra |= ECORE_CON_USE_SSL;
+   if ((features & ECORE_IPC_NO_PROXY) == ECORE_IPC_NO_PROXY)
+     extra |= ECORE_CON_NO_PROXY;
    switch (type)
      {
       case ECORE_IPC_LOCAL_USER:
@@ -473,7 +503,7 @@ ecore_ipc_server_del(Ecore_Ipc_Server *svr)
 
         EINA_LIST_FREE(svr->clients, cl)
           ecore_ipc_client_del(cl);
-        ecore_con_server_del(svr->server);
+        if (svr->server) ecore_con_server_del(svr->server);
         servers = eina_list_remove(servers, svr);
 
         if (svr->buf) free(svr->buf);
@@ -534,7 +564,7 @@ ecore_ipc_server_clients_get(Ecore_Ipc_Server *svr)
                          "ecore_ipc_server_clients_get");
         return NULL;
      }
-   return svr->client_list;
+   return svr->clients;
 }
 
 #define SVENC(_member) \
@@ -810,6 +840,8 @@ ecore_ipc_client_send(Ecore_Ipc_Client *cl, int major, int minor, int ref, int r
                          "ecore_ipc_client_send");
         return 0;
      }
+   EINA_SAFETY_ON_TRUE_RETURN_VAL(!cl->client, 0);
+   EINA_SAFETY_ON_TRUE_RETURN_VAL(!ecore_con_client_connected_get(cl->client), 0);
    if (size < 0) size = 0;
    msg.major    = major;
    msg.minor    = minor;
@@ -853,7 +885,7 @@ ecore_ipc_client_server_get(Ecore_Ipc_Client *cl)
                          "ecore_ipc_client_server_get");
         return NULL;
      }
-   return (ecore_con_server_data_get(ecore_con_client_server_get(cl->client)));
+   return cl->svr;
 }
 
 /**
@@ -880,8 +912,8 @@ ecore_ipc_client_del(Ecore_Ipc_Client *cl)
    cl->delete_me = 1;
    if (cl->event_count == 0)
      {
-        svr = ecore_con_server_data_get(ecore_con_client_server_get(cl->client));
-        ecore_con_client_del(cl->client);
+        svr = cl->svr;
+        if (cl->client) ecore_con_client_del(cl->client);
         svr->clients = eina_list_remove(svr->clients, cl);
         if (cl->buf) free(cl->buf);
         ECORE_MAGIC_SET(cl, ECORE_MAGIC_NONE);
@@ -1021,23 +1053,23 @@ static Eina_Bool
 _ecore_ipc_event_client_add(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Add *e;
+   Ecore_Ipc_Server *svr;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
+   svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
+   if (!eina_list_data_find(servers, svr)) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
         Ecore_Ipc_Client *cl;
-        Ecore_Ipc_Server *svr;
 
         cl = calloc(1, sizeof(Ecore_Ipc_Client));
         if (!cl) return ECORE_CALLBACK_CANCEL;
-        svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
+        cl->svr = svr;
         ECORE_MAGIC_SET(cl, ECORE_MAGIC_IPC_CLIENT);
         cl->client = e->client;
         cl->max_buf_size = 32 * 1024;
         ecore_con_client_data_set(cl->client, (void *)cl);
         svr->clients = eina_list_append(svr->clients, cl);
-        svr->client_list = eina_list_append(svr->client_list, cl);
         if (!cl->delete_me)
           {
              Ecore_Ipc_Event_Client_Add *e2;
@@ -1059,20 +1091,21 @@ static Eina_Bool
 _ecore_ipc_event_client_del(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Del *e;
+   Ecore_Ipc_Server *svr;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
+   if (!e->client) return ECORE_CALLBACK_RENEW;
+   svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
+   if (!eina_list_data_find(servers, svr)) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
         Ecore_Ipc_Client *cl;
 
         cl = ecore_con_client_data_get(e->client);
+        cl->client = NULL;
           {
              Ecore_Ipc_Event_Client_Del *e2;
-             Ecore_Ipc_Server *svr;
 
-             svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
-             svr->client_list = eina_list_remove(svr->client_list, cl);
              if (!cl->delete_me)
                {
                   e2 = calloc(1, sizeof(Ecore_Ipc_Event_Client_Del));
@@ -1130,6 +1163,7 @@ _ecore_ipc_event_server_del(void *data __UNUSED__, int ev_type __UNUSED__, void
         Ecore_Ipc_Server *svr;
 
         svr = ecore_con_server_data_get(e->server);
+        svr->server = NULL;
         if (!svr->delete_me)
           {
              Ecore_Ipc_Event_Server_Del *e2;
@@ -1192,9 +1226,11 @@ static Eina_Bool
 _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Data *e;
+   Ecore_Ipc_Server *svr;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
+   svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
+   if (!eina_list_data_find(servers, svr)) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
         Ecore_Ipc_Client *cl;
@@ -1263,11 +1299,9 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
              if ((cl->buf_size - offset) >= (s + msg.size))
                {
                   Ecore_Ipc_Event_Client_Data *e2;
-                  Ecore_Ipc_Server *svr;
                   int max, max2;
 
                   buf = NULL;
-                  svr = ecore_con_server_data_get(ecore_con_client_server_get(cl->client));
                   max = svr->max_buf_size;
                   max2 = cl->max_buf_size;
                   if ((max >= 0) && (max2 >= 0))
