@@ -2,18 +2,53 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #include "ecore_wl_private.h"
 
 /* local function prototypes */
 static void _ecore_wl_window_cb_ping(void *data __UNUSED__, struct wl_shell_surface *shell_surface, unsigned int serial);
 static void _ecore_wl_window_cb_configure(void *data, struct wl_shell_surface *shell_surface __UNUSED__, unsigned int edges, int w, int h);
 static void _ecore_wl_window_cb_popup_done(void *data, struct wl_shell_surface *shell_surface __UNUSED__);
-static void _ecore_wl_window_configure_send(Ecore_Wl_Window *win, int w, int h, unsigned int timestamp);
+static void _ecore_wl_window_cb_surface_enter(void *data, struct wl_surface *surface, struct wl_output *output __UNUSED__);
+static void _ecore_wl_window_cb_surface_leave(void *data, struct wl_surface *surface, struct wl_output *output __UNUSED__);
+static void _ecore_wl_window_configure_send(Ecore_Wl_Window *win, int w, int h);
+static char *_ecore_wl_window_id_str_get(unsigned int win_id);
 
 /* local variables */
 static Eina_Hash *_windows = NULL;
 
 /* wayland listeners */
+static const struct wl_surface_listener _ecore_wl_surface_listener = 
+{
+   _ecore_wl_window_cb_surface_enter,
+   _ecore_wl_window_cb_surface_leave
+};
+
 static const struct wl_shell_surface_listener _ecore_wl_shell_surface_listener = 
 {
    _ecore_wl_window_cb_ping,
@@ -25,7 +60,8 @@ static const struct wl_shell_surface_listener _ecore_wl_shell_surface_listener =
 void 
 _ecore_wl_window_init(void)
 {
-   if (!_windows) _windows = eina_hash_pointer_new(free);
+   if (!_windows) 
+     _windows = eina_hash_string_superfast_new(NULL);
 }
 
 void 
@@ -81,11 +117,12 @@ ecore_wl_window_new(Ecore_Wl_Window *parent, int x, int y, int w, int h, int buf
    win->allocation.h = h;
    win->saved_allocation = win->allocation;
    win->transparent = EINA_FALSE;
-   win->type = ECORE_WL_WINDOW_TYPE_TOPLEVEL;
+   /* win->type = ECORE_WL_WINDOW_TYPE_TOPLEVEL; */
+   win->type = ECORE_WL_WINDOW_TYPE_NONE;
    win->buffer_type = buffer_type;
    win->id = _win_id++;
 
-   eina_hash_add(_windows, &win->id, win);
+   eina_hash_add(_windows, _ecore_wl_window_id_str_get(win->id), win);
    return win;
 }
 
@@ -106,7 +143,7 @@ ecore_wl_window_free(Ecore_Wl_Window *win)
 
    if (!win) return;
 
-   eina_hash_del(_windows, &win->id, NULL);
+   eina_hash_del(_windows, _ecore_wl_window_id_str_get(win->id), win);
 
    wl_list_for_each(input, &_ecore_wl_disp->inputs, link)
      {
@@ -126,7 +163,8 @@ ecore_wl_window_free(Ecore_Wl_Window *win)
    if (win->surface) wl_surface_destroy(win->surface);
    win->surface = NULL;
 
-//   free(win);
+   /* HMMM, why was this disabled ? */
+   free(win);
 }
 
 /**
@@ -165,9 +203,9 @@ ecore_wl_window_move(Ecore_Wl_Window *win, int x, int y)
                }
           }
 
-        if ((!input) || (!input->input_device)) return;
+        if ((!input) || (!input->seat)) return;
 
-        wl_shell_surface_move(win->shell_surface, input->input_device,
+        wl_shell_surface_move(win->shell_surface, input->seat,
                               input->display->serial);
      }
 }
@@ -193,11 +231,11 @@ ecore_wl_window_resize(Ecore_Wl_Window *win, int w, int h, int location)
 
    if (!win) return;
 
-   win->allocation.w = w;
-   win->allocation.h = h;
-
    if (win->type != ECORE_WL_WINDOW_TYPE_FULLSCREEN)
      {
+        win->allocation.w = w;
+        win->allocation.h = h;
+
         win->region.input = 
           wl_compositor_create_region(_ecore_wl_disp->wl.compositor);
         wl_region_add(win->region.input, win->allocation.x, win->allocation.y, 
@@ -225,23 +263,33 @@ ecore_wl_window_resize(Ecore_Wl_Window *win, int w, int h, int location)
                }
           }
 
-        if ((!input) || (!input->input_device)) return;
+        if ((!input) || (!input->seat)) return;
 
-        wl_shell_surface_resize(win->shell_surface, input->input_device, 
+        wl_shell_surface_resize(win->shell_surface, input->seat, 
                                 input->display->serial, location);
      }
 }
 
-EAPI void 
+EAPI void
 ecore_wl_window_damage(Ecore_Wl_Window *win, int x, int y, int w, int h)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
    if (!win) return;
-   if (win->surface) 
+   if (win->surface)
      wl_surface_damage(win->surface, x, y, w, h);
 }
 
+EAPI void
+ecore_wl_window_commit(Ecore_Wl_Window *win)
+{
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!win) return;
+   if (win->surface)
+     wl_surface_commit(win->surface);
+}
+
 EAPI void 
 ecore_wl_window_buffer_attach(Ecore_Wl_Window *win, struct wl_buffer *buffer, int x, int y)
 {
@@ -252,14 +300,29 @@ ecore_wl_window_buffer_attach(Ecore_Wl_Window *win, struct wl_buffer *buffer, in
    switch (win->buffer_type)
      {
       case ECORE_WL_WINDOW_BUFFER_TYPE_EGL_WINDOW:
-        /* FIXME: weston has wl_egl_window_get_attached_size */
+        win->server_allocation = win->allocation;
         break;
       case ECORE_WL_WINDOW_BUFFER_TYPE_EGL_IMAGE:
       case ECORE_WL_WINDOW_BUFFER_TYPE_SHM:
         if (win->surface)
           {
-             if (buffer)
-               wl_surface_attach(win->surface, buffer, x, y);
+             if (win->edges & 4) //  resizing from the left
+               x = win->server_allocation.w - win->allocation.w;
+             else
+               x = 0;
+
+             if (win->edges & 1) // resizing from the top
+               y = win->server_allocation.h - win->allocation.h;
+             else
+               y = 0;
+
+             win->edges = 0;
+
+             /* if (buffer) */
+             wl_surface_attach(win->surface, buffer, x, y);
+             wl_surface_damage(win->surface, 0, 0, 
+                               win->allocation.w, win->allocation.h);
+             wl_surface_commit(win->surface);
 
              win->server_allocation = win->allocation;
           }
@@ -268,10 +331,6 @@ ecore_wl_window_buffer_attach(Ecore_Wl_Window *win, struct wl_buffer *buffer, in
         return;
      }
 
-   if (win->surface)
-     wl_surface_damage(win->surface, 0, 0, 
-                       win->allocation.w, win->allocation.h);
-
    if (win->region.input)
      {
         wl_surface_set_input_region(win->surface, win->region.input);
@@ -307,6 +366,7 @@ ecore_wl_window_show(Ecore_Wl_Window *win)
 
    win->surface = wl_compositor_create_surface(_ecore_wl_disp->wl.compositor);
    wl_surface_set_user_data(win->surface, win);
+   /* wl_surface_add_listener(win->surface, &_ecore_wl_surface_listener, win); */
 
    win->shell_surface = 
      wl_shell_get_shell_surface(_ecore_wl_disp->wl.shell, win->surface);
@@ -325,18 +385,19 @@ ecore_wl_window_show(Ecore_Wl_Window *win)
         break;
       case ECORE_WL_WINDOW_TYPE_TRANSIENT:
         wl_shell_surface_set_transient(win->shell_surface, 
-                                       win->parent->shell_surface, 
+                                       win->parent->surface, 
                                        win->allocation.x, win->allocation.y, 0);
         break;
       case ECORE_WL_WINDOW_TYPE_MENU:
         wl_shell_surface_set_popup(win->shell_surface, 
-                                   _ecore_wl_disp->input->input_device,
-                                   _ecore_wl_disp->input->timestamp,
-                                   /* win->parent->pointer_device->input_device,  */
-                                   /* win->parent->pointer_device->timestamp, */ 
-                                   win->parent->shell_surface, 
+                                   _ecore_wl_disp->input->seat,
+                                   _ecore_wl_disp->serial,
+                                   win->parent->surface, 
                                    win->allocation.x, win->allocation.y, 0);
         break;
+      case ECORE_WL_WINDOW_TYPE_NONE:
+        win->type = ECORE_WL_WINDOW_TYPE_TOPLEVEL;
+        /* fallthrough */
       case ECORE_WL_WINDOW_TYPE_TOPLEVEL:
         wl_shell_surface_set_toplevel(win->shell_surface);
         break;
@@ -344,21 +405,21 @@ ecore_wl_window_show(Ecore_Wl_Window *win)
         break;
      }
 
-   if (win->type != ECORE_WL_WINDOW_TYPE_FULLSCREEN)
-     {
-        win->region.input = 
-          wl_compositor_create_region(_ecore_wl_disp->wl.compositor);
-        wl_region_add(win->region.input, win->allocation.x, win->allocation.y, 
-                      win->allocation.w, win->allocation.h);
-     }
+   /* if (win->type != ECORE_WL_WINDOW_TYPE_FULLSCREEN) */
+   /*   { */
+   /*      win->region.input =  */
+   /*        wl_compositor_create_region(_ecore_wl_disp->wl.compositor); */
+   /*      wl_region_add(win->region.input, win->allocation.x, win->allocation.y,  */
+   /*                    win->allocation.w, win->allocation.h); */
+   /*   } */
 
-   if (!win->transparent)
-     {
-        win->region.opaque = 
-          wl_compositor_create_region(_ecore_wl_disp->wl.compositor);
-        wl_region_add(win->region.opaque, win->allocation.x, win->allocation.y, 
-                      win->allocation.w, win->allocation.h);
-     }
+   /* if (!win->transparent) */
+   /*   { */
+   /*      win->region.opaque =  */
+   /*        wl_compositor_create_region(_ecore_wl_disp->wl.compositor); */
+   /*      wl_region_add(win->region.opaque, win->allocation.x, win->allocation.y,  */
+   /*                    win->allocation.w, win->allocation.h); */
+   /*   } */
 }
 
 /**
@@ -407,6 +468,7 @@ ecore_wl_window_maximized_set(Ecore_Wl_Window *win, Eina_Bool maximized)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
    if (!win) return;
+
    if ((win->type == ECORE_WL_WINDOW_TYPE_MAXIMIZED) == maximized) return;
    if (win->type == ECORE_WL_WINDOW_TYPE_TOPLEVEL)
      {
@@ -415,19 +477,15 @@ ecore_wl_window_maximized_set(Ecore_Wl_Window *win, Eina_Bool maximized)
           wl_shell_surface_set_maximized(win->shell_surface, NULL);
         win->type = ECORE_WL_WINDOW_TYPE_MAXIMIZED;
      }
-   else
+   else if (win->type == ECORE_WL_WINDOW_TYPE_MAXIMIZED)
      {
-        Ecore_Wl_Input *input;
-
-        input = win->keyboard_device;
-
         if (win->shell_surface) 
           wl_shell_surface_set_toplevel(win->shell_surface);
         win->type = ECORE_WL_WINDOW_TYPE_TOPLEVEL;
-        win->allocation = win->saved_allocation;
-        _ecore_wl_window_configure_send(win, win->allocation.w, 
-                                        win->allocation.h, input->timestamp);
+        _ecore_wl_window_configure_send(win, win->saved_allocation.w, 
+                                        win->saved_allocation.h);
      }
+   win->edges = 0;
 }
 
 EAPI void 
@@ -448,17 +506,13 @@ ecore_wl_window_fullscreen_set(Ecore_Wl_Window *win, Eina_Bool fullscreen)
      }
    else 
      {
-        Ecore_Wl_Input *input;
-
-        input = win->keyboard_device;
-
         if (win->shell_surface)
           wl_shell_surface_set_toplevel(win->shell_surface);
         win->type = ECORE_WL_WINDOW_TYPE_TOPLEVEL;
-        win->allocation = win->saved_allocation;
-        _ecore_wl_window_configure_send(win, win->allocation.w, 
-                                        win->allocation.h, input->timestamp);
+        _ecore_wl_window_configure_send(win, win->saved_allocation.w, 
+                                        win->saved_allocation.h);
      }
+   win->edges = 0;
 }
 
 EAPI void 
@@ -468,6 +522,15 @@ ecore_wl_window_transparent_set(Ecore_Wl_Window *win, Eina_Bool transparent)
 
    if (!win) return;
    win->transparent = transparent;
+   if (win->region.opaque) wl_region_destroy(win->region.opaque);
+   win->region.opaque = NULL;
+   if (!win->transparent)
+     {
+        win->region.opaque = 
+          wl_compositor_create_region(_ecore_wl_disp->wl.compositor);
+        wl_region_add(win->region.opaque, win->allocation.x, win->allocation.y, 
+                      win->allocation.w, win->allocation.h);
+     }
 }
 
 EAPI void 
@@ -512,12 +575,10 @@ ecore_wl_window_shell_surface_get(Ecore_Wl_Window *win)
 EAPI Ecore_Wl_Window *
 ecore_wl_window_find(unsigned int id)
 {
-   Ecore_Wl_Window *win;
+   Ecore_Wl_Window *win = NULL;
 
-   if (!id) return NULL;
-   win = eina_hash_find(_windows, &id);
-   if (win) return win;
-   return NULL;
+   win = eina_hash_find(_windows, _ecore_wl_window_id_str_get(id));
+   return win;
 }
 
 EAPI void 
@@ -530,7 +591,7 @@ ecore_wl_window_type_set(Ecore_Wl_Window *win, Ecore_Wl_Window_Type type)
 }
 
 EAPI void 
-ecore_wl_window_pointer_set(Ecore_Wl_Window *win, struct wl_buffer *buffer, int hot_x, int hot_y, unsigned int timestamp)
+ecore_wl_window_pointer_set(Ecore_Wl_Window *win, struct wl_surface *surface, int hot_x, int hot_y)
 {
    Ecore_Wl_Input *input;
 
@@ -538,9 +599,34 @@ ecore_wl_window_pointer_set(Ecore_Wl_Window *win, struct wl_buffer *buffer, int
 
    if (!win) return;
 
-   input = _ecore_wl_disp->input;
-   wl_input_device_attach(input->input_device, timestamp, 
-                          buffer, hot_x, hot_y);
+   if ((input = win->pointer_device))
+     ecore_wl_input_pointer_set(input, surface, hot_x, hot_y);
+}
+
+EAPI void
+ecore_wl_window_cursor_from_name_set(Ecore_Wl_Window *win, const char *cursor_name)
+{
+   Ecore_Wl_Input *input;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!win) return;
+
+   if ((input = win->pointer_device))
+     ecore_wl_input_cursor_from_name_set(input, cursor_name);
+}
+
+EAPI void
+ecore_wl_window_cursor_default_restore(Ecore_Wl_Window *win)
+{
+   Ecore_Wl_Input *input;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!win) return;
+
+   if ((input = win->pointer_device))
+     ecore_wl_input_cursor_default_restore(input);
 }
 
 /* @since 1.2 */
@@ -556,6 +642,7 @@ ecore_wl_window_parent_set(Ecore_Wl_Window *win, Ecore_Wl_Window *parent)
 static void 
 _ecore_wl_window_cb_ping(void *data __UNUSED__, struct wl_shell_surface *shell_surface, unsigned int serial)
 {
+   if (!shell_surface) return;
    wl_shell_surface_pong(shell_surface, serial);
 }
 
@@ -572,14 +659,14 @@ _ecore_wl_window_cb_configure(void *data, struct wl_shell_surface *shell_surface
 
    if ((win->allocation.w != w) || (win->allocation.h != h))
      {
-        win->edges = edges;
+        if (win->type == ECORE_WL_WINDOW_TYPE_TOPLEVEL)
+          win->edges = edges;
         if (win->region.input) wl_region_destroy(win->region.input);
         win->region.input = NULL;
         if (win->region.opaque) wl_region_destroy(win->region.opaque);
         win->region.opaque = NULL;
 
-        /* FIXME: 0 timestamp here may not work. need to test */
-        _ecore_wl_window_configure_send(win, w, h, 0);
+        _ecore_wl_window_configure_send(win, w, h);
      }
 }
 
@@ -590,12 +677,35 @@ _ecore_wl_window_cb_popup_done(void *data, struct wl_shell_surface *shell_surfac
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
+   if (!shell_surface) return;
    if (!(win = data)) return;
    ecore_wl_input_ungrab(win->pointer_device);
 }
 
 static void 
-_ecore_wl_window_configure_send(Ecore_Wl_Window *win, int w, int h, unsigned int timestamp)
+_ecore_wl_window_cb_surface_enter(void *data, struct wl_surface *surface, struct wl_output *output __UNUSED__)
+{
+   Ecore_Wl_Window *win;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!surface) return;
+   if (!(win = data)) return;
+}
+
+static void 
+_ecore_wl_window_cb_surface_leave(void *data, struct wl_surface *surface, struct wl_output *output __UNUSED__)
+{
+   Ecore_Wl_Window *win;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+
+   if (!surface) return;
+   if (!(win = data)) return;
+}
+
+static void 
+_ecore_wl_window_configure_send(Ecore_Wl_Window *win, int w, int h)
 {
    Ecore_Wl_Event_Window_Configure *ev;
 
@@ -608,6 +718,26 @@ _ecore_wl_window_configure_send(Ecore_Wl_Window *win, int w, int h, unsigned int
    ev->y = win->allocation.y;
    ev->w = w;
    ev->h = h;
-   ev->timestamp = timestamp;
    ecore_event_add(ECORE_WL_EVENT_WINDOW_CONFIGURE, ev, NULL, NULL);
 }
+
+static char *
+_ecore_wl_window_id_str_get(unsigned int win_id)
+{
+   const char *vals = "qWeRtYuIoP5$&<~";
+   static char id[9];
+   unsigned int val;
+
+   val = win_id;
+   id[0] = vals[(val >> 28) & 0xf];
+   id[1] = vals[(val >> 24) & 0xf];
+   id[2] = vals[(val >> 20) & 0xf];
+   id[3] = vals[(val >> 16) & 0xf];
+   id[4] = vals[(val >> 12) & 0xf];
+   id[5] = vals[(val >> 8) & 0xf];
+   id[6] = vals[(val >> 4) & 0xf];
+   id[7] = vals[(val) & 0xf];
+   id[8] = 0;
+
+   return id;
+}
