@@ -221,7 +221,7 @@ struct _Ecore_Win32_Event_Mouse_In
    Ecore_Win32_Window *window; /**< The window that received the event */
    int                 x; /**< The x coordinate where the mouse leaved */
    int                 y; /**< The y coordinate where the mouse entered */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -233,7 +233,7 @@ struct _Ecore_Win32_Event_Mouse_Out
    Ecore_Win32_Window *window; /**< The window that received the event */
    int                 x; /**< The x coordinate where the mouse leaved */
    int                 y; /**< The y coordinate where the mouse leaved */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -243,7 +243,7 @@ struct _Ecore_Win32_Event_Mouse_Out
 struct _Ecore_Win32_Event_Window_Focus_In
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long  long          time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -253,7 +253,7 @@ struct _Ecore_Win32_Event_Window_Focus_In
 struct _Ecore_Win32_Event_Window_Focus_Out
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -266,8 +266,8 @@ struct _Ecore_Win32_Event_Window_Damage
    int                 x; /**< The x coordinate of the top left corner of the damaged region */
    int                 y; /**< The y coordinate of the top left corner of the damaged region */
    int                 width; /**< The width of the damaged region */
-   int                 height; /**< The time the event occured */
-   long                time; /**< The time the event occured */
+   int                 height; /**< The time the event occurred */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -277,7 +277,7 @@ struct _Ecore_Win32_Event_Window_Damage
 struct _Ecore_Win32_Event_Window_Create
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -287,7 +287,7 @@ struct _Ecore_Win32_Event_Window_Create
 struct _Ecore_Win32_Event_Window_Destroy
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -297,7 +297,7 @@ struct _Ecore_Win32_Event_Window_Destroy
 struct _Ecore_Win32_Event_Window_Hide
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -307,7 +307,7 @@ struct _Ecore_Win32_Event_Window_Hide
 struct _Ecore_Win32_Event_Window_Show
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -322,7 +322,7 @@ struct _Ecore_Win32_Event_Window_Configure
    int                 y; /**< The new y coordinate of the top left corner */
    int                 width; /**< The new width */
    int                 height; /**< The new height */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -334,7 +334,7 @@ struct _Ecore_Win32_Event_Window_Resize
    Ecore_Win32_Window *window; /**< The window that received the event */
    int                 width; /**< The new width */
    int                 height; /**< The new height */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -344,7 +344,7 @@ struct _Ecore_Win32_Event_Window_Resize
 struct _Ecore_Win32_Event_Window_Delete_Request
 {
    Ecore_Win32_Window *window; /**< The window that received the event */
-   long                time; /**< The time the event occured */
+   unsigned long       timestamp; /**< The time the event occurred */
 };
 
 /**
@@ -357,24 +357,24 @@ EAPI extern int ECORE_WIN32_EVENT_MOUSE_IN; /**< Ecore_Event for the #Ecore_Win3
 EAPI extern int ECORE_WIN32_EVENT_MOUSE_OUT; /**< Ecore_Event for the #Ecore_Win32_Event_Mouse_Out event */
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_FOCUS_IN; /**< Ecore_Event for the #Ecore_Win32_Event_Window_Focus_In event */
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_FOCUS_OUT; /**< Ecore_Event for the #Ecore_Win32_Event_Window_Focus_Out event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_DAMAGE; /**< Ecore_Event for the #Ecore_Win32_Event_Damage event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_CREATE; /**< Ecore_Event for the #Ecore_Win32_Event_Create event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_DESTROY; /**< Ecore_Event for the #Ecore_Win32_Event_Destroy event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_HIDE; /**< Ecore_Event for the #Ecore_Win32_Event_Hide event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_SHOW; /**< Ecore_Event for the #Ecore_Win32_Event_Show event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_CONFIGURE; /**< Ecore_Event for the #Ecore_Win32_Event_Configure event */
-EAPI extern int ECORE_WIN32_EVENT_WINDOW_RESIZE; /**< Ecore_Event for the #Ecore_Win32_Event_Resize event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_DAMAGE; /**< Ecore_Event for the Ecore_Win32_Event_Damage event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_CREATE; /**< Ecore_Event for the Ecore_Win32_Event_Create event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_DESTROY; /**< Ecore_Event for the Ecore_Win32_Event_Destroy event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_HIDE; /**< Ecore_Event for the Ecore_Win32_Event_Hide event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_SHOW; /**< Ecore_Event for the Ecore_Win32_Event_Show event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_CONFIGURE; /**< Ecore_Event for the Ecore_Win32_Event_Configure event */
+EAPI extern int ECORE_WIN32_EVENT_WINDOW_RESIZE; /**< Ecore_Event for the Ecore_Win32_Event_Resize event */
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_DELETE_REQUEST; /**< Ecore_Event for the #Ecore_Win32_Event_Window_Delete_Request event */
 
 
 /* Core */
 
-EAPI int    ecore_win32_init();
-EAPI int    ecore_win32_shutdown();
-EAPI int    ecore_win32_screen_depth_get();
-EAPI void   ecore_win32_double_click_time_set(double t);
-EAPI double ecore_win32_double_click_time_get(void);
-EAPI long   ecore_win32_current_time_get(void);
+EAPI int           ecore_win32_init();
+EAPI int           ecore_win32_shutdown();
+EAPI int           ecore_win32_screen_depth_get();
+EAPI void          ecore_win32_double_click_time_set(double t);
+EAPI double        ecore_win32_double_click_time_get(void);
+EAPI unsigned long ecore_win32_current_time_get(void);
 
 /* Window */
 
@@ -460,7 +460,9 @@ EAPI void ecore_win32_window_lower(Ecore_Win32_Window *window);
 EAPI void ecore_win32_window_title_set(Ecore_Win32_Window *window,
                                        const char         *title);
 
-EAPI void ecore_win32_window_focus_set(Ecore_Win32_Window *window);
+EAPI void ecore_win32_window_focus(Ecore_Win32_Window *window);
+
+EAPI void *ecore_win32_window_focus_get(void);
 
 EAPI void ecore_win32_window_iconified_set(Ecore_Win32_Window *window,
                                            Eina_Bool           on);
@@ -471,11 +473,6 @@ EAPI void ecore_win32_window_borderless_set(Ecore_Win32_Window *window,
 EAPI void ecore_win32_window_fullscreen_set(Ecore_Win32_Window *window,
                                             Eina_Bool           on);
 
-EINA_DEPRECATED EAPI void ecore_win32_window_shape_set(Ecore_Win32_Window *window,
-                                                       unsigned short      width,
-                                                       unsigned short      height,
-                                                       unsigned char      *mask);
-
 EAPI void ecore_win32_window_cursor_set(Ecore_Win32_Window *window,
                                         Ecore_Win32_Cursor *cursor);
 
