@@ -2,6 +2,32 @@
 # include <config.h>
 #endif
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#elif !defined alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+# elif defined _AIX
+#  define alloca __alloca
+# elif defined _MSC_VER
+#  include <malloc.h>
+#  define alloca _alloca
+# elif !defined HAVE_ALLOCA
+#  ifdef  __cplusplus
+extern "C"
+#  endif
+void *alloca (size_t);
+# endif
+#endif
+
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #undef WIN32_LEAN_AND_MEAN
@@ -99,13 +125,13 @@ ecore_wince_window_new(Ecore_WinCE_Window *parent,
                        int                 width,
                        int                 height)
 {
-   struct _Ecore_WinCE_Window *w;
-   HWND                        window;
-   RECT                        rect;
+   Ecore_WinCE_Window *w;
+   HWND                window;
+   RECT                rect;
 
    INF("creating window");
 
-   w = (struct _Ecore_WinCE_Window *)calloc(1, sizeof(struct _Ecore_WinCE_Window));
+   w = (Ecore_WinCE_Window *)calloc(1, sizeof(Ecore_WinCE_Window));
    if (!w)
      {
         ERR("malloc() failed");
@@ -129,7 +155,7 @@ ecore_wince_window_new(Ecore_WinCE_Window *parent,
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,
                            x, y,
                            rect.right - rect.left, rect.bottom - rect.top,
-                           parent ? ((struct _Ecore_WinCE_Window *)parent)->window : NULL,
+                           parent ? ((Ecore_WinCE_Window *)parent)->window : NULL,
                            NULL, _ecore_wince_instance, NULL);
    if (!window)
      {
@@ -177,7 +203,7 @@ ecore_wince_window_free(Ecore_WinCE_Window *window)
 
    INF("destroying window");
 
-   DestroyWindow(((struct _Ecore_WinCE_Window *)window)->window);
+   DestroyWindow(window->window);
    free(window);
 }
 
@@ -195,7 +221,7 @@ ecore_wince_window_hwnd_get(Ecore_WinCE_Window *window)
    if (!window)
      return NULL;
 
-   return ((struct _Ecore_WinCE_Window *)window)->window;
+   return window->window;
 }
 
 /**
@@ -215,21 +241,19 @@ ecore_wince_window_move(Ecore_WinCE_Window *window,
                         int                 y)
 {
    RECT rect;
-   HWND w;
 
-   if (!window || ((struct _Ecore_WinCE_Window *)window)->fullscreen)
+   if (!window || window->fullscreen)
      return;
 
    INF("moving window (%dx%d)", x, y);
 
-   w = ((struct _Ecore_WinCE_Window *)window)->window;
-   if (!GetWindowRect(w, &rect))
+   if (!GetWindowRect(window->window, &rect))
      {
         ERR("GetWindowRect() failed");
         return;
      }
 
-   if (!MoveWindow(w, x, y,
+   if (!MoveWindow(window->window, x, y,
                    rect.right - rect.left,
                    rect.bottom - rect.top,
                    TRUE))
@@ -254,20 +278,18 @@ ecore_wince_window_resize(Ecore_WinCE_Window *window,
                           int                 width,
                           int                 height)
 {
-   RECT                        rect;
-   struct _Ecore_WinCE_Window *w;
-   DWORD                       style;
-   DWORD                       exstyle;
-   int                         x;
-   int                         y;
+   RECT  rect;
+   DWORD style;
+   DWORD exstyle;
+   int   x;
+   int   y;
 
-   if (!window || ((struct _Ecore_WinCE_Window *)window)->fullscreen)
+   if (!window || window->fullscreen)
      return;
 
    INF("resizing window (%dx%d)", width, height);
 
-   w = (struct _Ecore_WinCE_Window *)window;
-   if (!GetWindowRect(w->window, &rect))
+   if (!GetWindowRect(window->window, &rect))
      {
         ERR("GetWindowRect() failed");
         return;
@@ -279,12 +301,12 @@ ecore_wince_window_resize(Ecore_WinCE_Window *window,
    rect.top = 0;
    rect.right = width;
    rect.bottom = height;
-   if (!(style = GetWindowLong(w->window, GWL_STYLE)))
+   if (!(style = GetWindowLong(window->window, GWL_STYLE)))
      {
         ERR("GetWindowLong() failed");
         return;
      }
-   if (!(exstyle = GetWindowLong(w->window, GWL_EXSTYLE)))
+   if (!(exstyle = GetWindowLong(window->window, GWL_EXSTYLE)))
      {
         ERR("GetWindowLong() failed");
         return;
@@ -295,7 +317,7 @@ ecore_wince_window_resize(Ecore_WinCE_Window *window,
         return;
      }
 
-   if (!MoveWindow(w->window, x, y,
+   if (!MoveWindow(window->window, x, y,
                    rect.right - rect.left,
                    rect.bottom - rect.top,
                    FALSE))
@@ -324,27 +346,25 @@ ecore_wince_window_move_resize(Ecore_WinCE_Window *window,
                                int                 width,
                                int                 height)
 {
-   RECT                        rect;
-   struct _Ecore_WinCE_Window *w;
-   DWORD                       style;
-   DWORD                       exstyle;
+   RECT                rect;
+   DWORD               style;
+   DWORD               exstyle;
 
-   if (!window || ((struct _Ecore_WinCE_Window *)window)->fullscreen)
+   if (!window || window->fullscreen)
      return;
 
    INF("moving and resizing window (%dx%d %dx%d)", x, y, width, height);
 
-   w = ((struct _Ecore_WinCE_Window *)window);
    rect.left = 0;
    rect.top = 0;
    rect.right = width;
    rect.bottom = height;
-   if (!(style = GetWindowLong(w->window, GWL_STYLE)))
+   if (!(style = GetWindowLong(window->window, GWL_STYLE)))
      {
         ERR("GetWindowLong() failed");
         return;
      }
-   if (!(exstyle = GetWindowLong(w->window, GWL_EXSTYLE)))
+   if (!(exstyle = GetWindowLong(window->window, GWL_EXSTYLE)))
      {
         ERR("GetWindowLong() failed");
         return;
@@ -355,10 +375,10 @@ ecore_wince_window_move_resize(Ecore_WinCE_Window *window,
         return;
      }
 
-   if (!MoveWindow(w->window, x, y,
-              rect.right - rect.left,
-              rect.bottom - rect.top,
-              TRUE))
+   if (!MoveWindow(window->window, x, y,
+                   rect.right - rect.left,
+                   rect.bottom - rect.top,
+                   TRUE))
      {
         ERR("MoveWindow() failed");
      }
@@ -379,16 +399,16 @@ ecore_wince_window_show(Ecore_WinCE_Window *window)
 
    INF("showing window");
 
-   if (!ShowWindow(((struct _Ecore_WinCE_Window *)window)->window, SW_SHOWNORMAL))
+   if (!ShowWindow(window->window, SW_SHOWNORMAL))
      {
         ERR("ShowWindow() failed");
         return;
      }
-   if (!UpdateWindow(((struct _Ecore_WinCE_Window *)window)->window))
+   if (!UpdateWindow(window->window))
      {
         ERR("UpdateWindow() failed");
      }
-   if (!SendMessage(((struct _Ecore_WinCE_Window *)window)->window, WM_SHOWWINDOW, 1, 0))
+   if (!SendMessage(window->window, WM_SHOWWINDOW, 1, 0))
      {
         ERR("SendMessage() failed");
      }
@@ -409,12 +429,12 @@ ecore_wince_window_hide(Ecore_WinCE_Window *window)
 
    INF("hiding window");
 
-   if (!ShowWindow(((struct _Ecore_WinCE_Window *)window)->window, SW_HIDE))
+   if (!ShowWindow(window->window, SW_HIDE))
      {
         ERR("ShowWindow() failed");
         return;
      }
-   if (!SendMessage(((struct _Ecore_WinCE_Window *)window)->window, WM_SHOWWINDOW, 0, 0))
+   if (!SendMessage(window->window, WM_SHOWWINDOW, 0, 0))
      {
         ERR("SendMessage() failed");
      }
@@ -445,13 +465,65 @@ ecore_wince_window_title_set(Ecore_WinCE_Window *window,
    wtitle = evil_char_to_wchar(title);
    if (!wtitle) return;
 
-   if (!SetWindowText(((struct _Ecore_WinCE_Window *)window)->window, wtitle))
+   if (!SetWindowText(window->window, wtitle))
      {
         ERR("SetWindowText() failed");
      }
    free(wtitle);
 }
 
+/**
+ * @brief Set the focus to the given window.
+ *
+ * @param window The window to give focus to.
+ *
+ * This function gives the focus to @p window. If @p window is
+ * @c NULL, this function does nothing.
+ */
+EAPI void
+ecore_wince_window_focus(Ecore_WinCE_Window *window)
+{
+   if (!window) return;
+
+   INF("focusing window");
+
+   if (!SetFocus(window->window))
+     {
+        ERR("SetFocus() failed");
+     }
+}
+
+/**
+ * @brief Get the current focused window.
+ *
+ * @return The window that has focus.
+ *
+ * This function returns the window that has focus. If the calling
+ * thread's message queue does not have an associated window with the
+ * keyboard focus, the return value is @c NULL.
+ *
+ * @note Even if the returned value is @c NULL, another thread's queue
+ * may be associated with a window that has the keyboard focus.
+ *
+ * @note The returned value is of type HWND.
+ */
+EAPI void *
+ecore_wince_window_focus_get(void)
+{
+   HWND focused;
+
+   INF("getting focused window");
+
+   focused = GetFocus();
+   if (!focused)
+     {
+        ERR("GetFocus() failed");
+        return NULL;
+     }
+
+   return focused;
+}
+
 /**
  * @brief Set the graphic backend used for the given window.
  *
@@ -477,15 +549,12 @@ EAPI void
 ecore_wince_window_backend_set(Ecore_WinCE_Window *window,
                                int                 backend)
 {
-   struct _Ecore_WinCE_Window *w;
-
    if (!window)
      return;
 
    INF("setting backend");
 
-   w = (struct _Ecore_WinCE_Window *)window;
-   w->backend = backend;
+   window->backend = backend;
 }
 
 /**
@@ -503,15 +572,12 @@ ecore_wince_window_backend_set(Ecore_WinCE_Window *window,
 EAPI void
 ecore_wince_window_suspend_cb_set(Ecore_WinCE_Window *window, int (*suspend_cb)(int))
 {
-   struct _Ecore_WinCE_Window *w;
-
    if (!window)
      return;
 
    INF("setting suspend callback");
 
-   w = (struct _Ecore_WinCE_Window *)window;
-   w->suspend_cb = suspend_cb;
+   window->suspend_cb = suspend_cb;
 }
 
 /**
@@ -529,15 +595,12 @@ ecore_wince_window_suspend_cb_set(Ecore_WinCE_Window *window, int (*suspend_cb)(
 EAPI void
 ecore_wince_window_resume_cb_set(Ecore_WinCE_Window *window, int (*resume_cb)(int))
 {
-   struct _Ecore_WinCE_Window *w;
-
    if (!window)
      return;
 
    INF("setting resume callback");
 
-   w = (struct _Ecore_WinCE_Window *)window;
-   w->resume_cb = resume_cb;
+   window->resume_cb = resume_cb;
 }
 
 /**
@@ -579,8 +642,7 @@ ecore_wince_window_geometry_get(Ecore_WinCE_Window *window,
         return;
      }
 
-   if (!GetClientRect(((struct _Ecore_WinCE_Window *)window)->window,
-                      &rect))
+   if (!GetClientRect(window->window, &rect))
      {
         ERR("GetClientRect() failed");
 
@@ -595,8 +657,7 @@ ecore_wince_window_geometry_get(Ecore_WinCE_Window *window,
    w = rect.right - rect.left;
    h = rect.bottom - rect.top;
 
-   if (!GetWindowRect(((struct _Ecore_WinCE_Window *)window)->window,
-                      &rect))
+   if (!GetWindowRect(window->window, &rect))
      {
         ERR("GetWindowRect() failed");
 
@@ -645,8 +706,7 @@ ecore_wince_window_size_get(Ecore_WinCE_Window *window,
         return;
      }
 
-   if (!GetClientRect(((struct _Ecore_WinCE_Window *)window)->window,
-                      &rect))
+   if (!GetClientRect(window->window, &rect))
      {
         ERR("GetClientRect() failed");
 
@@ -662,38 +722,34 @@ ecore_wince_window_size_get(Ecore_WinCE_Window *window,
  * @brief Set the given window to fullscreen.
  *
  * @param window The window.
- * @param on EINA_TRUE for fullscreen mode, EINA_FALSE for windowed mode.
+ * @param on @c EINA_TRUE for fullscreen mode, @c EINA_FALSE for windowed mode.
  *
- * This function set @p window to fullscreen or windowed mode. If @p on
- * is set to EINA_TRUE, the window will be fullscreen, if it is set to
- * EINA_FALSE, it will be windowed. If @p window is @c NULL or if the
- * state does not change (like setting to fullscreen while the window
- * is already fullscreen), this function does nothing.
+ * This function set @p window to fullscreen or windowed mode. If @p on is set
+ * to @c EINA_TRUE, the window will be fullscreen, if it is set to
+ * @c EINA_FALSE, it will be windowed. If @p window is @c NULL or if the state
+ * does not change (like setting to fullscreen while the window is already
+ * fullscreen), this function does nothing.
  */
 EAPI void
 ecore_wince_window_fullscreen_set(Ecore_WinCE_Window *window,
                                   Eina_Bool           on)
 {
-   struct _Ecore_WinCE_Window *ew;
-   HWND                        w;
-   HWND                        task_bar;
+   HWND task_bar;
 
    if (!window) return;
 
-   ew = (struct _Ecore_WinCE_Window *)window;
-   if (((ew->fullscreen) && (on)) ||
-       ((!ew->fullscreen) && (!on)))
+   if (((window->fullscreen) && (on)) ||
+       ((!window->fullscreen) && (!on)))
      return;
 
    INF("setting fullscreen: %s", on ? "yes" : "no");
 
-   ew->fullscreen = !!on;
-   w = ew->window;
+   window->fullscreen = !!on;
 
    if (on)
      {
         /* save the position and size of the window */
-        if (!GetWindowRect(w, &ew->rect))
+        if (!GetWindowRect(window->window, &window->rect))
           {
              ERR("GetWindowRect() failed");
              return;
@@ -715,15 +771,16 @@ ecore_wince_window_fullscreen_set(Ecore_WinCE_Window *window,
           }
 
         /* style: visible + popup */
-        if (!SetWindowLong(w, GWL_STYLE, WS_POPUP | WS_VISIBLE))
+        if (!SetWindowLong(window->window, GWL_STYLE, WS_POPUP | WS_VISIBLE))
           {
              INF("SetWindowLong() failed");
           }
 
         /* resize window to fit the entire screen */
-        if (!SetWindowPos(w, HWND_TOPMOST,
+        if (!SetWindowPos(window->window, HWND_TOPMOST,
                           0, 0,
-                          GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
+                          GetSystemMetrics(SM_CXSCREEN),
+                          GetSystemMetrics(SM_CYSCREEN),
                           SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED))
           {
              INF("SetWindowPos() failed");
@@ -733,9 +790,10 @@ ecore_wince_window_fullscreen_set(Ecore_WinCE_Window *window,
          * Call MoveWindow with the correct size and force painting.
          * Note that UpdateWindow (forcing repainting) is not sufficient
          */
-        if (!MoveWindow(w,
+        if (!MoveWindow(window->window,
                         0, 0,
-                        GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
+                        GetSystemMetrics(SM_CXSCREEN),
+                        GetSystemMetrics(SM_CYSCREEN),
                         TRUE))
           {
              INF("MoveWindow() failed");
@@ -759,16 +817,16 @@ ecore_wince_window_fullscreen_set(Ecore_WinCE_Window *window,
           }
 
         /* style: visible + caption + sysmenu */
-        if (!SetWindowLong(w, GWL_STYLE, WS_CAPTION | WS_SYSMENU | WS_VISIBLE))
+        if (!SetWindowLong(window->window, GWL_STYLE, WS_CAPTION | WS_SYSMENU | WS_VISIBLE))
           {
              INF("SetWindowLong() failed");
           }
         /* restaure the position and size of the window */
-        if (!SetWindowPos(w, HWND_TOPMOST,
-                          ew->rect.left,
-                          ew->rect.top,
-                          ew->rect.right - ew->rect.left,
-                          ew->rect.bottom - ew->rect.top,
+        if (!SetWindowPos(window->window, HWND_TOPMOST,
+                          window->rect.left,
+                          window->rect.top,
+                          window->rect.right - window->rect.left,
+                          window->rect.bottom - window->rect.top,
                           SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED))
           {
              INF("SetWindowLong() failed");
@@ -778,11 +836,11 @@ ecore_wince_window_fullscreen_set(Ecore_WinCE_Window *window,
          * Call MoveWindow with the correct size and force painting.
          * Note that UpdateWindow (forcing repainting) is not sufficient
          */
-        if (!MoveWindow(w,
-                        ew->rect.left,
-                        ew->rect.top,
-                        ew->rect.right - ew->rect.left,
-                        ew->rect.bottom - ew->rect.top,
+        if (!MoveWindow(window->window,
+                        window->rect.left,
+                        window->rect.top,
+                        window->rect.right - window->rect.left,
+                        window->rect.bottom - window->rect.top,
                         TRUE))
           {
              INF("MoveWindow() failed");
