@@ -585,34 +585,26 @@ ecore_x_screen_is_composited(int screen)
    char buff[32];
    xcb_get_selection_owner_cookie_t ocookie;
    xcb_get_selection_owner_reply_t *oreply;
+   xcb_intern_atom_cookie_t acookie;
+   xcb_intern_atom_reply_t *areply;
    Ecore_X_Window win;
-   static Ecore_X_Atom atom = XCB_NONE;
+   Ecore_X_Atom atom;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    CHECK_XCB_CONN;
 
    snprintf(buff, sizeof(buff), "_NET_WM_CM_S%i", screen);
-
-   if (atom == XCB_NONE)
-     {
-        xcb_intern_atom_cookie_t acookie;
-        xcb_intern_atom_reply_t *areply;
-
-        acookie =
-          xcb_intern_atom_unchecked(_ecore_xcb_conn, 0, strlen(buff), buff);
-        areply = xcb_intern_atom_reply(_ecore_xcb_conn, acookie, NULL);
-        if (!areply) return EINA_FALSE;
-        atom = areply->atom;
-        free(areply);
-     }
+   acookie = xcb_intern_atom_unchecked(_ecore_xcb_conn, 1, strlen(buff), buff);
+   areply = xcb_intern_atom_reply(_ecore_xcb_conn, acookie, NULL);
+   if (!areply) return EINA_FALSE;
+   atom = areply->atom;
+   free(areply);
    if (atom == XCB_NONE) return EINA_FALSE;
-
    ocookie = xcb_get_selection_owner_unchecked(_ecore_xcb_conn, atom);
    oreply = xcb_get_selection_owner_reply(_ecore_xcb_conn, ocookie, NULL);
    if (!oreply) return EINA_FALSE;
    win = oreply->owner;
    free(oreply);
-
    return (win != XCB_NONE) ? EINA_TRUE : EINA_FALSE;
 }
 
@@ -620,26 +612,20 @@ EAPI void
 ecore_x_screen_is_composited_set(int            screen,
                                  Ecore_X_Window win)
 {
-   static Ecore_X_Atom atom = XCB_NONE;
+   Ecore_X_Atom atom;
+   xcb_intern_atom_cookie_t acookie;
+   xcb_intern_atom_reply_t *areply;
    char buff[32];
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    CHECK_XCB_CONN;
 
    snprintf(buff, sizeof(buff), "_NET_WM_CM_S%i", screen);
-   if (atom == XCB_NONE)
-     {
-        xcb_intern_atom_cookie_t acookie;
-        xcb_intern_atom_reply_t *areply;
-
-        acookie =
-          xcb_intern_atom_unchecked(_ecore_xcb_conn, 0, strlen(buff), buff);
-        areply = xcb_intern_atom_reply(_ecore_xcb_conn, acookie, NULL);
-        if (!areply) return;
-        atom = areply->atom;
-        free(areply);
-     }
-   if (atom == XCB_NONE) return;
+   acookie = xcb_intern_atom_unchecked(_ecore_xcb_conn, 0, strlen(buff), buff);
+   areply = xcb_intern_atom_reply(_ecore_xcb_conn, acookie, NULL);
+   if (!areply) return;
+   atom = areply->atom;
+   free(areply);
    xcb_set_selection_owner(_ecore_xcb_conn, win, atom,
                            _ecore_xcb_events_last_time_get());
 }
@@ -793,6 +779,35 @@ ecore_x_netwm_desktop_request_send(Ecore_X_Window win,
 //   ecore_x_flush();
 }
 
+EAPI void
+ecore_x_netwm_moveresize_request_send(Ecore_X_Window win,
+                                      int x,
+                                      int y,
+                                      Ecore_X_Netwm_Direction direction,
+                                      unsigned int button)
+{
+   xcb_client_message_event_t ev;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   CHECK_XCB_CONN;
+
+   memset(&ev, 0, sizeof(xcb_client_message_event_t));
+
+   ev.response_type = XCB_CLIENT_MESSAGE;
+   ev.format = 32;
+   ev.window = win;
+   ev.type = ECORE_X_ATOM_NET_WM_MOVERESIZE;
+   ev.data.data32[0] = x;
+   ev.data.data32[1] = y;
+   ev.data.data32[2] = direction;
+   ev.data.data32[3] = button;
+   ev.data.data32[4] = 1;
+
+   xcb_send_event(_ecore_xcb_conn, 0, win,
+                  (XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT |
+                   XCB_EVENT_MASK_SUBSTRUCTURE_NOTIFY), (const char *)&ev);
+}
+
 EAPI void
 ecore_x_netwm_handled_icons_set(Ecore_X_Window win)
 {
