@@ -21,29 +21,34 @@ _ecore_x_fixes_init(void)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XFixesQueryVersion(_ecore_x_disp, &_fixes_major, &_fixes_minor))
-      _fixes_available = 1;
+     {
+        _fixes_available = 1;
+
+        ECORE_X_EVENT_FIXES_SELECTION_NOTIFY = ecore_event_type_new();
+     }
    else
-      _fixes_available = 0;
+     _fixes_available = 0;
 
 #else /* ifdef ECORE_XFIXES */
    _fixes_available = 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* _ecore_x_fixes_init */
+}
 
 #ifdef ECORE_XFIXES
 /* I don't know what to call this function. */
 static XRectangle *
-_ecore_x_rectangle_ecore_to_x(Ecore_X_Rectangle *rects, int num)
+_ecore_x_rectangle_ecore_to_x(Ecore_X_Rectangle *rects,
+                              int num)
 {
    XRectangle *xrect;
    int i;
 
    if (num == 0)
-      return NULL;
+     return NULL;
 
    xrect = malloc(sizeof(XRectangle) * num);
    if (!xrect)
-      return NULL;
+     return NULL;
 
    for (i = 0; i < num; i++)
      {
@@ -53,20 +58,21 @@ _ecore_x_rectangle_ecore_to_x(Ecore_X_Rectangle *rects, int num)
         xrect[i].height = rects[i].height;
      }
    return xrect;
-} /* _ecore_x_rectangle_ecore_to_x */
+}
 
 static Ecore_X_Rectangle *
-_ecore_x_rectangle_x_to_ecore(XRectangle *xrect, int num)
+_ecore_x_rectangle_x_to_ecore(XRectangle *xrect,
+                              int num)
 {
    Ecore_X_Rectangle *rects;
    int i;
 
    if (num == 0)
-      return NULL;
+     return NULL;
 
    rects = malloc(sizeof(Ecore_X_Rectangle) * num);
    if (!rects)
-      return NULL;
+     return NULL;
 
    for (i = 0; i < num; i++)
      {
@@ -76,12 +82,31 @@ _ecore_x_rectangle_x_to_ecore(XRectangle *xrect, int num)
         rects[i].height = xrect[i].height;
      }
    return rects;
-} /* _ecore_x_rectangle_x_to_ecore */
+}
 
 #endif /* ifdef ECORE_XFIXES */
 
+EAPI Eina_Bool
+ecore_x_fixes_selection_notification_request(Ecore_X_Atom selection)
+{
+#ifdef ECORE_XFIXES
+   if (_fixes_available)
+     {
+        XFixesSelectSelectionInput (_ecore_x_disp,
+                                    DefaultRootWindow(_ecore_x_disp),
+                                    selection,
+                                    XFixesSetSelectionOwnerNotifyMask |
+                                    XFixesSelectionWindowDestroyNotifyMask |
+                                    XFixesSelectionClientCloseNotifyMask);
+        return EINA_TRUE;
+     }
+#endif
+   return EINA_FALSE;
+}
+
 EAPI Ecore_X_Region
-ecore_x_region_new(Ecore_X_Rectangle *rects, int num)
+ecore_x_region_new(Ecore_X_Rectangle *rects,
+                   int num)
 {
 #ifdef ECORE_XFIXES
    Ecore_X_Region region;
@@ -95,7 +120,7 @@ ecore_x_region_new(Ecore_X_Rectangle *rects, int num)
 #else /* ifdef ECORE_XFIXES */
    return 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_new */
+}
 
 EAPI Ecore_X_Region
 ecore_x_region_new_from_bitmap(Ecore_X_Pixmap bitmap)
@@ -109,10 +134,11 @@ ecore_x_region_new_from_bitmap(Ecore_X_Pixmap bitmap)
 #else /* ifdef ECORE_XFIXES */
    return 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_new_from_bitmap */
+}
 
 EAPI Ecore_X_Region
-ecore_x_region_new_from_window(Ecore_X_Window win, Ecore_X_Region_Type type)
+ecore_x_region_new_from_window(Ecore_X_Window win,
+                               Ecore_X_Region_Type type)
 {
 #ifdef ECORE_XFIXES
    Ecore_X_Region region;
@@ -123,7 +149,7 @@ ecore_x_region_new_from_window(Ecore_X_Window win, Ecore_X_Region_Type type)
 #else /* ifdef ECORE_XFIXES */
    return 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_new_from_window */
+}
 
 EAPI Ecore_X_Region
 ecore_x_region_new_from_gc(Ecore_X_GC gc)
@@ -137,7 +163,7 @@ ecore_x_region_new_from_gc(Ecore_X_GC gc)
 #else /* ifdef ECORE_XFIXES */
    return 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_new_from_gc */
+}
 
 EAPI Ecore_X_Region
 ecore_x_region_new_from_picture(Ecore_X_Picture picture)
@@ -151,7 +177,7 @@ ecore_x_region_new_from_picture(Ecore_X_Picture picture)
 #else /* ifdef ECORE_XFIXES */
    return 0;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_new_from_picture */
+}
 
 EAPI void
 ecore_x_region_free(Ecore_X_Region region)
@@ -160,26 +186,29 @@ ecore_x_region_free(Ecore_X_Region region)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesDestroyRegion(_ecore_x_disp, region);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_free */
+}
 
 EAPI void
-ecore_x_region_set(Ecore_X_Region region, Ecore_X_Rectangle *rects, int num)
+ecore_x_region_set(Ecore_X_Region region,
+                   Ecore_X_Rectangle *rects,
+                   int num)
 {
 #ifdef ECORE_XFIXES
    XRectangle *xrect = _ecore_x_rectangle_ecore_to_x(rects, num);
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesSetRegion(_ecore_x_disp, region, xrect, num);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_set */
+}
 
 EAPI void
-ecore_x_region_copy(Ecore_X_Region dest, Ecore_X_Region source)
+ecore_x_region_copy(Ecore_X_Region dest,
+                    Ecore_X_Region source)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesCopyRegion(_ecore_x_disp, dest, source);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_copy */
+}
 
 EAPI void
 ecore_x_region_combine(Ecore_X_Region dest,
@@ -190,7 +219,7 @@ ecore_x_region_combine(Ecore_X_Region dest,
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesUnionRegion(_ecore_x_disp, dest, source1, source2);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_combine */
+}
 
 EAPI void
 ecore_x_region_intersect(Ecore_X_Region dest,
@@ -201,7 +230,7 @@ ecore_x_region_intersect(Ecore_X_Region dest,
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesIntersectRegion(_ecore_x_disp, dest, source1, source2);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_intersect */
+}
 
 EAPI void
 ecore_x_region_subtract(Ecore_X_Region dest,
@@ -212,45 +241,50 @@ ecore_x_region_subtract(Ecore_X_Region dest,
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesSubtractRegion(_ecore_x_disp, dest, source1, source2);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_subtract */
+}
 
 EAPI void
-ecore_x_region_invert(Ecore_X_Region     dest,
+ecore_x_region_invert(Ecore_X_Region dest,
                       Ecore_X_Rectangle *bounds,
-                      Ecore_X_Region     source)
+                      Ecore_X_Region source)
 {
 #ifdef ECORE_XFIXES
    XRectangle *xbound;
    int num = 0;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
-   while (bounds + num) num++;
+   while (bounds + num)
+     num++;
    xbound = _ecore_x_rectangle_ecore_to_x(bounds, num);
 
    XFixesInvertRegion(_ecore_x_disp, dest, xbound, source);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_invert */
+}
 
 EAPI void
-ecore_x_region_translate(Ecore_X_Region region, int dx, int dy)
+ecore_x_region_translate(Ecore_X_Region region,
+                         int dx,
+                         int dy)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesTranslateRegion(_ecore_x_disp, region, dx, dy);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_translate */
+}
 
 EAPI void
-ecore_x_region_extents(Ecore_X_Region dest, Ecore_X_Region source)
+ecore_x_region_extents(Ecore_X_Region dest,
+                       Ecore_X_Region source)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesRegionExtents(_ecore_x_disp, dest, source);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_extents */
+}
 
 EAPI Ecore_X_Rectangle *
-ecore_x_region_fetch(Ecore_X_Region region, int *num,
+ecore_x_region_fetch(Ecore_X_Region region,
+                     int *num,
                      Ecore_X_Rectangle *bounds){
 #ifdef ECORE_XFIXES
    Ecore_X_Rectangle *rects;
@@ -267,40 +301,40 @@ ecore_x_region_fetch(Ecore_X_Region region, int *num,
 #else /* ifdef ECORE_XFIXES */
    return NULL;
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_fetch */
+}
 
 EAPI void
 ecore_x_region_expand(Ecore_X_Region dest,
                       Ecore_X_Region source,
-                      unsigned int   left,
-                      unsigned int   right,
-                      unsigned int   top,
-                      unsigned int   bottom)
+                      unsigned int left,
+                      unsigned int right,
+                      unsigned int top,
+                      unsigned int bottom)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesExpandRegion(_ecore_x_disp, dest, source, left, right, top, bottom);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_expand */
+}
 
 EAPI void
 ecore_x_region_gc_clip_set(Ecore_X_Region region,
-                           Ecore_X_GC     gc,
-                           int            x_origin,
-                           int            y_origin)
+                           Ecore_X_GC gc,
+                           int x_origin,
+                           int y_origin)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFixesSetGCClipRegion(_ecore_x_disp, gc, x_origin, y_origin, region);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_gc_clip_set */
+}
 
 EAPI void
-ecore_x_region_window_shape_set(Ecore_X_Region     region,
-                                Ecore_X_Window     win,
+ecore_x_region_window_shape_set(Ecore_X_Region region,
+                                Ecore_X_Window win,
                                 Ecore_X_Shape_Type type,
-                                int                x_offset,
-                                int                y_offset)
+                                int x_offset,
+                                int y_offset)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
@@ -311,13 +345,13 @@ ecore_x_region_window_shape_set(Ecore_X_Region     region,
                               y_offset,
                               region);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_window_shape_set */
+}
 
 EAPI void
-ecore_x_region_picture_clip_set(Ecore_X_Region  region,
+ecore_x_region_picture_clip_set(Ecore_X_Region region,
                                 Ecore_X_Picture picture,
-                                int             x_origin,
-                                int             y_origin)
+                                int x_origin,
+                                int y_origin)
 {
 #ifdef ECORE_XFIXES
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
@@ -327,5 +361,5 @@ ecore_x_region_picture_clip_set(Ecore_X_Region  region,
                               y_origin,
                               region);
 #endif /* ifdef ECORE_XFIXES */
-} /* ecore_x_region_picture_clip_set */
+}
 
