@@ -1605,6 +1605,33 @@ ecore_x_netwm_desktop_request_send(Ecore_X_Window win,
               SubstructureNotifyMask | SubstructureRedirectMask, &xev);
 }
 
+EAPI void
+ecore_x_netwm_moveresize_request_send(Ecore_X_Window win,
+                                      int x,
+                                      int y,
+                                      Ecore_X_Netwm_Direction direction,
+                                      unsigned int button)
+{
+   XEvent xev;
+
+   if (!win)
+     return;
+
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   xev.xclient.window = win;
+   xev.xclient.type = ClientMessage;
+   xev.xclient.message_type = ECORE_X_ATOM_NET_WM_MOVERESIZE;
+   xev.xclient.format = 32;
+   xev.xclient.data.l[0] = x;
+   xev.xclient.data.l[1] = y;
+   xev.xclient.data.l[2] = direction;
+   xev.xclient.data.l[3] = button;
+   xev.xclient.data.l[4] = 1;
+
+   XSendEvent(_ecore_x_disp, win, False,
+              SubstructureNotifyMask | SubstructureRedirectMask, &xev);
+}
+
 int
 _ecore_x_netwm_startup_info_begin(Ecore_X_Window win __UNUSED__,
                                   char *data __UNUSED__)
@@ -2020,19 +2047,14 @@ EAPI Eina_Bool
 ecore_x_screen_is_composited(int screen)
 {
    Ecore_X_Window win;
-   static Ecore_X_Atom atom = None;
+   Ecore_X_Atom atom;
    char buf[32];
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    snprintf(buf, sizeof(buf), "_NET_WM_CM_S%i", screen);
-   if (atom == None)
-     atom = XInternAtom(_ecore_x_disp, buf, False);
-
-   if (atom == None)
-     return EINA_FALSE;
-
+   atom = XInternAtom(_ecore_x_disp, buf, True);
+   if (atom == None) return EINA_FALSE;
    win = XGetSelectionOwner(_ecore_x_disp, atom);
-
    return (win != None) ? EINA_TRUE : EINA_FALSE;
 }
 
@@ -2040,17 +2062,13 @@ EAPI void
 ecore_x_screen_is_composited_set(int screen,
                                  Ecore_X_Window win)
 {
-   static Ecore_X_Atom atom = None;
+   Ecore_X_Atom atom;
    char buf[32];
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    snprintf(buf, sizeof(buf), "_NET_WM_CM_S%i", screen);
-   if (atom == None)
-     atom = XInternAtom(_ecore_x_disp, buf, False);
-
-   if (atom == None)
-     return;
-
+   atom = XInternAtom(_ecore_x_disp, buf, False);
+   if (atom == None) return;
    XSetSelectionOwner(_ecore_x_disp, atom, win, _ecore_x_event_last_time);
 }
 
