@@ -25,17 +25,20 @@
  * @ingroup Ecore_X_Pixmap_Group
  */
 EAPI Ecore_X_Pixmap
-ecore_x_pixmap_new(Ecore_X_Window win, int w, int h, int dep)
+ecore_x_pixmap_new(Ecore_X_Window win,
+                   int w,
+                   int h,
+                   int dep)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (win == 0)
-      win = DefaultRootWindow(_ecore_x_disp);
+     win = DefaultRootWindow(_ecore_x_disp);
 
    if (dep == 0)
-      dep = DefaultDepth(_ecore_x_disp, DefaultScreen(_ecore_x_disp));
+     dep = DefaultDepth(_ecore_x_disp, DefaultScreen(_ecore_x_disp));
 
    return XCreatePixmap(_ecore_x_disp, win, w, h, dep);
-} /* ecore_x_pixmap_new */
+}
 
 /**
  * Deletes the reference to the given pixmap.
@@ -51,7 +54,7 @@ ecore_x_pixmap_free(Ecore_X_Pixmap pmap)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFreePixmap(_ecore_x_disp, pmap);
-} /* ecore_x_pixmap_free */
+}
 
 /**
  * Pastes a rectangular area of the given pixmap onto the given drawable.
@@ -68,13 +71,19 @@ ecore_x_pixmap_free(Ecore_X_Pixmap pmap)
  * @ingroup Ecore_X_Pixmap_Group
  */
 EAPI void
-ecore_x_pixmap_paste(Ecore_X_Pixmap pmap, Ecore_X_Drawable dest,
-                     Ecore_X_GC gc, int sx, int sy,
-                     int w, int h, int dx, int dy)
+ecore_x_pixmap_paste(Ecore_X_Pixmap pmap,
+                     Ecore_X_Drawable dest,
+                     Ecore_X_GC gc,
+                     int sx,
+                     int sy,
+                     int w,
+                     int h,
+                     int dx,
+                     int dy)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XCopyArea(_ecore_x_disp, pmap, dest, gc, sx, sy, w, h, dx, dy);
-} /* ecore_x_pixmap_paste */
+}
 
 /**
  * Retrieves the size of the given pixmap.
@@ -86,12 +95,16 @@ ecore_x_pixmap_paste(Ecore_X_Pixmap pmap, Ecore_X_Drawable dest,
  * @ingroup Ecore_X_Pixmap_Group
  */
 EAPI void
-ecore_x_pixmap_geometry_get(Ecore_X_Pixmap pmap, int *x, int *y, int *w, int *h)
+ecore_x_pixmap_geometry_get(Ecore_X_Pixmap pmap,
+                            int *x,
+                            int *y,
+                            int *w,
+                            int *h)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (pmap)
-      ecore_x_drawable_geometry_get(pmap, x, y, w, h);
-} /* ecore_x_pixmap_geometry_get */
+     ecore_x_drawable_geometry_get(pmap, x, y, w, h);
+}
 
 /**
  * Retrieves the depth of the given pixmap.
@@ -104,5 +117,5 @@ ecore_x_pixmap_depth_get(Ecore_X_Pixmap pmap)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_drawable_depth_get(pmap);
-} /* ecore_x_pixmap_depth_get */
+}
 
