@@ -182,16 +182,6 @@ typedef struct _Ecore_X_DND_Target
    int          will_accept;
 } Ecore_X_DND_Target;
 
-extern int ECORE_X_MODIFIER_SHIFT;
-extern int ECORE_X_MODIFIER_CTRL;
-extern int ECORE_X_MODIFIER_ALT;
-extern int ECORE_X_MODIFIER_WIN;
-
-extern int ECORE_X_LOCK_SCROLL;
-extern int ECORE_X_LOCK_NUM;
-extern int ECORE_X_LOCK_CAPS;
-extern int ECORE_X_LOCK_SHIFT;
-
 extern Display *_ecore_x_disp;
 extern double _ecore_x_double_click_time;
 extern Time _ecore_x_event_last_time;
@@ -268,6 +258,9 @@ void _ecore_x_event_handle_fixes_selection_notify(XEvent *xevent);
 #ifdef ECORE_XDAMAGE
 void      _ecore_x_event_handle_damage_notify(XEvent *xevent);
 #endif /* ifdef ECORE_XDAMAGE */
+#ifdef ECORE_XKB
+void   _ecore_x_event_handle_xkb(XEvent *xevent);
+#endif  /* ifdef ECORE_XKB */
 void      _ecore_x_event_handle_generic_event(XEvent *xevent);
 
 void      _ecore_x_selection_data_init(void);
@@ -372,6 +365,7 @@ Ecore_Event_Mouse_Button *_ecore_mouse_button(int event,
                                               double mry);
 
 void _ecore_x_modifiers_get(void);
+KeySym _ecore_x_XKeycodeToKeysym(Display *display, KeyCode keycode, int index);
 
 //#define LOGFNS 1
 
